/*
 * sc5xx-tdm.c - Analog Devices SC5XX tdm interface driver
 *
 * Copyright (c) 2023 Analog Devices Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <linux/device.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/platform_device.h>
#include <sound/pcm.h>
#include <sound/pcm_params.h>
#include <sound/soc.h>
#include <sound/soc-dai.h>
#include <linux/soc/adi/sc59x.h>
#include <sound/sc5xx-dai.h>
#include <linux/soc/adi/hardware.h>

#include "sc5xx-sport.h"

static struct sport_params param;
static unsigned int gbSystemInit = 0u;
void __iomem *pcg0_ctla0_base, *reg_dai0_clk4, *reg_dai0_extd_clk4;

static int sc5xx_tdm_set_dai_fmt(struct snd_soc_dai *cpu_dai, unsigned int fmt)
{
	struct sport_device *sport = snd_soc_dai_get_drvdata(cpu_dai);
	struct device *dev = &sport->pdev->dev;
	int ret = 0;

	param.spctl &= ~(SPORT_CTL_OPMODE | SPORT_CTL_CKRE | SPORT_CTL_FSR
			| SPORT_CTL_LFS | SPORT_CTL_LAFS);
	switch (fmt & SND_SOC_DAIFMT_FORMAT_MASK) {
	case SND_SOC_DAIFMT_I2S:
		param.spctl |= SPORT_CTL_OPMODE | SPORT_CTL_CKRE
			| SPORT_CTL_LFS;
		break;
	case SND_SOC_DAIFMT_DSP_A:
		param.spctl |= SPORT_CTL_FSR;
		break;
	default:
		dev_err(dev, "%s: DAI format type is not supported\n",
				__func__);
		dev_err(dev, "%s: DAI format = %d\n", __func__, fmt);
		/*ret = -EINVAL;*/
		break;
	}

	param.spctl &= ~(SPORT_CTL_ICLK | SPORT_CTL_IFS);
	switch (fmt & SND_SOC_DAIFMT_MASTER_MASK) {
	case SND_SOC_DAIFMT_CBM_CFM:
	case SND_SOC_DAIFMT_CBS_CFS: /* Possible when PCG sources */
		break;

	case SND_SOC_DAIFMT_CBM_CFS:
	case SND_SOC_DAIFMT_CBS_CFM:
		ret = -EINVAL;
		break;
	default:
		dev_err(dev, "%s: Unknown DAI master type\n", __func__);
		ret = -EINVAL;
		break;
	}

	param.spmctl = SPORT_MCTL_MCE | SPORT_MCTL_MCPDE
			| (0x10 & SPORT_MCTL_MFD)
			| (0x700 & SPORT_MCTL_WSIZE);
	param.spcs0 = 0xff;

	return ret;
}

static int sc5xx_tdm_hw_params(struct snd_pcm_substream *substream,
				struct snd_pcm_hw_params *params,
				struct snd_soc_dai *dai)
{
	struct sport_device *sport = snd_soc_dai_get_drvdata(dai);
	struct device *dev = &sport->pdev->dev;
	int ret = 0;

	param.spctl &= ~SPORT_CTL_SLEN;
	switch (params_format(params)) {
	case SNDRV_PCM_FORMAT_S32_LE:
		param.spctl |= 0x1f0;
		sport->wdsize = 4;
		break;
	default:
		dev_err(dev, "%s: PCM format is not supported\n", __func__);
		return -EINVAL;
	}

	if (substream->stream == SNDRV_PCM_STREAM_PLAYBACK) {
		ret = sport_set_tx_params(sport, &param);
		if (ret) {
			dev_err(dev, "SPORT tx is busy!\n");
			return ret;
		}
	} else {
		ret = sport_set_rx_params(sport, &param);
		if (ret) {
			dev_err(dev, "SPORT rx is busy!\n");
			return ret;
		}
	}
	return 0;
}

static int sc5xx_tdm_set_channel_map(struct snd_soc_dai *dai,
		unsigned int tx_num, unsigned int *tx_slot,
		unsigned int rx_num, unsigned int *rx_slot)
{
	struct sport_device *sport = snd_soc_dai_get_drvdata(dai);
	int i;

	if ((tx_num > TDM_MAX_SLOTS) || (rx_num > TDM_MAX_SLOTS))
		return -EINVAL;

	for (i = 0; i < tx_num; i++)
		sport->tx_map[i] = tx_slot[i];
	for (i = 0; i < rx_num; i++)
		sport->rx_map[i] = rx_slot[i];

	return 0;
}

#ifdef CONFIG_PM
static int sc5xx_tdm_suspend(struct snd_soc_dai *dai)
{
	return 0;
}

static int sc5xx_tdm_resume(struct snd_soc_dai *dai)
{
	struct sport_device *sport = snd_soc_dai_get_drvdata(dai);
	struct device *dev = &sport->pdev->dev;
	int ret;

	ret = sport_set_tx_params(sport, &param);
	if (ret) {
		dev_err(dev, "SPORT tx is busy!\n");
		return ret;
	}
	ret = sport_set_rx_params(sport, &param);
	if (ret) {
		dev_err(dev, "SPORT rx is busy!\n");
		return ret;
	}

	return 0;
}
#else
#define sc5xx_tdm_suspend      NULL
#define sc5xx_tdm_resume       NULL
#endif

static const struct snd_soc_dai_ops sc5xx_tdm_dai_ops = {
	.hw_params       = sc5xx_tdm_hw_params,
	.set_fmt         = sc5xx_tdm_set_dai_fmt,
	.set_channel_map = sc5xx_tdm_set_channel_map,
};

static struct snd_soc_dai_driver sc5xx_tdm_dai = {
	.suspend = sc5xx_tdm_suspend,
	.resume = sc5xx_tdm_resume,
	.playback = {
		.channels_min = 2,
		.channels_max = 32,
		.rates = SNDRV_PCM_RATE_48000,
		.formats = SNDRV_PCM_FMTBIT_S32_LE,
	},
	.capture = {
		.channels_min = 2,
		.channels_max = 32,
		.rates = SNDRV_PCM_RATE_48000,
		.formats = SNDRV_PCM_FMTBIT_S32_LE,
	},
	.ops = &sc5xx_tdm_dai_ops,
};

static const struct snd_soc_component_driver sc5xx_tdm_component = {
	.name	= "sc5xx-tdm",
};

#ifdef CONFIG_OF
static const struct of_device_id sc5xx_audio_of_match[] = {
	{
		.compatible = "sc5xx,tdm-dai",
	},
	{},
};
MODULE_DEVICE_TABLE(of, sc5xx_audio_of_match);
#endif


/* PCG initialization for A2B */
static void pcg_init_a2b(void)
{
        volatile unsigned int nCtla0Val ;
        volatile unsigned int nCtla1Val ;
        unsigned int nTDMSize = 8u;

        /* TDM 2 */
        if(nTDMSize == 2u)
        {
#ifdef A2B_I2S_WAVE
            nCtla1Val = 8U|(uint32_t)(4U<<20U);
#else
            nCtla1Val = 8U;
#endif
        }
        else if(nTDMSize == 4u)
        {
#ifdef A2B_I2S_WAVE
            nCtla1Val = 4U |(uint32_t)(2U<<20U);
#else
            nCtla1Val = 4U;
#endif
        }
        else if(nTDMSize == 8u)
        {
#ifdef A2B_I2S_WAVE
            nCtla1Val = 2u |(uint32_t)(1U<<20U);
#else
            nCtla1Val = 2u;
#endif
        }
        /* TDM 16 */
        else  /* if(nTDMSize == 16u) Supported only till TDM16 */
        {
            nCtla1Val = (uint32_t)1u;
        }
        /* Enable FS */
        nCtla0Val = 512 |0xC0000000;
        nCtla1Val|= (uint32_t)(1U<<30U)|(uint32_t)(1U<<31U);

        pcg0_ctla0_base = ioremap(REG_PCG0_CTLA0,4);
        writel(nCtla0Val, pcg0_ctla0_base + 0);//REG_PCG0_CTLA0
        writel(nCtla1Val, pcg0_ctla0_base + 4);//REG_PCG0_CTLA1
}

/* Global PCG Init */
void pcg_init(void)
{
        if(gbSystemInit)
        {
            return;
        }
        gbSystemInit = 1u;
       
        reg_dai0_clk4 = ioremap(REG_DAI0_CLK4,4);	
	reg_dai0_extd_clk4 = ioremap(REG_DAI0_EXTD_CLK4,4);
	writel(0x3def7bc2, reg_dai0_clk4);
	writel(0x00000002,reg_dai0_extd_clk4);
	pcg_init_a2b();
}

static int sc5xx_tdm_probe(struct platform_device *pdev)
{
	struct sport_device *sport;
	struct device *dev = &pdev->dev;
	struct clk *clk;
	int ret;

        clk = devm_clk_get(dev, "sclk");
        if (IS_ERR(clk)) {
                dev_err(dev, "Missing clock node `sclk` for i2s\n");
                return PTR_ERR(clk);
        }

        pcg_init();

	sport = sport_create(pdev);
	if (!sport)
		return -ENODEV;
	
	sport->clk = clk;
	clk_prepare_enable(clk);

	/* register with the ASoC layers */
	ret = snd_soc_register_component(dev, &sc5xx_tdm_component,
					 &sc5xx_tdm_dai, 1);
	if (ret) {
		dev_err(dev, "Failed to register DAI: %d\n", ret);
		sport_delete(sport);
		return ret;
	}
	platform_set_drvdata(pdev, sport);

	return 0;
}

static int sc5xx_tdm_remove(struct platform_device *pdev)
{
	struct sport_device *sport = platform_get_drvdata(pdev);

        snd_soc_unregister_component(&pdev->dev);
        sport_delete(sport);
        iounmap(pcg0_ctla0_base);
        iounmap(reg_dai0_clk4);
        iounmap(reg_dai0_extd_clk4);
        return 0;
}

static struct platform_driver sc5xx_tdm_driver = {
	.probe  = sc5xx_tdm_probe,
	.remove = sc5xx_tdm_remove,
	.driver = {
		.name   = "sc5xx-tdm",
		.owner  = THIS_MODULE,
		.of_match_table = of_match_ptr(sc5xx_audio_of_match),
	},
};

module_platform_driver(sc5xx_tdm_driver);

MODULE_DESCRIPTION("Analog Devices SC5XX tdm interface driver");
MODULE_AUTHOR("ADI Automotive Software Team, Bangalore");
MODULE_LICENSE("GPL v2");

