/*=============================================================================
 *
 * Project: a2bstack
 *
 * Copyright (c) 2023 - Analog Devices Inc. All Rights Reserved.
 * This software is subject to the terms and conditions of the license set 
 * forth in the project LICENSE file. Downloading, reproducing, distributing or 
 * otherwise using the software constitutes acceptance of the license. The 
 * software may not be used except as expressly authorized under the license.
 *
 *=============================================================================
 *
 * \file:   pal_audio.c
 * \author: Mentor Graphics, Embedded Software Division
 * \brief:  Implementation of the platform abstraction layer for audio services.
 *
 *=============================================================================
 */

/*======================= I N C L U D E S =========================*/

#include "a2b/error.h"
#include "pal_audio.h"
#include "adi_a2b_audiorouting.h"
#include <alsa/asoundlib.h>

/*======================= D E F I N E S ===========================*/

#define A2B_PAL_AUDIO_DEBUG				(0)
#define A2B_PCM_WAIT					(15)
#define A2B_MAX_APPBUFFER				(8u)	/*! Maximum number application sub buffers */
#define A2B_USE_CODEC					(1u)	/*! Use on board codec for audio */
#define SPORT_BUFFER_SIZE				(1024u)	/*! Number of SPORT i/o samples per channel */
#define A2B_NUM_CH					(8U)   /*!< Max num Input Channels */
#define CODEC_NUM_CH					(2U)   /*!< Max num Output Channels */
#define A2B_BUFSIZE 					(SPORT_BUFFER_SIZE * 16)
#define CODEC_BUFSIZE 					(SPORT_BUFFER_SIZE * 2)

#if A2B_PAL_AUDIO_DEBUG
#define PAL_AUDIO_LOG(...)				do{\
										printf(__VA_ARGS__);\
                        				}while(0)

#define PAL_AUDIO_LOGF(...)				do{\
										fprintf(__VA_ARGS__);\
                        				}while(0)
#else
#define PAL_AUDIO_LOG(...)
#define PAL_AUDIO_LOGF(...)
#endif

/*======================= D A T A T Y P E S =======================*/

typedef struct
{
	ADI_A2B_SCOMM_HANDLER oAudioCommHandler;

	snd_pcm_t *adc_capture_handle;
	snd_pcm_t *dac_playback_handle;
	snd_pcm_t *a2b_capture_handle;
	snd_pcm_t *a2b_playback_handle;

} adi_a2b_audio;

/*======================= L O C A L  P R O T O T Y P E S  =========*/

static a2b_Int32 configure_alsa_audio(snd_pcm_t *device, uint8 channels);
static void get_configure_alsa_audio_config(snd_pcm_t *device, snd_pcm_hw_params_t *hw_params);
static void adi_a2b_DACTx(a2b_UInt32* pTxBufferPtr, a2b_UInt32 nChannel);
static void adi_a2b_ADCRx(a2b_UInt32* pRxBufferPtr, a2b_UInt32 nChannel);
static void adi_a2b_DownStreamTx(a2b_UInt32* pTxBufferPtr, a2b_UInt32 nChannel);
static void adi_a2b_UpstreamRx(a2b_UInt32 pRxBufferPtr[], a2b_UInt32 nChannel);
static void adi_a2b_upstreamToDwnRouting(ADI_A2B_SCOMM_HANDLER* pAudioRoutingHandle, a2b_UInt32 anDwnstreamBuffer[], a2b_UInt32 anUpstreamBuffer[][SPORT_BUFFER_SIZE], a2b_UInt32 nChannel);
#if A2B_USE_CODEC
static void adi_a2b_ADCToDwnRouting(ADI_A2B_SCOMM_HANDLER* pAudioRoutingHandle, a2b_UInt32 anDwnstreamBuffer[], a2b_UInt32 anADCBuffer[][SPORT_BUFFER_SIZE], a2b_UInt32 nChannel);
static void adi_a2b_UpstreamToDACRouting(ADI_A2B_SCOMM_HANDLER* pAudioRoutingHandle, a2b_UInt32 anDACBuffer[], a2b_UInt32 anUpstreamBuffer[][SPORT_BUFFER_SIZE], a2b_UInt32 nChannel);
static void adi_a2b_ADCToDACRouting(ADI_A2B_SCOMM_HANDLER* pAudioRoutingHandle, a2b_UInt32 anDACBuffer[], a2b_UInt32 anADCBuffer[][SPORT_BUFFER_SIZE], a2b_UInt32 nChannel);
#endif
static void adi_a2b_DeInterleaveCopy(a2b_UInt32 anDesbuffer[][SPORT_BUFFER_SIZE], a2b_UInt32 anSrcBuffer[], a2b_UInt32 nChannel);

/*======================= D A T A  ================================*/

uint8 nRxSportCnt = 0u, nTxSportCnt = 0u, nADCTxCnt = 0u, nADCRxCnt = 0u;
a2b_Int32 err;
static a2b_Int32 avail_min = -1;
size_t n;
adi_a2b_audio goA2bAudio;

/*! Upstream data processing buffer   */
static a2b_UInt32 ganUpstreamProcessBuffer[A2B_MAX_APPBUFFER][A2B_NUM_CH][SPORT_BUFFER_SIZE];
/*! ADC data processing buffer   */
static a2b_UInt32 ganADCProcessBuffer[A2B_MAX_APPBUFFER][CODEC_NUM_CH][SPORT_BUFFER_SIZE];
/*! Codec buffers */
a2b_Int32 codec_buf[CODEC_BUFSIZE * 4];
/*! a2b buffers */
a2b_Int32 a2b_buf[A2B_BUFSIZE * 4];

char *snd_codec_device_in = "hw:0,0"; /* CODEC: hw:0,0	*/
char *snd_codec_device_out = "hw:0,0"; /* CODEC: hw:0,0	*/

char *snd_a2b_device_in = "hw:0,1"; /* A2B: hw:0,1	*/
char *snd_a2b_device_out = "hw:0,1"; /* A2B: hw:0,1	*/

static  a2b_UInt32 rate = 48000;
static  a2b_UInt32 format = SND_PCM_FORMAT_S32_LE;

static a2b_Int32 a2b_openStream(snd_pcm_t **handle, const char *name, a2b_Int32 bufsize, a2b_Int32 dir, a2b_UInt8 nNumCh);

/*======================= C O D E =================================*/

/*!****************************************************************************
 *
 *  \b              pal_audioInit
 *
 *  <b> API Details: </b><br>
 *  This routine is called to do initialization the audio subsystem
 *  during the stack allocation process.
 *                                                                       <br><br>
 *  <b> Linux Implementation Details: </b><br>
 *  Nothing to do, always A2B_RESULT_SUCCESS
 *                                                                           <br>
 *  \param          [in]    ecb      The environment control block (ecb) for
 *                                   this platform.
 *  \pre            None
 *
 *  \post           None
 *
 *  \return         A status code that can be checked with the A2B_SUCCEEDED()
 *                  or A2B_FAILED() for success or failure.
 *
 ******************************************************************************/
a2b_HResult pal_audioInit(
A2B_ECB* ecb)
{

	A2B_UNUSED(ecb);
	return A2B_RESULT_SUCCESS;
} /* pal_audioInit */

/*!****************************************************************************
 *
 *  \b              pal_audioOpen
 *
 *  <b> API Details: </b><br>
 *  This routine is called to do post-initialization the audio subsystem
 *  during the stack allocation process.  This routine is called immediately
 *  after the pal_audioInit (assuming the init was successful).
 *                                                                       <br><br>
 *  <b> Linux Implementation Details: </b><br>
 *  Always returns a non-NULL value to let the stack allocation pass.
 *                                                                           <br>
 *  \pre            None
 *
 *  \post           None
 *
 *  \return         If non-zero the open was considered sucessfully.  If A2B_NULL
 *                  audio initialization must have failed and the stack
 *                  allocation will fail.  This returned handle will be passed
 *                  back into the other PAL audio (pal_audioXXX) API calls.
 *
 ******************************************************************************/
a2b_Handle pal_audioOpen(void)
{
	a2b_Int32 err;
	a2b_Int32 nCodecCh;
	a2b_Int32 nA2BCh;
	ADI_A2B_SCOMM_HANDLER *pAudioCommHandler;

	pAudioCommHandler = &goA2bAudio.oAudioCommHandler;

	/* Initialization */
	pAudioCommHandler->pRoutingTable = gaAudioRoutingtab;
	pAudioCommHandler->nWordLength = 32u;
	pAudioCommHandler->nUpstrProcWriteIndex = 0u;
	pAudioCommHandler->nADCProcWriteIndex = 0u;
	pAudioCommHandler->nUpstrToDwnstrReadIndex = 1u;
	pAudioCommHandler->nADCToDwnstrReadIndex = 2u;
	pAudioCommHandler->nADCtoDACReadIndex = 1u;
	pAudioCommHandler->nUpStrtoDACReadIndex = 2u;
	pAudioCommHandler->nAD2410TDMSize = A2B_NUM_CH;
	pAudioCommHandler->nCodecTDMSize = CODEC_NUM_CH;

	PAL_AUDIO_LOG("\nALSA: Opening the required sound devices\n");

	nCodecCh = goA2bAudio.oAudioCommHandler.nCodecTDMSize;
	nA2BCh = goA2bAudio.oAudioCommHandler.nAD2410TDMSize;

	if ((err = a2b_openStream(&goA2bAudio.dac_playback_handle, snd_codec_device_out, SPORT_BUFFER_SIZE, SND_PCM_STREAM_PLAYBACK, nCodecCh)) < 0)
		return err;

	PAL_AUDIO_LOG("\nALSA: Opening codec device out  %s \n", snd_codec_device_out);

	if ((err = a2b_openStream(&goA2bAudio.adc_capture_handle, snd_codec_device_in, SPORT_BUFFER_SIZE, SND_PCM_STREAM_CAPTURE, nCodecCh)) < 0)
		return err;

	PAL_AUDIO_LOG("\nALSA: Opening codec device in  %s \n", snd_codec_device_in);

	if ((err = a2b_openStream(&goA2bAudio.a2b_playback_handle, snd_a2b_device_out, SPORT_BUFFER_SIZE, SND_PCM_STREAM_PLAYBACK, nA2BCh)) < 0)
		return err;

	PAL_AUDIO_LOG("\nALSA: Opening a2b device out \n", snd_a2b_device_out);

	if ((err = a2b_openStream(&goA2bAudio.a2b_capture_handle, snd_a2b_device_in, SPORT_BUFFER_SIZE, SND_PCM_STREAM_CAPTURE, nA2BCh)) < 0)
		return err;

	PAL_AUDIO_LOG("\nALSA: Opening A2B capture device %s \n",snd_a2b_device_in);

	return (a2b_Handle)1;

} /* pal_audioOpen */

/*!****************************************************************************
 *
 *  \b              pal_audioClose
 *
 *  <b> API Details: </b><br>
 *  This routine is called to de-initialization the audio subsystem
 *  during the stack destroy process.
 *                                                                       <br><br>
 *  <b> Linux Implementation Details: </b><br>
 *  Always returns A2B_RESULT_SUCCESS.
 *                                                                           <br>
 *  \param          [in]    hnd      The handle returned from pal_audioOpen
 *
 *  \pre            None
 *
 *  \post           None
 *
 *  \return         A status code that can be checked with the A2B_SUCCEEDED()
 *                  or A2B_FAILED() for success or failure.
 *
 ******************************************************************************/
a2b_HResult pal_audioClose(a2b_Handle hnd)
{
	A2B_UNUSED(hnd);

	PAL_AUDIO_LOG("\nALSA: Closing a2b capture devices \n");

	snd_pcm_hw_free(goA2bAudio.a2b_capture_handle);
	snd_pcm_close(goA2bAudio.a2b_capture_handle);

	PAL_AUDIO_LOG("\nALSA: Closing a2b Playback \n");

	snd_pcm_hw_free(goA2bAudio.a2b_playback_handle);
	snd_pcm_close(goA2bAudio.a2b_playback_handle);

	PAL_AUDIO_LOG("\nALSA: Closing adc Capture \n");
	snd_pcm_hw_free(goA2bAudio.adc_capture_handle);
	snd_pcm_close(goA2bAudio.adc_capture_handle);

	PAL_AUDIO_LOG("\nALSA: Closing dac Capture \n");
	snd_pcm_hw_free(goA2bAudio.dac_playback_handle);
	snd_pcm_close(goA2bAudio.dac_playback_handle);

	PAL_AUDIO_LOG("\nALSA: Completed \n");

	return A2B_RESULT_SUCCESS;
} /* pal_audioClose */

/*!****************************************************************************
 *
 *  \b              pal_audioConfig
 *
 *  <b> API Details: </b><br>
 *  This routine is called to configure the audio subsystem master node
 *  during the discovery process.  This routine is called during the
 *  "NetComplete" process after all nodes are discovered and before the master
 *  node "NodeComplete" process which fully initializes the master A2B
 *  registers and starts the up/downstream flow.
 *                                                                       <br><br>
 *  The application will typically override this routine so it will get the TDM
 *  settings of the complete A2B network and can properly setup the audio
 *  that is transmitted into the master node to "feed" the network.
 *                                                                       <br><br>
 *  <b> Linux Implementation Details: </b><br>
 *  Always returns A2B_RESULT_SUCCESS.
 *                                                                           <br>
 *  \param          [in]    hnd          The handle returned from pal_audioOpen
 *
 *  \param          [in]    tdmSettings  TDM settings for all nodes in the
 *                                       A2B network.  This information is used
 *                                       to properly configure the audio that
 *                                       is transmitted into the master node.
 *
 *  \pre            None
 *
 *  \post           None
 *
 *  \return         A status code that can be checked with the A2B_SUCCEEDED()
 *                  or A2B_FAILED() for success or failure.
 *
 ******************************************************************************/
a2b_HResult pal_audioConfig(a2b_Handle hnd, a2b_TdmSettings* tdmSettings)
{
	A2B_UNUSED(hnd);
	A2B_UNUSED(tdmSettings);
	PAL_AUDIO_LOG("\nALSA:Config\n");


	A2B_UNUSED(tdmSettings);
	return A2B_RESULT_SUCCESS;
} /* pal_audioConfig */

/*!****************************************************************************
 *
 *  \b              pal_audioShutdown
 *
 *  <b> API Details: </b><br>
 *  This routine is called to shutdown the audio subsystem
 *  during the stack destroy process.  This routine is called immediately
 *  after the pal_audioClose (assuming the close was successful).
 *                                                                       <br><br>
 *  <b> Linux Implementation Details: </b><br>
 *  Always returns A2B_RESULT_SUCCESS.
 *                                                                           <br>
 *  \param          [in]    ecb      The environment control block (ecb) for
 *                                   this platform.
 *
 *  \pre            None
 *
 *  \post           None
 *
 *  \return         A status code that can be checked with the A2B_SUCCEEDED()
 *                  or A2B_FAILED() for success or failure.
 *
 ******************************************************************************/
a2b_HResult pal_audioShutdown(A2B_ECB* ecb)
{
	A2B_UNUSED(ecb);

	PAL_AUDIO_LOG("\nALSA:Shut down\n");

	return A2B_RESULT_SUCCESS;
} /* pal_audioShutdown */


/****************************************************************************/
/*!
 @brief			Periodic tick function for audio processing

 @return         void

 Note:
 */
/********************************************************************************/
a2b_HResult a2b_audioStart()
{

	adi_a2b_audio* a2bAudioHnd = (adi_a2b_audio*)&goA2bAudio;
	if ((err = snd_pcm_prepare(a2bAudioHnd->dac_playback_handle)) < 0)
	{
		PAL_AUDIO_LOGF(stderr, "cannot prepare audio interface for use(%s)\n", snd_strerror(err));
		return err;
	}

	if ((err = snd_pcm_start(a2bAudioHnd->adc_capture_handle)) < 0)
	{
		PAL_AUDIO_LOGF(stderr, "cannot prepare audio interface for use(%s)\n", snd_strerror(err));
		return err;
	}

	if ((err = snd_pcm_prepare(a2bAudioHnd->a2b_playback_handle)) < 0)
	{
		PAL_AUDIO_LOGF(stderr, "cannot prepare audio interface for use(%s)\n", snd_strerror(err));
		return err;
	}

	if ((err = snd_pcm_start(a2bAudioHnd->a2b_capture_handle)) < 0)
	{
		PAL_AUDIO_LOGF(stderr, "cannot prepare audio interface for use(%s)\n", snd_strerror(err));
		return err;
	}
	return A2B_RESULT_SUCCESS;

}

/****************************************************************************/
/*!
 @brief			Periodic tick function for audio processing

 @return         void

 Note:
 */
/********************************************************************************/
void a2b_audioTick()
{
	/* Process A2B Audio */
	adi_a2b_UpstreamRx(&a2b_buf, goA2bAudio.oAudioCommHandler.nAD2410TDMSize);
	adi_a2b_ADCRx(&codec_buf, goA2bAudio.oAudioCommHandler.nCodecTDMSize);

	adi_a2b_DownStreamTx(&a2b_buf, goA2bAudio.oAudioCommHandler.nAD2410TDMSize);
	adi_a2b_DACTx(&codec_buf, goA2bAudio.oAudioCommHandler.nCodecTDMSize);

}

/****************************************************************************/
/*!
 @brief			This function opens the stream

 @param [in]     handle    Transmission sub buffer pointer.
 @param [in]     name      Name of the device.
 @param [in]     dir   	   Capture or Play back
 @param [in]     nNumCh    Number of channels.

 @return         void

 Note:
 */
/********************************************************************************/
static a2b_Int32 a2b_openStream(snd_pcm_t **handle, const char *name, a2b_Int32 bufsize, a2b_Int32 dir, a2b_UInt8 nNumCh)
{
	snd_pcm_hw_params_t *hw_params;
	snd_pcm_sw_params_t *sw_params;
	const char *dirname = (dir == SND_PCM_STREAM_PLAYBACK) ? "PLAYBACK" : "CAPTURE";
	a2b_Int32 err;

	if ((err = snd_pcm_open(handle, name, dir, 0)) < 0)
	{
		PAL_AUDIO_LOGF(stderr, "%s (%s): cannot open audio device (%s)\n", name, dirname, snd_strerror(err));
		return err;
	}

	if ((err = snd_pcm_hw_params_malloc(&hw_params)) < 0)
	{
		PAL_AUDIO_LOGF(stderr, "%s (%s): cannot allocate hardware parameter structure(%s)\n", name, dirname, snd_strerror(err));
		return err;
	}

	if ((err = snd_pcm_hw_params_any(*handle, hw_params)) < 0)
	{
		PAL_AUDIO_LOGF(stderr, "%s (%s): cannot initialize hardware parameter structure(%s)\n", name, dirname, snd_strerror(err));
		return err;
	}

	if ((err = snd_pcm_hw_params_set_access(*handle, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED)) < 0)
	{
		PAL_AUDIO_LOGF(stderr, "%s (%s): cannot set access type(%s)\n", name, dirname, snd_strerror(err));
		return err;
	}

	if ((err = snd_pcm_hw_params_set_format(*handle, hw_params, format)) < 0)
	{
		PAL_AUDIO_LOGF(stderr, "%s (%s): cannot set sample format(%s)\n", name, dirname, snd_strerror(err));
		return err;
	}

	if ((err = snd_pcm_hw_params_set_rate_near(*handle, hw_params, &rate, NULL)) < 0)
	{
		PAL_AUDIO_LOGF(stderr, "%s (%s): cannot set sample rate(%s)\n", name, dirname, snd_strerror(err));
		return err;
	}

	if ((err = snd_pcm_hw_params_set_channels(*handle, hw_params, nNumCh)) < 0)
	{
		PAL_AUDIO_LOGF(stderr, "%s (%s): cannot set channel count(%s)\n", name, dirname, snd_strerror(err));
		return err;
	}

	if ((err = snd_pcm_hw_params(*handle, hw_params)) < 0)
	{
		PAL_AUDIO_LOGF(stderr, "%s (%s): cannot set parameters(%s)\n", name, dirname, snd_strerror(err));
		return err;
	}

	snd_pcm_hw_params_free(hw_params);

	if ((err = snd_pcm_sw_params_malloc(&sw_params)) < 0)
	{
		PAL_AUDIO_LOGF(stderr, "%s (%s): cannot allocate software parameters structure(%s)\n", name, dirname, snd_strerror(err));
		return err;
	}
	if ((err = snd_pcm_sw_params_current(*handle, sw_params)) < 0)
	{
		PAL_AUDIO_LOGF(stderr, "%s (%s): cannot initialize software parameters structure(%s)\n", name, dirname, snd_strerror(err));
		return err;
	}
	if ((err = snd_pcm_sw_params_set_avail_min(*handle, sw_params, bufsize)) < 0)
	{
		PAL_AUDIO_LOGF(stderr, "%s (%s): cannot set minimum available count(%s)\n", name, dirname, snd_strerror(err));
		return err;
	}
	if ((err = snd_pcm_sw_params_set_start_threshold(*handle, sw_params, 0U)) < 0)
	{
		PAL_AUDIO_LOGF(stderr, "%s (%s): cannot set start mode(%s)\n", name, dirname, snd_strerror(err));
		return err;
	}
	if ((err = snd_pcm_sw_params(*handle, sw_params)) < 0)
	{
		PAL_AUDIO_LOGF(stderr, "%s (%s): cannot set software parameters(%s)\n", name, dirname, snd_strerror(err));
		return err;
	}

	return 0;
}

/****************************************************************************/
/*!
 @brief			Callback function for upstream data reception

 @param [in]     pTxBufferPtr    Transmission sub buffer pointer.
 @param [in]     nChannel     	TDM size.

 @return         void

 Note:
 */
/********************************************************************************/
static void adi_a2b_UpstreamRx(a2b_UInt32 pRxBufferPtr[], a2b_UInt32 nChannel)
{

	snd_pcm_t *capture_handle = goA2bAudio.a2b_capture_handle;
	ADI_A2B_SCOMM_HANDLER* pAudioRoutingHandle = (ADI_A2B_SCOMM_HANDLER*)&goA2bAudio.oAudioCommHandler;
	a2b_Int32 availSrc;
	availSrc = snd_pcm_avail_update(capture_handle);
	if (availSrc > 0)
	{
		snd_pcm_readi(capture_handle, pRxBufferPtr, availSrc);

		adi_a2b_DeInterleaveCopy(ganUpstreamProcessBuffer[pAudioRoutingHandle->nUpstrProcWriteIndex], pRxBufferPtr, nChannel);
		pAudioRoutingHandle->nUpstrProcWriteIndex = (pAudioRoutingHandle->nUpstrProcWriteIndex + 1u) % A2B_MAX_APPBUFFER;
	}
}

/****************************************************************************/
/*!
 @brief			Callback function

 @param [in]     pTxBufferPtr    Transmission sub buffer pointer.
 @param [in]     nChannel     	TDM size.

 @return         void

 Note:
 */
/********************************************************************************/
static void adi_a2b_DownStreamTx(a2b_UInt32* pTxBufferPtr, a2b_UInt32 nChannel)
{
	snd_pcm_t *playback_handle = goA2bAudio.a2b_playback_handle;
	ADI_A2B_SCOMM_HANDLER* pAudioRoutingHandle = (ADI_A2B_SCOMM_HANDLER*)&goA2bAudio.oAudioCommHandler;
	a2b_Int32 availDst;

	if ((err = snd_pcm_wait(playback_handle, A2B_PCM_WAIT)) < 0)
	{
		PAL_AUDIO_LOGF(stderr, "poll failed(%s)\n", strerror(errno));
	}

	availDst = snd_pcm_avail_update(playback_handle);

	if (availDst > 0)
	{
		nADCTxCnt++;

		adi_a2b_upstreamToDwnRouting(pAudioRoutingHandle, pTxBufferPtr, ganUpstreamProcessBuffer[pAudioRoutingHandle->nUpstrToDwnstrReadIndex], nChannel);
#if A2B_USE_CODEC
		adi_a2b_ADCToDwnRouting(pAudioRoutingHandle, pTxBufferPtr, ganADCProcessBuffer[pAudioRoutingHandle->nADCToDwnstrReadIndex], nChannel);
#endif

		snd_pcm_writei(playback_handle, pTxBufferPtr, availDst);

		pAudioRoutingHandle->nUpstrToDwnstrReadIndex = (pAudioRoutingHandle->nUpstrToDwnstrReadIndex + 1u) % A2B_MAX_APPBUFFER;
		pAudioRoutingHandle->nADCToDwnstrReadIndex = (pAudioRoutingHandle->nADCToDwnstrReadIndex + 1u) % A2B_MAX_APPBUFFER;
	}

}

/****************************************************************************/
/*!
 @brief			Callback function ADC data recpetionC

 @param [in]     pRxBufferPtr    Transmission sub buffer pointer.
 @param [in]     nChannel     	TDM size.

 @return         void

 Note:
 */
/********************************************************************************/
static void adi_a2b_ADCRx(a2b_UInt32* pRxBufferPtr, a2b_UInt32 nChannel)
{
	snd_pcm_t *capture_handle = goA2bAudio.adc_capture_handle;
	ADI_A2B_SCOMM_HANDLER* pAudioRoutingHandle = (ADI_A2B_SCOMM_HANDLER*)&goA2bAudio.oAudioCommHandler;
	a2b_Int32 availSrc;

	availSrc = snd_pcm_avail_update(capture_handle);
	if (availSrc > 0)
	{
		snd_pcm_readi(capture_handle, pRxBufferPtr, availSrc);

		/* Copy SPORT sub buffer data to application buffer */
		adi_a2b_DeInterleaveCopy(ganADCProcessBuffer[pAudioRoutingHandle->nADCProcWriteIndex], pRxBufferPtr, nChannel);
		pAudioRoutingHandle->nADCProcWriteIndex = (pAudioRoutingHandle->nADCProcWriteIndex + 1u) % A2B_MAX_APPBUFFER;
	}
}

/****************************************************************************/
/*!
 @brief			Callback function for transmission completion to DAC

 @param [in]     pTxBufferPtr    Transmission sub buffer pointer.
 @param [in]     nChannel     	TDM size.

 @return         void

 Note:
 */
/********************************************************************************/
static void adi_a2b_DACTx(a2b_UInt32* pTxBufferPtr, a2b_UInt32 nChannel)
{
	snd_pcm_t *playback_handle = goA2bAudio.dac_playback_handle;
	ADI_A2B_SCOMM_HANDLER* pAudioRoutingHandle = (ADI_A2B_SCOMM_HANDLER*)&goA2bAudio.oAudioCommHandler;
	a2b_Int32 availDst;

	if ((err = snd_pcm_wait(playback_handle, A2B_PCM_WAIT)) < 0)
	{
		PAL_AUDIO_LOGF(stderr, "poll failed(%s)\n", strerror(errno));
	}

	availDst = snd_pcm_avail_update(playback_handle);
	nADCTxCnt++;
	if (availDst > 0)
	{
#if A2B_USE_CODEC
		/* Upstream to DAC routing */
		adi_a2b_UpstreamToDACRouting(pAudioRoutingHandle, pTxBufferPtr, ganUpstreamProcessBuffer[pAudioRoutingHandle->nUpStrtoDACReadIndex], nChannel);
		/* ADC data to DAC routing */
		adi_a2b_ADCToDACRouting(pAudioRoutingHandle, pTxBufferPtr, ganADCProcessBuffer[pAudioRoutingHandle->nADCtoDACReadIndex], nChannel);
#endif

		snd_pcm_writei(playback_handle, pTxBufferPtr, availDst);

		/* Index update */
		pAudioRoutingHandle->nADCtoDACReadIndex = (pAudioRoutingHandle->nADCtoDACReadIndex + 1u) % A2B_MAX_APPBUFFER;
		pAudioRoutingHandle->nUpStrtoDACReadIndex = (pAudioRoutingHandle->nUpStrtoDACReadIndex + 1u) % A2B_MAX_APPBUFFER;
	}

}

/***********************************************************************************/
/*!
 @brief          This function copies upstream data to the downstream in accordance with routing table

 @param [in]     pHandle       			Pointer to serial port structure.
 @param [in]     anDwnstreamBuffer     	Pointer to downstream buffer/Destination.
 @param [in]     anUpstreamBuffer     	Source buffer address/Upstream  .
 @param [in]     nChannel         		TDM size

 @return         Return code
 - 0: Success
 - 1: Failure
 */
/***********************************************************************************/
static void adi_a2b_upstreamToDwnRouting(ADI_A2B_SCOMM_HANDLER* pAudioRoutingHandle, a2b_UInt32 anDwnstreamBuffer[], a2b_UInt32 anUpstreamBuffer[][SPORT_BUFFER_SIZE], a2b_UInt32 nChannel)
{
	a2b_UInt16 nIndex1, nIndex2, nIndex3;
	a2b_UInt8 nRoutingIndex;

	/* Outer most loop over TX channel */
	for (nIndex1 = 0u; nIndex1 < nChannel; nIndex1++)
	{
		/* Get Routing Index  */
		nRoutingIndex = pAudioRoutingHandle->pRoutingTable[nIndex1 + A2B_MAX_CODEC_CHANNELS];

		if (nRoutingIndex <= (a2b_UInt8)A2B_UPSTREAM_CH31)
		{
			nRoutingIndex -= (a2b_UInt8)A2B_UPSTREAM_CH00;
			nIndex3 = 0u;
			/* Transfer the data from Rx to Tx */
			for (nIndex2 = 0u; nIndex2 < (a2b_UInt16)SPORT_BUFFER_SIZE * nChannel; nIndex2 += (a2b_UInt16)nChannel)
			{
				anDwnstreamBuffer[nIndex1 + nIndex2] = anUpstreamBuffer[nRoutingIndex][nIndex3++];
			}
		}

	}
}

/***********************************************************************************/
/*!
 @brief          This function copies ADC data to downstream

 @param [in]     pAudioRoutingHandle       			Pointer to serial port structure.
 @param [in]     anDwnstreamBuffer     	Pointer to downstream buffer/Destination.
 @param [in]     anUpstreamBuffer     	Source buffer address/Upstream  .
 @param [in]     nChannel         		TDM size

 @return         Return code
 - 0: Success
 - 1: Failure
 */
/***********************************************************************************/
static void adi_a2b_ADCToDwnRouting(ADI_A2B_SCOMM_HANDLER* pAudioRoutingHandle, a2b_UInt32 anDwnstreamBuffer[], a2b_UInt32 anADCBuffer[][SPORT_BUFFER_SIZE], a2b_UInt32 nChannel)
{
	a2b_UInt16 nIndex1, nIndex2, nIndex3;
	a2b_UInt8 nRoutingIndex;

	/* Outer most loop- Rx to Tx copy */
	for (nIndex1 = 0u; nIndex1 < nChannel; nIndex1++)
	{
		nIndex3 = 0u;
		/* Get routing table */
		nRoutingIndex = pAudioRoutingHandle->pRoutingTable[nIndex1 + A2B_MAX_CODEC_CHANNELS];
		if ((nRoutingIndex > (a2b_UInt8)A2B_UPSTREAM_CH31) && (nRoutingIndex < (a2b_UInt8)A2B_CHANNEL_UNUSED))
		{
			nRoutingIndex -= (a2b_UInt8)A2B_ADC_CH00;
			/* Transfer the data from Rx to Tx */
			for (nIndex2 = 0u; nIndex2 < nChannel * SPORT_BUFFER_SIZE; nIndex2 += (a2b_UInt16)nChannel)
			{
				anDwnstreamBuffer[nIndex1 + nIndex2] = anADCBuffer[nRoutingIndex][nIndex3++];
			}
		}

	}

}

/***********************************************************************************/
/*!
 @brief          This function copies upstream data to DAC

 @param [in]     pHandle       Pointer to serial port structure.
 @param [in]     iTxBuffer     Destination buffer address.
 @param [in]     iRxBuffer     Source buffer address  .
 @param [in]     nSize         Size of the buffer .

 @return         Return code
 - 0: Success
 - 1: Failure
 */
/***********************************************************************************/
static void adi_a2b_UpstreamToDACRouting(ADI_A2B_SCOMM_HANDLER* pAudioRoutingHandle, a2b_UInt32 anDACBuffer[], a2b_UInt32 anUpstreamBuffer[][SPORT_BUFFER_SIZE], a2b_UInt32 nChannel)
{
	a2b_UInt16 nIndex1, nIndex2, nIndex3;
	a2b_UInt8 nRoutingIndex;

	/* Outer most loop */
	for (nIndex1 = 0u; nIndex1 < nChannel; nIndex1++)
	{
		nIndex3 = 0u;
		nRoutingIndex = pAudioRoutingHandle->pRoutingTable[nIndex1];

		if (nRoutingIndex <= (a2b_UInt8)A2B_UPSTREAM_CH31)
		{
			nRoutingIndex -= (a2b_UInt8)A2B_UPSTREAM_CH00;
			/* Transfer the data from Rx to Tx */
			for (nIndex2 = 0u; nIndex2 < nChannel * SPORT_BUFFER_SIZE; nIndex2 += (a2b_UInt16)nChannel)
			{
				anDACBuffer[nIndex1 + nIndex2] = anUpstreamBuffer[nRoutingIndex][nIndex3++];
			}
		}

	}

}

/***********************************************************************************/
/*!
 @brief          This function copies ADC data to DAC

 @param [in]     pHandle       Pointer to serial port structure.
 @param [in]     iTxBuffer     Destination buffer address.
 @param [in]     iRxBuffer     Source buffer address.
 @param [in]     nSize         Size of the buffer.

 @return         Return code
 - 0: Success
 - 1: Failure
 */
/***********************************************************************************/
static void adi_a2b_ADCToDACRouting(ADI_A2B_SCOMM_HANDLER* pAudioRoutingHandle, a2b_UInt32 anDACBuffer[], a2b_UInt32 anADCBuffer[][SPORT_BUFFER_SIZE], a2b_UInt32 nChannel)
{
	a2b_UInt16 nIndex1, nIndex2, nIndex3;
	a2b_UInt8 nRoutingIndex;

	/* Outer most loop */
	for (nIndex1 = 0u; nIndex1 < nChannel; nIndex1++)
	{
		nIndex3 = 0u;

		nRoutingIndex = pAudioRoutingHandle->pRoutingTable[nIndex1];

		if ((nRoutingIndex > (a2b_UInt8)A2B_UPSTREAM_CH31) && (nRoutingIndex < (a2b_UInt8)A2B_CHANNEL_UNUSED))
		{
			nRoutingIndex -= (a2b_UInt8)A2B_ADC_CH00;
			/* Transfer the data from Rx to Tx */
			for (nIndex2 = 0u; nIndex2 < nChannel * SPORT_BUFFER_SIZE; nIndex2 += (a2b_UInt16)nChannel)
			{
				anDACBuffer[nIndex1 + nIndex2] = anADCBuffer[nRoutingIndex][nIndex3++];
			}
		}

	}

}

/***********************************************************************************/
/*!
 @brief          This function deinterleaves multi channel audio and stores to processing buffer


 @param [in]     anDesbuffer     Destination buffer address.
 @param [in]     anADCBuffer     Source buffer address.
 @param [in]     nChannel        Number of TDM channels.

 @return         Return code
 - 0: Success
 - 1: Failure
 */
/***********************************************************************************/
static void adi_a2b_DeInterleaveCopy(a2b_UInt32 anDesbuffer[][SPORT_BUFFER_SIZE], a2b_UInt32 anSrcBuffer[], a2b_UInt32 nChannel)
{
	a2b_UInt16 nIndex1, nIndex2;

	/* Outer most loop */
	for (nIndex1 = 0u; nIndex1 < nChannel; nIndex1++)
	{
		/* Transfer the data from SPORT buffer to App buffer */
		for (nIndex2 = 0u; nIndex2 < (a2b_UInt16)SPORT_BUFFER_SIZE; nIndex2++)
		{
			anDesbuffer[nIndex1][nIndex2] = anSrcBuffer[nIndex1 + nIndex2 * (a2b_UInt16)nChannel];
		}

	}
}
