
/*=============================================================================
 *
 * Project: a2bstack
 *
 * Copyright (c) 2015 - Analog Devices Inc. All Rights Reserved.
 * This software is subject to the terms and conditions of the license set 
 * forth in the project LICENSE file. Downloading, reproducing, distributing or 
 * otherwise using the software constitutes acceptance of the license. The 
 * software may not be used except as expressly authorized under the license.
 *
 *=============================================================================
 *
 * \file:   pal_log.c
 * \author: Mentor Graphics, Embedded Software Division
 * \brief:  Implementation of the platform abstraction layer routines.
 *
 *=============================================================================
 */

/*======================= I N C L U D E S =========================*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/uio.h>
#include <netinet/in.h>
#include <netdb.h>
#include <errno.h>
#include <string.h>
#include "a2b/pal.h"
#include "a2b/ctypes.h"
#include "a2b/conf.h"
#include "a2b/platform.h"
#include "a2b/error.h"
#include "a2b/pal.h"
#include "a2b/util.h"
#include "pal_log.h"

/*----------------------------------------------------------------------------*/
/** 
 * \ingroup  a2bstack_pal_log 
 * \defgroup a2bstack_pal_log_priv          \<Private\> 
 *  
 * This defines the private API's that are private to log.
 *  
 * \{ */
/*----------------------------------------------------------------------------*/
/**
 * \defgroup a2bstack_pal_log_defs          Types/Defs
 *  
 * The various defines and data types used within the log module.
 *
 * \{ */
/*----------------------------------------------------------------------------*/

/*======================= D E F I N E S ===========================*/

#define PAL_RESOURCE_MAX_LEN    (128)

/*======================= D A T A T Y P E S =======================*/

typedef enum
{
    PAL_CHAN_PROTO_UNKNOWN = 0,
    PAL_CHAN_PROTO_STDIO,
    PAL_CHAN_PROTO_FILE,
    PAL_CHAN_PROTO_TCP,
    PAL_CHAN_PROTO_UDP,
    PAL_CHAN_PROTO_SYSLOG,
} pal_ChanProtocol;


typedef struct pal_LogChannel
{
    a2b_Bool inUse;
    a2b_Int32 fd;
    pal_ChanProtocol protocol;
    struct sockaddr_in svrAddr;
    a2b_Int32 sysLogPriority;
} pal_LogChannel;

/*======================= L O C A L  P R O T O T Y P E S  =========*/

/*======================= D A T A  ================================*/

static a2b_Bool       gbChannelPoolInit = A2B_FALSE;
static pal_LogChannel gsChannelPool[A2B_CONF_MAX_NUM_MASTER_NODES * \
                                    A2B_CONF_LOG_NUM_CHANNELS];

/** \} -- a2bstack_pal_log_defs */
/** \} -- a2bstack_pal_log_priv */


/*======================= C O D E =================================*/

/*!****************************************************************************
*
*  \b              pal_logInit
*
*  <b> API Details: </b><br>
*  This routine is called to do initialization the log subsystem
*  during the stack allocation process.
*                                                                       <br><br>
*  <b> Linux Implementation Details: </b><br>
*  Initialize the pool of log channels. Always returns A2B_RESULT_SUCCESS
*                                                                           <br>
*  \param          [in]    ecb      The environment control block (ecb) for
*                                   this platform.
*
*  \pre            None
*
*  \post           None
*
*  \return         A status code that can be checked with the A2B_SUCCEEDED()
*                  or A2B_FAILED() for success or failure.
*
******************************************************************************/
a2b_HResult
pal_logInit
    (
    A2B_ECB*    ecb
    )
{
    a2b_UInt16 idx;
    pal_LogChannel* chan;

    A2B_UNUSED(ecb);
    if ( !gbChannelPoolInit )
    {
        for ( idx = 0; idx < A2B_ARRAY_SIZE(gsChannelPool); idx++ )
        {
            chan = &gsChannelPool[idx];
            chan->inUse = A2B_FALSE;
            chan->fd = A2B_INVALID_FD;
            chan->protocol = PAL_CHAN_PROTO_UNKNOWN;
            chan->sysLogPriority = LOG_USER;
        }
        gbChannelPoolInit = A2B_TRUE;
    }
    
    return A2B_RESULT_SUCCESS;
} /* pal_logInit */


/*!****************************************************************************
*
*  \b              pal_logShutdown
*
*  <b> API Details: </b><br>
*  This routine is called to shutdown the log subsystem
*  during the stack destroy process.  This routine is called immediately
*  after the pal_logClose (assuming the close was successful).
*                                                                       <br><br>
*  <b> Linux Implementation Details: </b><br>
*  Only clears the pool init flag which causes pal_logInit to reinit the
*  channel tracking.  Always returns A2B_RESULT_SUCCESS.
*                                                                           <br>
*  \param          [in]    ecb      The environment control block (ecb) for
*                                   this platform.
*
*  \pre            None
*
*  \post           None
*
*  \return         A status code that can be checked with the A2B_SUCCEEDED()
*                  or A2B_FAILED() for success or failure.
*
******************************************************************************/
a2b_HResult
pal_logShutdown
    (
    A2B_ECB*    ecb
    )
{
    A2B_UNUSED(ecb);
    gbChannelPoolInit = A2B_FALSE;
    return A2B_RESULT_SUCCESS;
} /* pal_logShutdown */


/*!****************************************************************************
*
*  \b              pal_logOpen
*
*  <b> API Details: </b><br>
*  This routine is called to do post-initialization the log subsystem
*  during the stack allocation process.  This routine is called immediately
*  after the pal_logInit (assuming the init was successful).
*                                                                       <br><br>
*  <b> Linux Implementation Details: </b><br>
*  Linux supports a number of log options.  The URL is used to define the 
*  destination for logging. The following log destinations are:
*  \verbatim 
   file://<filename>
   stdio://stderr
   stdio://stdout
   tcp://<host>:<port>
   udp://<host>:<port>
   syslog://<level>
       where level = emerg, alert, crit, err, warning, notice, info, debug
   \endverbatim
*                                                                           <br>
*                                  Example: 
*                                  \verbatim udp://localhost:18001 \endverbatim
*                                                                           <br>
*  \param          [in]    url    A URL indicating where traces should be 
*                                 delivered. This can be tailored and 
*                                 interpreted by the platform implementation 
*                                 of the "generic" stack.
*
*  \pre            None
*
*  \post           None
*
*  \return         If non-zero the open was considered sucessfully.  If A2B_NULL
*                  I2C initialization must have failed and the stack
*                  allocation will fail.  This returned handle will be passed
*                  back into the other PAL log (pal_logXXX) API calls.
*
******************************************************************************/
a2b_Handle
pal_logOpen
    (
    const a2b_Char* url
    )
{
    a2b_Handle* hnd = A2B_NULL;
    pal_LogChannel* chan = A2B_NULL;
    a2b_UInt16 idx;
    a2b_Char* sep;
    a2b_Char resource[PAL_RESOURCE_MAX_LEN] = { '\0' };
    a2b_Int32 port = -1;
    struct hostent* server;
    const char* sysLogLvlName[] = { "emerg", "alert", "crit", "err", "warning",
                                    "notice", "info", "debug" };

    a2b_Int32 sysLogLvlValue[] = { LOG_EMERG, LOG_ALERT, LOG_CRIT, LOG_ERR,
                                    LOG_WARNING, LOG_NOTICE, LOG_INFO,
                                    LOG_DEBUG };

    if ( A2B_NULL != url )
    {
        for ( idx = 0; idx < A2B_ARRAY_SIZE(gsChannelPool); idx++ )
        {
            if ( !gsChannelPool[idx].inUse )
            {
                /* We found an available channel */
                chan = &gsChannelPool[idx];
                break;
            }
        }

        if ( A2B_NULL != chan )
        {
            sep = strstr(url, "://");
            if ( sep )
            {
                /* Skip the separator characters */
                sep += 3;
                strncpy(resource, sep, PAL_RESOURCE_MAX_LEN);
                resource[PAL_RESOURCE_MAX_LEN - 1]= '\0';
            }

            chan->fd = A2B_INVALID_FD;
            if ( 0 == strncmp(url, "stdio", 5) )
            {
                chan->protocol = PAL_CHAN_PROTO_STDIO;
                if ( 0 == strcmp(resource, "stdout") )
                {
                    chan->fd = dup(STDOUT_FILENO);
                }
                else if ( 0 == strcmp(resource, "stderr") )
                {
                    chan->fd = dup(STDERR_FILENO);
                }
            }
            else if ( 0 == strncmp(url, "file", 4) )
            {
                chan->protocol = PAL_CHAN_PROTO_FILE;
                chan->fd = open(resource, O_WRONLY|O_CREAT|O_TRUNC,
                                S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP |
                                S_IROTH | S_IWOTH);
            }
            else if ( 0 == strncmp(url, "tcp", 3) )
            {
                chan->protocol = PAL_CHAN_PROTO_TCP;
                chan->fd = socket(AF_INET, SOCK_STREAM, 0);
                if ( chan->fd >= 0 )
                {
                    sep = strstr(resource, ":");
                    if ( sep )
                    {
                        sep[0] = '\0'; /* terminate for the resource string */
                        sep++;
                        port = atoi(sep);
                    }

                    server = gethostbyname(resource);
                    if ( NULL == server )
                    {
                        close(chan->fd);
                        chan->fd = A2B_INVALID_FD;
                    }
                    else
                    {
                        a2b_memset(&chan->svrAddr, 0, sizeof(chan->svrAddr));
                        chan->svrAddr.sin_family = AF_INET;
                        chan->svrAddr.sin_addr = *((struct in_addr*)
                                                    server->h_addr);
                        chan->svrAddr.sin_port = htons(port);

                        if ( connect(chan->fd, (struct sockaddr*)&chan->svrAddr,
                            sizeof(struct sockaddr)) < 0 )
                        {
                            close(chan->fd);
                            chan->fd = A2B_INVALID_FD;
                        }
                    }
                }
            }
            else if ( 0 == strncmp(url, "udp", 3) )
            {
                chan->protocol = PAL_CHAN_PROTO_UDP;
                chan->fd = socket(AF_INET, SOCK_DGRAM, 0);
                if ( chan->fd >= 0 )
                {
                    sep = strstr(resource, ":");
                    if ( sep )
                    {
                        sep[0] = '\0'; /* terminate for the resource string */
                        sep++;
                        port = atoi(sep);
                    }

                    server = gethostbyname(resource);
                    if ( NULL == server )
                    {
                        close(chan->fd);
                        chan->fd = A2B_INVALID_FD;
                    }
                    else
                    {
                        a2b_memset(&chan->svrAddr, 0, sizeof(chan->svrAddr));
                        chan->svrAddr.sin_family = AF_INET;
                        chan->svrAddr.sin_addr = *((struct in_addr*)
                                                    server->h_addr);
                        chan->svrAddr.sin_port = htons(port);
                    }
                }
            }
            else if ( 0 == strncmp(url, "syslog", 6) )
            {
                chan->protocol = PAL_CHAN_PROTO_SYSLOG;
                chan->fd = 0;

                for ( idx = 0; idx < A2B_ARRAY_SIZE(sysLogLvlName); ++idx )
                {
                    if ( 0 == strcmp(resource, sysLogLvlName[idx]) )
                    {
                        chan->sysLogPriority =
                                LOG_MAKEPRI(LOG_USER, sysLogLvlValue[idx]);
                        break;
                    }
                }

                if ( idx >= A2B_ARRAY_SIZE(sysLogLvlName) )
                {
                    chan->sysLogPriority = LOG_MAKEPRI(LOG_USER,LOG_DEBUG);
                }

                /* We assume the application using this driver has opened
                 * the syslog with a suitable identifier.
                 */
            }

            if ( chan->fd >= 0 )
            {
                chan->inUse = A2B_TRUE;
                hnd = (a2b_Handle)chan;
            }
        }
    }

    return hnd;
} /* pal_logOpen */


/*!****************************************************************************
*
*  \b              pal_logClose
*
*  <b> API Details: </b><br>
*  This routine is called to de-initialization the log subsystem
*  during the stack destroy process.
*                                                                       <br><br>
*  <b> Linux Implementation Details: </b><br>
*  The close operation for the specific log destination is called.
*                                                                           <br>
*  \param          [in]    hnd      The handle returned from pal_logOpen
*
*  \pre            None
*
*  \post           None
*
*  \return         A status code that can be checked with the A2B_SUCCEEDED()
*                  or A2B_FAILED() for success or failure.
*
******************************************************************************/
a2b_HResult
pal_logClose
    (
    a2b_Handle  hnd
    )
{
    pal_LogChannel* chan;
    a2b_HResult status = A2B_MAKE_HRESULT(A2B_SEV_FAILURE,
                                            A2B_FAC_PLATFORM,
                                            EBADF);

    if ( A2B_NULL != hnd )
    {
        chan = (pal_LogChannel*)hnd;

        if ( 0 <= chan->fd )
        {
            if ( (PAL_CHAN_PROTO_TCP == chan->protocol) ||
                (PAL_CHAN_PROTO_UDP == chan->protocol) )
            {
                shutdown(chan->fd, SHUT_RDWR);
                status = A2B_RESULT_SUCCESS;
            }

            if ( PAL_CHAN_PROTO_SYSLOG == chan->protocol )
            {
                /* Not necessary to close the system log */
                status = A2B_RESULT_SUCCESS;
            }
            else
            {
                if ( 0 != close(chan->fd) )
                {
                    status = A2B_MAKE_HRESULT(A2B_SEV_FAILURE, A2B_FAC_PLATFORM,
                                            errno);
                }
                else
                {
                    status = A2B_RESULT_SUCCESS;
                }
                /* We tried our best to close it */
                chan->fd = A2B_INVALID_FD;
            }

            chan->inUse = A2B_FALSE;
        }
    }

    return status;
} /* pal_logClose */


/*!****************************************************************************
*
*  \b              pal_logWrite
*
*  <b> API Details: </b><br>
*  This routine writes to a log channel.
*                                                                       <br><br>
*  <b> Linux Implementation Details: </b><br>
*  Writes a NULL terminated string to the log channel.
*                                                                           <br>
*  \param          [in]    hnd      The handle returned from pal_logOpen
* 
*  \param          [in]    msg      NULL terminated string to log
*
*  \pre            None
*
*  \post           None
*
*  \return         A status code that can be checked with the A2B_SUCCEEDED()
*                  or A2B_FAILED() for success or failure.
*
******************************************************************************/
a2b_HResult
pal_logWrite
    (
    a2b_Handle      hnd,
    const a2b_Char* msg
    )
{
    pal_LogChannel* chan;
    a2b_HResult status = A2B_MAKE_HRESULT(A2B_SEV_FAILURE, A2B_FAC_PLATFORM,
                                            EIO);
    struct iovec vec[2];
    struct msghdr mHdr;
    static const a2b_Char terminator[] = "\n";

    a2b_Int32 nWritten = 0;

    if ( (A2B_NULL != hnd) && (A2B_NULL != msg) )
    {
        chan = (pal_LogChannel*)hnd;

        if ( chan->protocol == PAL_CHAN_PROTO_SYSLOG )
        {
            syslog(chan->sysLogPriority, "%s", msg);
        }
        else
        {
            vec[0].iov_base = (void*)msg;
            vec[0].iov_len = a2b_strlen(msg) * sizeof(a2b_Char);
            vec[1].iov_base = (void*)terminator;
            vec[1].iov_len = a2b_strlen(terminator) * sizeof(a2b_Char);

            if ( chan->protocol == PAL_CHAN_PROTO_UDP )
            {
                a2b_memset(&mHdr, 0, sizeof(mHdr));
                mHdr.msg_name = &chan->svrAddr;
                mHdr.msg_namelen = sizeof(chan->svrAddr);
                mHdr.msg_iov = vec;
                mHdr.msg_iovlen = A2B_ARRAY_SIZE(vec);
                mHdr.msg_control = 0;
                mHdr.msg_controllen = 0;

                if ( -1 == sendmsg(chan->fd, &mHdr, MSG_DONTWAIT) )
                {
                    nWritten = -1;
                }
                else
                {
                    nWritten = (vec[0].iov_len + vec[1].iov_len) *
                                sizeof(a2b_Char);
                }

            }
            else
            {
                nWritten = writev(chan->fd, &vec[0], A2B_ARRAY_SIZE(vec));
            }

            if ( nWritten == (a2b_Int32)((vec[0].iov_len + vec[1].iov_len) *
                sizeof(a2b_Char)) )
            {
                status = A2B_RESULT_SUCCESS;
            }
            else if ( nWritten == -1 )
            {
                status = A2B_MAKE_HRESULT(A2B_SEV_FAILURE, A2B_FAC_PLATFORM,
                                            errno);
            }
        }
    }

    return status;
} /* pal_logWrite */
