
/*=============================================================================
 *
 * Project: a2bstack
 *
 * Copyright (c) 2015 - Analog Devices Inc. All Rights Reserved.
 * This software is subject to the terms and conditions of the license set 
 * forth in the project LICENSE file. Downloading, reproducing, distributing or 
 * otherwise using the software constitutes acceptance of the license. The 
 * software may not be used except as expressly authorized under the license.
 *
 *=============================================================================
 *
 * \file:   pal_plugin.c
 * \author: Mentor Graphics, Embedded Software Division
 * \brief:  Implementation of the platform abstraction for plugin services.
 *
 *=============================================================================
 */

/*======================= I N C L U D E S =========================*/

#include <stdlib.h>
#include <string.h>
#include <glob.h>
#include <dlfcn.h>
#include "a2b/error.h"
#include "a2b/pluginapi.h"
#include "a2b/platform.h"
#include "pal_plugin.h"

/*----------------------------------------------------------------------------*/
/** 
 * \ingroup  a2bstack_pal_plugin 
 * \defgroup a2bstack_pal_plugin_priv          \<Private\> 
 *  
 * This defines the private API's that are private to plugins.
 *  
 * \{ */
/*----------------------------------------------------------------------------*/
/** 
 * \defgroup a2bstack_pal_plugin_defs          Types/Defs
 *  
 * The various defines and data types used within the plugin module.
 *
 * \{ */
/*----------------------------------------------------------------------------*/

/*======================= D E F I N E S ===========================*/

/*======================= D A T A T Y P E S =======================*/

/**
 * PAL plugin data control block that tracks the list of plugins 
 * for a single stack. 
 */
typedef struct pal_PluginDcb
{
    a2b_Char*           searchPattern;
    a2b_PluginApi*      plugins;
    void**              pluginHnds;
    a2b_UInt32          numHnds;
    a2b_UInt32          refCnt;
} pal_PluginDcb;


/*======================= L O C A L  P R O T O T Y P E S  =========*/

/*======================= D A T A  ================================*/

static pal_PluginDcb gsModulePool[A2B_CONF_MAX_NUM_MASTER_NODES];

/** \} -- a2bstack_pal_plugin_defs */
/** \} -- a2bstack_pal_plugin_priv */


/*======================= C O D E =================================*/


/*!****************************************************************************
*
*  \b              pal_pluginsLoad
*
*  <b> API Details: </b><br>
*  This routine is called to search and load all available plugins.
*  The list of available plugins are returned and typically used to query
*  for plugins during discovery when nodes are found.
*                                                                       <br><br>
*  This routine can be overeidden to allow for specialize plugin handling.
*  For example, loading static libraries with or without dynamic support.
*                                                                       <br><br>
*  <b> Linux Implementation Details: </b><br>
*  This routine searches A2B_CONF_PLUGIN_SEARCH_PATTERN looking for shared
*  DLL plugins to load.  The list of available plugins are returned to query
*  for plugins during discovery when nodes are found.
*                                                                           <br>
*  \param          [out]       plugins      List of found/loaded plugins
* 
*  \param          [in,out]    numPlugins   Number of plugins found 
* 
*  \param          [in]        ecb          The environment control block (ecb)
*                                           for this platform.
*
*  \pre            None
*
*  \post           None
*
*  \return         A status code that can be checked with the A2B_SUCCEEDED()
*                  or A2B_FAILED() for success or failure.
*
******************************************************************************/
a2b_HResult
pal_pluginsLoad
    (
    struct a2b_PluginApi**  plugins,
    a2b_UInt16*             numPlugins,
    A2B_ECB*                ecb
    )
{
    glob_t shlibs;
    a2b_UInt32 idx;
    void* hnd;
    a2b_PluginInitFunc initFunc;
    pal_PluginDcb* dcb = A2B_NULL;

    a2b_HResult status = A2B_RESULT_SUCCESS;

    if ( (A2B_NULL == plugins) || (A2B_NULL == numPlugins) ||
        (A2B_NULL == ecb) )
    {
        status = A2B_MAKE_HRESULT(A2B_SEV_FAILURE, A2B_FAC_PLATFORM,
                                    A2B_EC_INVALID_PARAMETER);
    }
    else
    {
        /* We start with no plugins loaded */
        *plugins = A2B_NULL;
        *numPlugins = 0;

        /* First let's see if the module search pattern has already been
         * seen.
         */
        for ( idx = 0; idx < A2B_ARRAY_SIZE(gsModulePool); ++idx )
        {
            if ( (gsModulePool[idx].searchPattern != A2B_NULL) &&
                (0 == strcmp(gsModulePool[idx].searchPattern,
                 ecb->palEcb.pluginSearchPattern)) )
            {
                dcb = &gsModulePool[idx];
                break;
            }
        }

        if ( A2B_NULL != dcb )
        {
            *plugins = dcb->plugins;
            *numPlugins = dcb->numHnds;
            dcb->refCnt += 1;
        }
        else
        {
            if ( 0 != glob(ecb->palEcb.pluginSearchPattern,
                GLOB_TILDE, A2B_NULL, &shlibs) )
            {
                /* It's possible there are no plugins to load */
                shlibs.gl_pathc = 0;
            }


            if ( A2B_SUCCEEDED(status) )
            {
                /* Look for an available module control block */
                for ( idx = 0; idx < A2B_ARRAY_SIZE(gsModulePool); ++idx )
                {
                    /* Did we find one? */
                    if ( A2B_NULL == gsModulePool[idx].plugins )
                    {
                        dcb = &gsModulePool[idx];
                        dcb->numHnds = 0;
                        dcb->plugins = A2B_NULL;
                        dcb->searchPattern =
                                    strdup(ecb->palEcb.pluginSearchPattern);
                        dcb->pluginHnds = calloc(shlibs.gl_pathc,
                                                sizeof(void*));
                        dcb->plugins = calloc(shlibs.gl_pathc,
                                            sizeof(*dcb->plugins));
                        if ( (A2B_NULL == dcb->searchPattern) ||
                            (A2B_NULL == dcb->pluginHnds) ||
                            (A2B_NULL == dcb->plugins))
                        {
                            /* It's safe to 'free' a null pointer */
                            free(dcb->pluginHnds);
                            free(dcb->searchPattern);
                            free(dcb->plugins);
                            /* We couldn't allocate the required memory */
                            dcb = A2B_NULL;
                        }
                        break;
                    }
                }

                if ( A2B_NULL == dcb )
                {
                    status = A2B_MAKE_HRESULT(A2B_SEV_FAILURE,
                                              A2B_FAC_PLATFORM,
                                              A2B_EC_ALLOC_FAILURE);
                }
            }


            if ( A2B_SUCCEEDED(status) )
            {
                for ( idx = 0; idx < shlibs.gl_pathc; ++idx )
                {
                    /* We do a best effort to load as many plugins as we
                     * can skipping those that fail to load or that do not
                     * export the initialization function.
                     */
                    hnd = dlopen(shlibs.gl_pathv[idx], RTLD_LAZY);
                    if (A2B_NULL != hnd )
                    {
                        initFunc = (a2b_PluginInitFunc)dlsym(hnd,
                                                        "a2b_pluginInit");

                        if ( (initFunc == A2B_NULL) ||
                            (A2B_TRUE !=
                            initFunc(&dcb->plugins[dcb->numHnds])) )
                        {
                            dlclose(hnd);
                        }
                        else
                        {
                            dcb->pluginHnds[dcb->numHnds] = hnd;
                            dcb->numHnds += 1;
                        }
                    }
                }
            }

            if ( A2B_SUCCEEDED(status) )
            {
                *numPlugins = dcb->numHnds;
                *plugins = dcb->plugins;
                dcb->refCnt = 1;
            }

            if ( shlibs.gl_pathc > 0 )
            {
                globfree(&shlibs);
            }

            if ( A2B_FAILED(status) )
            {
                if ( A2B_NULL != dcb )
                {
                    if ( A2B_NULL != dcb->pluginHnds )
                    {
                        for ( idx = 0; idx < dcb->numHnds; ++idx )
                        {
                            dlclose(dcb->pluginHnds[idx]);
                        }
                        free(dcb->pluginHnds);
                        dcb->pluginHnds = A2B_NULL;
                        dcb->numHnds = 0;
                        dcb->refCnt = 0;
                        free(dcb->searchPattern);
                        dcb->searchPattern = A2B_NULL;
                        free(dcb->plugins);
                        dcb->plugins = A2B_NULL;
                    }
                }
            }
        }
    }
    return status;

} /* pal_pluginsLoad */


/*!****************************************************************************
*
*  \b              pal_pluginsUnload
*
*  <b> API Details: </b><br>
*  This routine is called to unload previously loaded plugins from
*  pal_pluginsLoad.
*                                                                       <br><br>
*  <b> Linux Implementation Details: </b><br>
*  This routine performs dlclose operations on all previously loaded shared
*  DLL plugins.
*                                                                           <br>
*  \param          [in]    plugins      List of plugins
* 
*  \param          [in]    numPlugins   Total plugins in "plugins"
* 
*  \param          [in]    ecb          The environment control block (ecb)
*                                       for this platform.
*
*  \pre            None
*
*  \post           None
*
*  \return         A status code that can be checked with the A2B_SUCCEEDED()
*                  or A2B_FAILED() for success or failure.
*
******************************************************************************/
a2b_HResult
pal_pluginsUnload
    (
    struct a2b_PluginApi*   plugins,
    a2b_UInt16              numPlugins,
    A2B_ECB*                ecb
    )
{
    a2b_UInt32 idx;
    pal_PluginDcb* dcb = A2B_NULL;
    a2b_HResult status = A2B_RESULT_SUCCESS;
    if ( (A2B_NULL == ecb) || ((A2B_NULL == plugins) && (numPlugins != 0)) )
    {
        status = A2B_MAKE_HRESULT(A2B_SEV_FAILURE, A2B_FAC_PLATFORM,
                                    A2B_EC_INVALID_PARAMETER);
    }
    else
    {
        /* Look the matching module control block */
        for ( idx = 0; idx < A2B_ARRAY_SIZE(gsModulePool); ++idx )
        {
            /* Did we find one? */
            if ( plugins == gsModulePool[idx].plugins )
            {
                dcb = &gsModulePool[idx];
                break;
            }
        }

        /* If we can find the associated control block then ... */
        if ( A2B_NULL == dcb )
        {
            status = A2B_MAKE_HRESULT(A2B_SEV_FAILURE, A2B_FAC_PLATFORM,
                                    A2B_EC_INVALID_PARAMETER);
        }
        else
        {
            if ( dcb->refCnt > 1 )
            {
                dcb->refCnt -= 1;
            }
            /* Else no one else is referencing these plugins so free it */
            else
            {
                for ( idx = 0; idx < dcb->numHnds; ++idx )
                {
                    dlclose(dcb->pluginHnds[idx]);
                }
                free(dcb->pluginHnds);
                dcb->pluginHnds = A2B_NULL;
                dcb->numHnds = 0;
                dcb->refCnt = 0;
                free(dcb->searchPattern);
                dcb->searchPattern = A2B_NULL;
                free(dcb->plugins);
                dcb->plugins = A2B_NULL;
            }
        }
    }
    return status;

} /* pal_pluginsUnload */

