/*******************************************************************************
Copyright (c) 2014 - Analog Devices Inc. All Rights Reserved.
This software is proprietary & confidential to Analog Devices, Inc.
and its licensors.
******************************************************************************
 * @file:    adi_a2b_sys.c
 * @brief:   System component APIs are defined in this file
 * @version: $Revision: 66678 $
 * @date:    $Date: 2021-08-04 06:49:25 -0400 (Wed, 04 Aug 2021) $
 * Developed by: Automotive Software and Systems team, Bangalore, India
*****************************************************************************/
/*! \addtogroup ADI_A2B_SYS
 *  @{
 */

/*============= I N C L U D E S =============*/

#include "adi_a2b_sys.h"
#include <sys/adi_core.h>

/*============= D E F I N E S =============*/
#if defined(__ADSPSC589__)
#define LED10_PORT    (ADI_A2B_HAL_GPIO_PORT_E)     /*!< HPC EZ-Kit uses port E for LED 10 */
#define LED10_PIN     (ADI_A2B_HAL_GPIO_PIN_13)     /*!< HPC EZ-Kit uses pin 13 for LED 10 */
#define LED11_PORT    (ADI_A2B_HAL_GPIO_PORT_E)     /*!< HPC EZ-Kit uses port E for LED 11 */
#define LED11_PIN     (ADI_A2B_HAL_GPIO_PIN_14)     /*!< HPC EZ-Kit uses pin 14 for LED 11 */
#define LED12_PORT    (ADI_A2B_HAL_GPIO_PORT_E)     /*!< HPC EZ-Kit uses port E for LED 12 */
#define LED12_PIN     (ADI_A2B_HAL_GPIO_PIN_15)     /*!< HPC EZ-Kit uses pin 15 for LED 12 */
#define A2B_MSTR_PORT (ADI_A2B_HAL_GPIO_PORT_F)  /*!< SAM uses port F for enabling MSTR pin on AD2421 */
#define A2B_MSTR_PIN  (ADI_A2B_HAL_GPIO_PIN_13)  /*!< SAM uses pin 13 for enabling MSTR port on AD2421  */
#elif defined(__ADSPSC584__)
#define LED10_PORT    (ADI_A2B_HAL_GPIO_PORT_E)     /*!< LPC EZ-Kit uses port E for LED 10 */
#define LED10_PIN     (ADI_A2B_HAL_GPIO_PIN_1)      /*!< LPC EZ-Kit uses pin 1 for LED 10 */
#define LED11_PORT    (ADI_A2B_HAL_GPIO_PORT_E)     /*!< LPC EZ-Kit uses port E for LED 11 */
#define LED11_PIN     (ADI_A2B_HAL_GPIO_PIN_2)      /*!< LPC EZ-Kit uses pin 2 for LED 11 */
#define LED12_PORT    (ADI_A2B_HAL_GPIO_PORT_E)     /*!< LPC EZ-Kit uses port E for LED 12 */
#define LED12_PIN     (ADI_A2B_HAL_GPIO_PIN_3)      /*!< LPC EZ-Kit uses pin 3 for LED 12 */
#define LED13_PORT    (ADI_A2B_HAL_GPIO_PORT_E)     /*!< LPC EZ-Kit uses port E for LED 13 */
#define LED13_PIN     (ADI_A2B_HAL_GPIO_PIN_4)      /*!< LPC EZ-Kit uses pin 4 for LED 13 */
#define LED14_PORT    (ADI_A2B_HAL_GPIO_PORT_E)     /*!< LPC EZ-Kit uses port E for LED 14 */
#define LED14_PIN     (ADI_A2B_HAL_GPIO_PIN_5)      /*!< LPC EZ-Kit uses pin 5 for LED 14 */
#define LED15_PORT    (ADI_A2B_HAL_GPIO_PORT_E)     /*!< LPC EZ-Kit uses port E for LED 15 */
#define LED15_PIN     (ADI_A2B_HAL_GPIO_PIN_6)      /*!< LPC EZ-Kit uses pin 6 for LED 15 */
#define LED16_PORT    (ADI_A2B_HAL_GPIO_PORT_E)     /*!< LPC EZ-Kit uses port E for LED 16 */
#define LED16_PIN     (ADI_A2B_HAL_GPIO_PIN_7)      /*!< LPC EZ-Kit uses pin 7 for LED 16 */
#define LED17_PORT    (ADI_A2B_HAL_GPIO_PORT_E)     /*!< LPC EZ-Kit uses port E for LED 17 */
#define LED17_PIN     (ADI_A2B_HAL_GPIO_PIN_8)      /*!< LPC EZ-Kit uses pin 8 for LED 17 */
#endif

#define PORT_CODEC_RESET    (ADI_A2B_HAL_GPIO_PORT_A)            /*!< Port used for resetting the ADC and DAC. This is
                                                                     a hard wired port number for CODEC resetting. */
#define PIN_CODEC_RESET     (ADI_A2B_HAL_GPIO_PIN_14)            /*!< Pin used for resetting the ADC and DAC. This
                                                                     is a hard wired pin number for CODEC resetting. */
#define PORT_A2B_MASTER_ENABLE	(ADI_A2B_HAL_GPIO_PORT_F)      /*!< Set this PORT to enable master mode on AD2421 */

#define PIN_A2B_MASTER_ENABLE   (ADI_A2B_HAL_GPIO_PIN_13)      /*!< Set this PIN to enable master mode on AD2421*/

#ifndef _TESSY_
#define WAIT_CLOCK_CYCLES(n)      \
        do                        \
        {                         \
			do                    \
			{                     \
            (n)--;                \
			}while((n)>0);        \
        }while(0)
#else	/* _TESSY_ */
#define WAIT_CLOCK_CYCLES(n)      \
			do                    \
			{                     \
            (n)--;                \
			}while((n)>0);
#endif	/* _TESSY_ */

#define CODEC_RESET_DELAY    (100000U)
/*============= D A T A =============*/


/*============= C O D E =============*/

/*
** Function Prototype section
** (static-scoped functions)
*/
static ADI_A2B_SYS_RESULT MASTER_Init(void);

/*
 * Function Definition section
 */

/*!
    @brief        This function allocates memory for the sys components and initializes them.
    @param        pMemBlock    Pointer to blocks of memory required for the creation of
                               system component
    @param        pSysConfig   System component configuration structure.
    @return       Return code of ADI_A2B_SYS_RESULT type.
                   - #ADI_A2B_SYS_SUCCESS   : System component initialized successfully
                   - #ADI_A2B_SYS_FAILURE   : System initialization failed
                   - #ADI_A2B_SYS_INSUFFICIENT_MEM : Insufficient memory for initialization of system component
                   - #ADI_A2B_SYS_CORE_SECONDARY : Secondary core configured for system initialization. However system
                                                                          initialization must happen from primary core

    <b> Global Variables Used: <b>
                None

   @note        None.
 */
ADI_A2B_SYS_RESULT adi_a2b_sys_Init(void *pMemBlock, uint32_t const  nMemorySize,ADI_A2B_SYS_CONFIG *pSysConfig)
{
    ADI_A2B_SYS_RESULT eResult;

    if(pSysConfig->bProcMaster == true)
    {
        if(nMemorySize < ADI_A2B_SYS_MEMSIZE_BYTES)
        {
            return ADI_A2B_SYS_INSUFFICIENT_MEM;
        }
        eResult = MASTER_Init();
    }
    else
    {
        return ADI_A2B_SYS_CORE_SECONDARY;
    }

    return ADI_A2B_SYS_SUCCESS;
}
static ADI_A2B_SYS_RESULT MASTER_Init(void)
{
	ADI_A2B_HAL_GPIO_STATUS eGpioRetVal;
	uint32_t nDelay;

	eGpioRetVal = adi_a2b_hal_gpio_SetDirection(PORT_A2B_MASTER_ENABLE, PIN_A2B_MASTER_ENABLE, ADI_A2B_HAL_GPIO_DIRECTION_OUTPUT);
	if(eGpioRetVal != ADI_A2B_HAL_GPIO_STATUS_SUCCESS)
	{
		return ADI_A2B_SYS_FAILURE;
	}
	eGpioRetVal = adi_a2b_hal_gpio_Set(PORT_A2B_MASTER_ENABLE, PIN_A2B_MASTER_ENABLE);
	if(eGpioRetVal != ADI_A2B_HAL_GPIO_STATUS_SUCCESS)
	{
		return ADI_A2B_SYS_FAILURE;
	}

	/* Wait for a duration of 1000 clock cycles */
	nDelay = CODEC_RESET_DELAY;
	WAIT_CLOCK_CYCLES(nDelay);

	return ADI_A2B_SYS_SUCCESS;
}

void PADS_Init(void)
{
    *pREG_PADS0_DAI0_IE  = (uint32_t)0xffffffffU;
    *pREG_PADS0_DAI1_IE  = (uint32_t)0xffffffffU;

    *pREG_SPU0_SECUREP19 = (uint32_t)0x3;         /*  SPORT 2A        */
    *pREG_SPU0_SECUREP21 = (uint32_t)0x3;         /*  SPORT 3A        */

    *pREG_SPU0_SECUREP15 = (uint32_t)0x3;         /*  SPORT 0A        */
    *pREG_SPU0_SECUREP17 = (uint32_t)0x3;         /*  SPORT 1A        */

    *pREG_SPU0_SECUREP66 = (uint32_t)0x3;         /*  SPORT 0A DMA0  */
    *pREG_SPU0_SECUREP68 = (uint32_t)0x3;         /*  SPORT 1A DMA2  */

    *pREG_SPU0_SECUREP70=(uint32_t)0x3;            /*  SPORT 2A DMA4  */
    *pREG_SPU0_SECUREP72=(uint32_t)0x3;            /*  SPORT 3A DMA6  */

    *pREG_SPU0_SECUREP88 = (uint32_t)0x3;         /*  MDMA0 SRC DMA8  */
    *pREG_SPU0_SECUREP89 = (uint32_t)0x3;         /*  MDMA0 DST DMA9  */
    *pREG_SPU0_SECUREP90 = (uint32_t)0x3;         /*  MDMA1 SRC DMA18 */
    *pREG_SPU0_SECUREP91 = (uint32_t)0x3;         /*  MDMA1 DST DMA19 */
}

/*!
   @brief        This function does the address translation for the given L1 mem address based on the SHARC ID.
   @param        nMemAddr            L1 Memory address which has to be translated
   @param        eProcID             PROCESSOR ID (only for SHARCS)
   @param        IsGetId             Boolean flag indicating whether to read the processor ID or use passed
                                     eProcID for address translation.
   @return       Translated address
   <b> Global Variables Used: <b>
                None

  @note            None.
*/

uint32 adi_a2b_address_translation(uint32 nMemAddr, ADI_A2B_PROC_ID eProcID, bool IsGetId)
{
    uint32_t result = nMemAddr;
    uint32_t nProcId;

    /* Address translation only applies to SHARC processors */

    if(IsGetId)
    {
    	nProcId = adi_core_id();
    }
    else
    {
    	nProcId = (uint32_t) eProcID;
    }

    /* IF (Supplied memory address is in L1) */
    if ((nMemAddr >= ADI_SC589_SHARC_L1_START_ADDR) && (nMemAddr <= ADI_SC589_SHARC_L1_END_ADDR))
    {
        /* IF (SHARC Core 0) */
        if (nProcId == PROCESSOR_SH0)
        {
            result = (nMemAddr | ADI_SC589_SHARC1_L1_SLAVE1_OFFSET);
        }
        /* ELSE IF (SHARC Core 1) */
        else if (nProcId == PROCESSOR_SH1)
        {
            result = (nMemAddr | ADI_SC589_SHARC2_L1_SLAVE1_OFFSET);
        }
        else
        {
        	result = nMemAddr;
        }
    }
    /* ELSE (Supplied address could be L2 or L3) */
    else
    {
    	result = nMemAddr;
    }

    return result;
}

/*
 *
 * EOF:
 *
*/
