/*
 * Copyright (c) 2019 Analog Devices, Inc.  All rights reserved.
 *
 * Adapted from the SAM Bare-Metal ("BM") SDK Rel 2.1.0
 *
 */

#ifndef _BM_ADAU_DEVICE_H
#define _BM_ADAU_DEVICE_H

#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>
#include <stdlib.h>

#include "bm_twi.h"
#include "register_maps/registers_adau1761.h"


// ADAU driver return values
typedef enum
{
    ADAU_SUCCESS,                 // The API call is success
    ADAU_CORRUPT_INIT_FILE,       // SS-generated initialization file is corrupt
    ADAU_TWI_TIMEOUT_ERROR,       // A TWI timeout error occurred while trying to communicate with device
    ADAU_PLL_LOCK_TIMEOUT_ERROR,      // The PLL of the device we're driving failed to lock
    ADAU_SIMPLE_ERROR               // General failure
} BM_ADAU_RESULT;

// Various known I2C addresses
#define     SAM_ADAU1761_I2C_ADDR         (0x38)

// The number of bytes these devices use for the control register addresses over I2C/TWI
#define     ADAU1761_ADDR_BYTES                 (2)

// Various know MCLK values
#define     SHARC_SAM_MCLK                      (12288000)
#define     SHARC_SAM_AUTOMOTIVE_MCLK           (12288000)

#ifdef __cplusplus
extern "C" {
#endif

// A simple structure that contains information for a single instance of this driver
typedef struct
{
    BM_TWI twi;         // Simple TWI driver
    uint8_t address_bytes;  // Number of bytes used for control register address
} BM_ADAU_DEVICE;

typedef struct
{
    uint8_t     *data_tx_buffer;
    uint16_t    *data_num_bytes;
    uint16_t total_lines;
    bool ignore_first_byte_of_init_file;
} BM_ADAU_DEVICE_INIT_DATA;

/*==============  CONVERTER INITIALIZATION FILES  ===============*/

/*
 * These structures contain the initialization data generated by SigmaStudio.  The schematics that were
 * used to generate the initialization code can be found in the ss_schematics directory.
 */

// ADAU1761 (CODEC on SHARC board and certain A2B boards)
extern BM_ADAU_DEVICE_INIT_DATA adau1761_2ch_i2s_master;

// Initializes an ADAU device
BM_ADAU_RESULT adau_initialize(BM_ADAU_DEVICE *adau_device,
                               BM_TWI_PERIPHERAL_NUMBER device_num,
                               uint8_t i2c_address,
                               BM_ADAU_DEVICE_INIT_DATA *adau_init_data,
                               uint8_t address_bytes);

// Loads a bulk initialization file exported from SigmaStudio into an ADAU device
BM_ADAU_RESULT adau_load_bulk_reg_file(BM_ADAU_DEVICE *adau_device,
                                       uint8_t *values,
                                       uint16_t *lengths,
                                       uint16_t total_lines,
                                       bool ignore_first_byte_of_init_file);

// Reads a control register value from an ADAU device
BM_ADAU_RESULT adau_read_ctrl_reg(BM_ADAU_DEVICE *adau_device,
                                  uint16_t address,
                                  uint8_t *value);

// Writes a control register value from an ADAU device
BM_ADAU_RESULT adau_write_ctrl_reg(BM_ADAU_DEVICE *adau_device,
                                   uint16_t address,
                                   uint8_t value);

// Reads a parameter RAM location in a SigmaDSP ADAU device
BM_ADAU_RESULT adau_read_parameter_ram(BM_ADAU_DEVICE *adau_device,
                                       uint16_t address,
                                       uint32_t *value);

// Writes a parameter RAM location in a SigmaDSP ADAU device
BM_ADAU_RESULT adau_write_parameter_ram(BM_ADAU_DEVICE *adau_device,
                                        uint16_t address,
                                        uint32_t value);

/*
 * Some devices require specific operations that go beyond the basic functions
 * defined above.
 */

#ifdef __cplusplus
} // extern "C"
#endif

#endif  //_BM_ADAU_DEVICE_H
