/*=============================================================================
 *
 * Project: a2bstack
 *
 * Copyright (c) 2015 - Analog Devices Inc. All Rights Reserved.
 * This software is subject to the terms and conditions of the license set 
 * forth in the project LICENSE file. Downloading, reproducing, distributing or 
 * otherwise using the software constitutes acceptance of the license. The 
 * software may not be used except as expressly authorized under the license.
 *
 *=============================================================================
 *
 * \file:   pal_i2c_ioctl.c
 * \author: Mentor Graphics, Embedded Software Division
 * \brief:  The platform I2C implementation using the Linux I2C ioctl
 *          API.
 *
 *=============================================================================
 */

/*============================================================================*/
/** 
 * \ingroup  a2bstack_pal_i2c
 * \defgroup a2bstack_pal_i2c_ioctl         IOCTL I2C Module 
 *   
 * The platform I2C implementation using the Linux I2C ioctl API.
 *  
 * \{ */
/*============================================================================*/

/*======================= I N C L U D E S =========================*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <errno.h>
#include <string.h>
#include <linux/i2c.h>
#include <linux/i2c-dev.h>
#include <sys/ioctl.h>
#include "pal_i2c.h"
#include "a2b/conf.h"
#include "a2b/error.h"
#include "a2b/platform.h"


/*----------------------------------------------------------------------------*/
/** 
 * \defgroup a2bstack_pal_i2c_ioctl_priv             \<Private\> 
 *  
 * This defines the private API's that are private to ioctl I2C. 
 *  
 * \{ */
/*----------------------------------------------------------------------------*/
/**
 * \defgroup a2bstack_pal_i2c_ioctl_defs             Types/Defs
 *  
 * The various defines and data types used within the ioctl I2C modules.
 *
 * \{ */
/*----------------------------------------------------------------------------*/

/*======================= D E F I N E S ===========================*/


/*======================= L O C A L  P R O T O T Y P E S  =========*/

typedef struct pal_I2cDcb
{
    a2b_Bool        inUse;
    a2b_Int32       fd;
    a2b_I2cAddrFmt  fmt;
} pal_I2cDcb;

/*======================= D A T A  ================================*/

static pal_I2cDcb gsI2cDcbPool[A2B_CONF_MAX_NUM_I2C_DEVICES];

/** \} -- a2bstack_pal_i2c_ioctl_defs */


/*======================= C O D E =================================*/

/*!****************************************************************************
*
*  \b              pal_convertIoctlError
*
*  This routine converts a Linux IOCTL API error to an A2B stack error.
*
*  \param          [in]    retCode      Linux IOCTL API error code
*
*  \pre            None
*
*  \post           None
*
*  \return         A status code that can be checked with the A2B_SUCCEEDED()
*                  or A2B_FAILED() for success or failure.
*
******************************************************************************/
static a2b_HResult
pal_convertIoctlError
    (
    int retCode
    )
{
    a2b_HResult status;

    if ( retCode >= 0 )
    {
        retCode = 0;
    }
    else if ( retCode == -1 )
    {
        retCode = errno;
    }
    else if ( retCode < 0 )
    {
        retCode = -retCode;
    }
    switch ( retCode )
    {
        case 0:
            status = A2B_RESULT_SUCCESS;
            break;

        case EBUSY:
            status = A2B_MAKE_HRESULT(A2B_SEV_FAILURE, A2B_FAC_I2C,
                                      A2B_EC_BUSY);
            break;

        case EAGAIN:
            status = A2B_MAKE_HRESULT(A2B_SEV_FAILURE, A2B_FAC_I2C,
                                      A2B_EC_RESOURCE_UNAVAIL);
            break;

        case EPERM:
        case EACCES:
            status = A2B_MAKE_HRESULT(A2B_SEV_FAILURE, A2B_FAC_I2C,
                                      A2B_EC_PERMISSION);
            break;

        case EIO:
        case EREMOTEIO:
            status = A2B_MAKE_HRESULT(A2B_SEV_FAILURE, A2B_FAC_I2C,
                                      A2B_EC_IO);
            break;

        default:
            status = A2B_MAKE_HRESULT(A2B_SEV_FAILURE, A2B_FAC_I2C,
                                      A2B_EC_INTERNAL);
            break;
    }

    return status;
} /* pal_convertIoctlError */

/** \} -- a2bstack_pal_i2c_ioctl_priv */


/*!****************************************************************************
*
*  \b              pal_i2cInit
*
*  <b> API Details: </b><br>
*  This routine is called to do initialization for the I2C subsystem
*  during the stack allocation process.
*                                                                       <br><br>
*  <b> Linux Implementation Details: </b><br>
*  Nothing to do, always returns A2B_RESULT_SUCCESS
*                                                                           <br>
*  \param          [in]    ecb      The environment control block (ecb) for
*                                   this platform.
*  \pre            None
*
*  \post           None
*
*  \return         A status code that can be checked with the A2B_SUCCEEDED()
*                  or A2B_FAILED() for success or failure.
*
******************************************************************************/
a2b_HResult
pal_i2cInit
    (
    A2B_ECB*    ecb
    )
{
    A2B_UNUSED(ecb);
    return A2B_RESULT_SUCCESS;
} /* pal_i2cInit */


/*!****************************************************************************
*
*  \b              pal_i2cOpen
*
*  <b> API Details: </b><br>
*  This routine is called to do post-initialization the I2C subsystem
*  during the stack allocation process.  This routine is called immediately
*  after the pal_i2cInit (assuming the init was successful).
*                                                                       <br><br>
*  <b> Linux Implementation Details: </b><br>
*  Attempts to find and open an available Linux IOCTL I2C device
*  with the user defined settings for the address.
* 
*  \note    The speed setting is not supported in Linux.
*                                                                           <br>
*  \param          [in]    fmt      The I2C address format (7-bit or 10-bit)
* 
*  \param          [in]    speed    The I2C bus speed (100KHz, 400KHz)
*                                   (Speed setting is not supported in Linux)
* 
*  \param          [in]    ecb      The environment control block (ecb) for
*                                   this platform.
*
*  \pre            None
*
*  \post           None
*
*  \return         If non-zero the open was considered sucessfully.  If A2B_NULL
*                  I2C initialization must have failed and the stack
*                  allocation will fail.  This returned handle will be passed
*                  back into the other PAL I2C (pal_i2cXXX) API calls.
*
******************************************************************************/
a2b_Handle
pal_i2cOpen
    (
    a2b_I2cAddrFmt  fmt,
    a2b_I2cBusSpeed speed,
    A2B_ECB*        ecb
    )
{
    pal_I2cDcb* dcb = A2B_NULL;
    a2b_UInt16 idx;

    /* Speed setting is not supported on Linux */
    A2B_UNUSED(speed);

    if ( A2B_NULL != ecb )
    {
        for ( idx = 0; idx < A2B_ARRAY_SIZE(gsI2cDcbPool); ++idx )
        {
            if ( !gsI2cDcbPool[idx].inUse )
            {
                dcb = &gsI2cDcbPool[idx];
                dcb->fd = open(ecb->palEcb.i2cDevicePath, O_RDWR);
                if ( dcb->fd < 0 )
                {
                    /* Error opening the I2C device */
                    dcb = A2B_NULL;
                }
                else
                {
                    if ( ioctl(dcb->fd, I2C_TENBIT,
                        (fmt == A2B_I2C_ADDR_FMT_7BIT) ? 0 : 1) < 0 )
                    {
                        close(dcb->fd);
                        dcb->fd = A2B_INVALID_FD;
                        dcb = A2B_NULL;
                    }
                    else
                    {
                        dcb->fmt = fmt;
                        dcb->inUse = A2B_TRUE;
                    }
                }
                break;
            }
        }
    }
    return dcb;
} /* pal_i2cOpen */

/*!****************************************************************************
*
*  \b              pal_i2cClose
*
*  <b> API Details: </b><br>
*  This routine is called to de-initialization the i2c subsystem
*  during the stack destroy process.
*                                                                       <br><br>
*  <b> Linux Implementation Details: </b><br>
*  If a Linux IOCTL I2C device was opened the handles are cleaned up.  Because
*  this is an IOCTL interface there is no need to close anything.
*                                                                           <br>
*  \param          [in]    hnd          The handle returned from pal_i2cOpen
*
*  \pre            None
*
*  \post           None
*
*  \return         A status code that can be checked with the A2B_SUCCEEDED()
*                  or A2B_FAILED() for success or failure.
*
******************************************************************************/
a2b_HResult
pal_i2cClose
    (
    a2b_Handle  hnd
    )
{
    a2b_HResult status = A2B_MAKE_HRESULT(A2B_SEV_FAILURE, A2B_FAC_I2C,
                                        A2B_EC_INVALID_PARAMETER);
    pal_I2cDcb* dcb;

    if ( A2B_NULL != hnd )
    {
        dcb = (pal_I2cDcb*)hnd;

        status = pal_convertIoctlError(close(dcb->fd));
        dcb->fd = A2B_INVALID_FD;
        dcb->inUse = A2B_FALSE;
    }
    return status;
} /* pal_i2cClose */


/*!****************************************************************************
*
*  \b              pal_i2cRead
*
*  <b> API Details: </b><br>
*  This routine reads bytes from an I2C device.
*                                                                       <br><br>
*  <b> Linux Implementation Details: </b><br>
*  The Linux IOCTL I2C device API is called to read the I2C device.
*                                                                           <br>
*  \param          [in]      hnd        The handle returned from pal_i2cOpen.
*                            
*  \param          [in]      addr       The I2C address.
*                            
*  \param          [in]      nRead      The number of bytes to read from
*                                       the device.
* 
*  \param          [in,out]  rBuf       A buffer in which to write the results
*                                       of the read.
*
*  \pre            None
*
*  \post           The read buffer holds the contents of the read on success.
*
*  \return         A status code that can be checked with the A2B_SUCCEEDED()
*                  or A2B_FAILED() for success or failure.
*
******************************************************************************/
a2b_HResult
pal_i2cRead
    (
    a2b_Handle  hnd,
    a2b_UInt16  addr,
    a2b_UInt16  nRead,
    a2b_Byte*   rBuf
    )
{
    struct i2c_rdwr_ioctl_data msgData;
    struct i2c_msg msg;
    a2b_HResult status = A2B_MAKE_HRESULT(A2B_SEV_FAILURE, A2B_FAC_I2C,
                                        A2B_EC_INVALID_PARAMETER);
    pal_I2cDcb* dcb;

    if ( A2B_NULL != hnd )
    {
        dcb = (pal_I2cDcb*)hnd;

        if ( (nRead == 0) || (rBuf != A2B_NULL) )
        {
            msg.addr = addr;
            msg.buf = rBuf;
            msg.len = nRead;
            msg.flags = ((dcb->fmt == A2B_I2C_ADDR_FMT_7BIT ) ? 0 : I2C_M_TEN);
            msg.flags |= I2C_M_RD;
            msgData.msgs = &msg;
            msgData.nmsgs = 1;
            status = pal_convertIoctlError(ioctl(dcb->fd, I2C_RDWR, &msgData));
        }
    }

    return status;
} /* pal_i2cRead */


/*!****************************************************************************
*
*  \b              pal_i2cWrite
*
*  <b> API Details: </b><br>
*  This routine writes bytes to an I2C device.
*                                                                       <br><br>
*  <b> Linux Implementation Details: </b><br>
*  The Linux IOCTL I2C device API is called to write to the I2C device.
*                                                                           <br>
*  \param          [in]    hnd          The handle returned from pal_i2cOpen
* 
*  \param          [in]    addr         The I2C address.
* 
*  \param          [in]    nWrite       The number of bytes to write to
*                                       the device.
* 
*  \param          [in]    wBuf         A buffer containing the data to write.
*                                       The buffer is of size 'nWrite' bytes.
*
*  \pre            None
*
*  \post           None
*
*  \return         A status code that can be checked with the A2B_SUCCEEDED()
*                  or A2B_FAILED() for success or failure.
*
******************************************************************************/
a2b_HResult
pal_i2cWrite
    (
    a2b_Handle      hnd,
    a2b_UInt16      addr,
    a2b_UInt16      nWrite,
    const a2b_Byte* wBuf
    )
{
    struct i2c_rdwr_ioctl_data msgData;
    struct i2c_msg msg;
    a2b_HResult status = A2B_MAKE_HRESULT(A2B_SEV_FAILURE, A2B_FAC_I2C,
                                        A2B_EC_INVALID_PARAMETER);
    pal_I2cDcb* dcb;

    if ( A2B_NULL != hnd )
    {
        dcb = (pal_I2cDcb*)hnd;

        if ( (nWrite == 0) || (wBuf != A2B_NULL) )
        {
            msg.addr = addr;
            msg.buf = (a2b_Byte*)wBuf;
            msg.len = nWrite;
            msg.flags = ((dcb->fmt == A2B_I2C_ADDR_FMT_7BIT ) ? 0 : I2C_M_TEN);
            msgData.msgs = &msg;
            msgData.nmsgs = 1;
            status = pal_convertIoctlError(ioctl(dcb->fd, I2C_RDWR, &msgData));
        }
    }

    return status;
} /* pal_i2cWrite */


/*!****************************************************************************
*
*  \b              pal_i2cWriteRead
*
*  <b> API Details: </b><br>
*  This routine writes and then reads bytes from the I2C device *without* an
*  I2C stop sequence separating the two operations. Instead a repeated I2C start
*  sequence is used as the operation separator.
*                                                                       <br><br>
*  <b> Linux Implementation Details: </b><br>
*  The Linux IOCTL I2C device API is called to write/read the I2C device.
*                                                                           <br>
*  \param          [in]    hnd          The handle returned from pal_i2cOpen
* 
*  \param          [in]    addr         The I2C address.
*                                                                                          
*  \param          [in]    nWrite       The number of bytes to write.
*                                                                                          
*  \param          [in]    wBuf         A buffer containing the data to write.
*                                       The buffer is of size 'nWrite' bytes.
*                                                                                          
*  \param          [in]    nRead        The number of bytes to read from
*                                       the device.
*                                                                                          
*  \param          [in]    rBuf         A buffer in which to write the results
*                                       of the read.
*
*  \pre            None
*
*  \post           The read buffer holds the contents of the read on success.
*
*  \return         A status code that can be checked with the A2B_SUCCEEDED()
*                  or A2B_FAILED() for success or failure.
*
******************************************************************************/
a2b_HResult
pal_i2cWriteRead
    (
    a2b_Handle      hnd,
    a2b_UInt16      addr,
    a2b_UInt16      nWrite,
    const a2b_Byte* wBuf,
    a2b_UInt16      nRead,
    a2b_Byte*       rBuf
    )
{
    struct i2c_rdwr_ioctl_data msgData;
    struct i2c_msg msgs[2];
    struct i2c_msg* pMsg = msgs;
    a2b_HResult status = A2B_MAKE_HRESULT(A2B_SEV_FAILURE, A2B_FAC_I2C,
                                        A2B_EC_INVALID_PARAMETER);
    pal_I2cDcb* dcb;

    if ( A2B_NULL != hnd )
    {
        dcb = (pal_I2cDcb*)hnd;
        msgData.msgs = msgs;
        msgData.nmsgs = 0;

        if ( (nWrite == 0) || (wBuf != A2B_NULL) )
        {
            pMsg->addr = addr;
            pMsg->buf = (a2b_Byte*)wBuf;
            pMsg->len = nWrite;
            pMsg->flags = ((dcb->fmt == A2B_I2C_ADDR_FMT_7BIT ) ?
                                0 : I2C_M_TEN);
            msgData.nmsgs += 1;
            pMsg += 1;

            status = A2B_RESULT_SUCCESS;
        }

        if ( A2B_SUCCEEDED(status) )
        {
            if ( (nRead > 0) && (rBuf == A2B_NULL) )
            {
                status = A2B_MAKE_HRESULT(A2B_SEV_FAILURE, A2B_FAC_I2C,
                                          A2B_EC_INVALID_PARAMETER);
            }
            /* Else nRead >= 0 */
            else
            {
                pMsg->addr = addr;
                pMsg->buf = rBuf;
                pMsg->len = nRead;
                pMsg->flags = ((dcb->fmt == A2B_I2C_ADDR_FMT_7BIT ) ?
                                    0 : I2C_M_TEN);
                pMsg->flags |= I2C_M_RD;
                msgData.nmsgs += 1;
            }
        }

        if ( A2B_SUCCEEDED(status) )
        {
            status = pal_convertIoctlError(ioctl(dcb->fd, I2C_RDWR, &msgData));
        }
    }

    return status;
} /* pal_i2cWriteRead */


/*!****************************************************************************
*
*  \b              pal_i2cShutdown
*
*  <b> API Details: </b><br>
*  This routine is called to shutdown the I2C subsystem
*  during the stack destroy process.  This routine is called immediately
*  after the pal_i2cClose (assuming the close was successful).
*                                                                       <br><br>
*  <b> Linux Implementation Details: </b><br>
*  Nothing to do, always returns A2B_RESULT_SUCCESS.
*                                                                           <br>
*  \param          [in]    ecb      The environment control block (ecb) for
*                                   this platform.
*
*  \pre            None
*
*  \post           None
*
*  \return         A status code that can be checked with the A2B_SUCCEEDED()
*                  or A2B_FAILED() for success or failure.
*
******************************************************************************/
a2b_HResult
pal_i2cShutdown
    (
    A2B_ECB*    ecb
    )
{
    A2B_UNUSED(ecb);
    return A2B_RESULT_SUCCESS;
} /* pal_i2cShutdown */

/** \} -- a2bstack_pal_i2c_ioctl */

