
/*=============================================================================
 *
 * Project: a2bstack
 *
 * Copyright (c) 2023 - Analog Devices Inc. All Rights Reserved.
 * This software is subject to the terms and conditions of the license set 
 * forth in the project LICENSE file. Downloading, reproducing, distributing or 
 * otherwise using the software constitutes acceptance of the license. The 
 * software may not be used except as expressly authorized under the license.
 *
 *=============================================================================
 *
 * \file:   pal_init.c
 * \author: Mentor Graphics, Embedded Software Division
 * \brief:  Implementation of the platform abstraction layer routines.
 *
 *=============================================================================
 */

/*============================================================================*/
/** 
 * \defgroup a2bstack_pal_init     Core PAL Init Module 
 *  
 * Implementation of the platform abstraction layer routines.
 *  
 * \{ */
/*============================================================================*/

/*======================= I N C L U D E S =========================*/

#include <stdlib.h>
#include <errno.h>
#include "a2b/util.h"
#include "a2b/conf.h"
#include "a2b/platform.h"
#include "a2b/stack.h"
#include "a2b/i2c.h"
#ifdef A2B_FEATURE_TRACE
#include "a2b/trace.h"
#endif
#include "pal_i2c.h"
#include "pal_log.h"
#include "pal_audio.h"
#include "pal_plugin.h"
#include "pal_timer.h"
#include "pal_info.h"

#ifndef A2B_FEATURE_MEMORY_MANAGER
#include "pal_memmgr.h"
#endif
#include "adi_a2b_externs.h"
#include "../app/a2bapp_defs.h"

/*======================= D E F I N E S ===========================*/

/*======================= D A T A T Y P E S =======================*/

/*======================= L O C A L  P R O T O T Y P E S  =========*/

/*======================= D A T A  ================================*/

/*======================= C O D E =================================*/

/*!****************************************************************************
*
*  \b              a2b_palInit
* 
*  <b> API Details: </b><br>
*  Initializes the Platform Abstraction Layer (PAL) structure [function
*  pointers] with defaults for the given platform. These defaults can be
*  over-ridden as needed if necessary.
*                                                                       <br><br>
*  <b> Linux Implementation Details: </b><br>
*  This sets up all the function pointers for core PAL functionality used
*  by the A2B stack.  If A2B_FEATURE_MEMORY_MANAGER is NOT defined then
*  Linux malloc/free is used instead of the stack memory manager.  This
*  also implements some environment variable support for Linux:             <br>
*  - A2B_CONF_TRACE_CHAN_URL                                                <br>
*  - A2B_CONF_TRACE_LEVEL                                                   <br>
*  - A2B_CONF_I2C_DEVICE_PATH                                               <br>
*  - A2B_CONF_PLUGIN_SEARCH_PATTERN                                         <br>
*                                                                           <br>
*  \param          [in]    pal      Pointer to the Platform Abstraction Layer to
*                                   initialize with defaults.
* 
*  \param          [in]    ecb      The environment control block (ecb) for
*                                   this platform.
*
*  \pre            None
*
*  \post           The pal function pointer API will be fully
*                  populated with default values.
*
*  \return         None
*
******************************************************************************/
void
a2b_palInit
    (
    struct a2b_StackPal*    pal,
    A2B_ECB*                ecb
    )
{
    a2b_Char* envValue;

    if ( A2B_NULL != pal )
    {
        a2b_memset(pal, 0, sizeof(*pal));
        a2b_memset(ecb, 0, sizeof(*ecb));

        /* Do necessary base initialization */
        a2b_stackPalInit(pal, ecb);

#ifndef A2B_FEATURE_MEMORY_MANAGER
        /* Use Linux malloc/free */
        pal->memMgrInit      = pal_memMgrInit;
        pal->memMgrOpen      = pal_memMgrOpen;
        pal->memMgrMalloc    = pal_memMgrMalloc;
        pal->memMgrFree      = pal_memMgrFree;
        pal->memMgrClose     = pal_memMgrClose;
        pal->memMgrShutdown  = pal_memMgrShutdown;
#endif

        pal->timerInit       = pal_timerInit;
        pal->timerGetSysTime = pal_timerGetSysTime;
        pal->timerShutdown   = pal_timerShutdown;

        pal->logInit         = pal_logInit;
        pal->logOpen         = pal_logOpen;
        pal->logClose        = pal_logClose;
        pal->logWrite        = pal_logWrite;
        pal->logShutdown     = pal_logShutdown;

        pal->i2cInit         = pal_i2cInit;
        pal->i2cOpen         = pal_i2cOpen;
        pal->i2cClose        = pal_i2cClose;
        pal->i2cRead         = pal_i2cRead;
        pal->i2cWrite        = pal_i2cWrite;
        pal->i2cWriteRead    = pal_i2cWriteRead;
        pal->i2cShutdown     = pal_i2cShutdown;

#ifdef A2B_ALSA_AUDIO
        pal->audioInit       = pal_audioInit;
        pal->audioOpen       = pal_audioOpen;
        pal->audioClose      = pal_audioClose;
        pal->audioConfig     = pal_audioConfig;
        pal->audioShutdown   = pal_audioShutdown;
#endif
	    pal->pluginsLoad     = a2bapp_pluginsLoad;
	    pal->pluginsUnload   = a2bapp_pluginsUnload;

        pal->getVersion      = pal_infoGetVersion;
        pal->getBuild        = pal_infoGetBuild;

        if ( A2B_NULL != ecb )
        {
            ecb->baseEcb.i2cAddrFmt    = A2B_I2C_ADDR_FMT_7BIT;
            ecb->baseEcb.i2cBusSpeed   = A2B_I2C_BUS_SPEED_100KHZ;
            ecb->baseEcb.i2cMasterAddr = A2B_CONF_DEFAULT_MASTER_NODE_I2C_ADDR;

            envValue = getenv("A2B_CONF_TRACE_CHAN_URL");
            if ( A2B_NULL != envValue )
            {
                ecb->baseEcb.traceUrl = envValue;
            }
            else
            {
                ecb->baseEcb.traceUrl = A2B_CONF_DEFAULT_TRACE_CHAN_URL;
            }

#ifdef A2B_FEATURE_TRACE
            envValue = getenv("A2B_CONF_TRACE_LEVEL");
            if ( A2B_NULL != envValue )
            {
                a2b_Char*  endptr;

                ecb->baseEcb.traceLvl = (a2b_UInt32)strtoul(envValue,
                                                           &endptr, 16);

               /* Check for various possible errors */
               if ((errno == ERANGE) || 
                   (errno != 0 && ecb->baseEcb.traceLvl == 0) ||
                   (endptr == envValue))
               {
                   /* strtol error | no digits found */
                   envValue = A2B_NULL;
               }
            }

            if ( A2B_NULL == envValue )
            {
                ecb->baseEcb.traceLvl = A2B_CONF_DEFAULT_TRACE_LVL;
            }
#endif

#ifdef A2B_I2C_ADAPTER_IOCTL
            envValue = getenv("A2B_CONF_I2C_DEVICE_PATH");
            if ( A2B_NULL != envValue )
            {
                ecb->palEcb.i2cDevicePath = envValue;
            }
            else
            {
                ecb->palEcb.i2cDevicePath = A2B_CONF_DEFAULT_I2C_DEVICE_PATH;
            }
#elif defined(A2B_I2C_ADAPTER_AARDVARK)
            ecb->palEcb.useFirstAvailable = A2B_TRUE;
            ecb->palEcb.adapterId   = 0;
            ecb->palEcb.lockTimeout = A2B_CONF_DEFAULT_AARDVARK_LOCK_TIMEOUT;
#elif defined(A2B_I2C_ADAPTER_USBI)
            ecb->palEcb.useFirstAvailable = A2B_TRUE;
            ecb->palEcb.busNum  = 0;
            ecb->palEcb.devAddr = 0;
            ecb->palEcb.usbCtx  = A2B_NULL;
            ecb->palEcb.xferTimeout = A2B_CONF_DEFAULT_USBI_XFER_TIMEOUT;
#endif
            envValue = getenv("A2B_CONF_PLUGIN_SEARCH_PATTERN");
            if ( A2B_NULL != envValue )
            {
                ecb->palEcb.pluginSearchPattern = envValue;
            }
            else
            {
                ecb->palEcb.pluginSearchPattern =
                                    A2B_CONF_DEFAULT_PLUGIN_SEARCH_PATTERN;
            }
        }
    }
} /* a2b_palInit */

/** \} -- a2bstack_pal_init */

