/*=============================================================================
 *
 * Project: a2bstack
 *
 * Copyright (c) 2015 - Analog Devices Inc. All Rights Reserved.
 * This software is subject to the terms and conditions of the license set 
 * forth in the project LICENSE file. Downloading, reproducing, distributing or 
 * otherwise using the software constitutes acceptance of the license. The 
 * software may not be used except as expressly authorized under the license.
 *
 *=============================================================================
 *
 * \file:   pal_memmgr.c
 * \author: Mentor Graphics, Embedded Software Division
 * \brief:  This the PAL implementation of the memory management service.
 *          This implementation is a direct mapping to Linux malloc/free
 *          only when A2B_FEATURE_MEMORY_MANAGER is NOT enabled.
 *
 *=============================================================================
 */

/*======================= I N C L U D E S =========================*/

#include <stdlib.h>

#include "a2b/conf.h"
#include "a2b/error.h"
#include "pal_memmgr.h"

/* A2B_FEATURE_MEMORY_MANAGER_DOXY is only to allow doxygen to document the code */
#if !defined(A2B_FEATURE_MEMORY_MANAGER) || defined(A2B_FEATURE_MEMORY_MANAGER_DOXY)

/*======================= D E F I N E S ===========================*/


/*======================= L O C A L  P R O T O T Y P E S  =========*/


/*======================= D A T A  ================================*/

static a2b_UInt32   gHeapPtr = 0x68656170;

/*======================= C O D E =================================*/


/*!****************************************************************************
*
*  \b              pal_memMgrInit
* 
*  <b> API Details: </b><br>
*  This routine is called to do initialization required by the memory manager 
*  service during the stack [private] allocation process.
*                                                                       <br><br>
*  <b> Linux Implementation Details: </b><br>
*  Nothing to do, always returns A2B_RESULT_SUCCESS
*                                                                           <br>
*  \param          [in]    ecb      The environment control block (ecb) for
*                                   this platform.
*
*  \pre            None
*
*  \post           None
*
*  \return         A status code that can be checked with the A2B_SUCCEEDED()
*                  or A2B_FAILED() for success or failure.
*
******************************************************************************/
a2b_HResult
pal_memMgrInit
    (
    A2B_ECB*    ecb
    )
{
    A2B_UNUSED( ecb );
    return A2B_RESULT_SUCCESS;
} /* pal_memMgrInit */


/*!****************************************************************************
*
*  \b              pal_memMgrOpen
* 
*  <b> API Details: </b><br>
*  This routine opens a memory managed heap located at the specified address 
*  and of the specified size. If the A2B stack's heap cannot be opened and 
*  managed at the specified location (perhaps because the size is insufficient)
*  then the returned handle will be A2B_NULL. The managed heap will use
*  memory pools to avoid fragmentation within the managed region.
*                                                                       <br><br>
*  <b> Linux Implementation Details: </b><br>
*  Nothing to do, always returns non-NULL
*                                                                           <br>
*  \param          [in]    heap         The memory address of the start of the
*                                       managed region.
* 
*  \param          [in]    heapSize     The size of the managed region in bytes.
*
*  \pre            None
*
*  \post           None
*
*  \return         If non-zero the open was considered sucessfully.  If A2B_NULL
*                  I2C initialization must have failed and the stack
*                  allocation will fail.  This returned handle will be passed
*                  back into the other PAL I2C (pal_memMgrXXX) API calls.
*
******************************************************************************/
a2b_Handle
pal_memMgrOpen
    (
    a2b_Byte*   heap,
    a2b_UInt32  heapSize
    )
{
    A2B_UNUSED( heap );
    A2B_UNUSED( heapSize );

    /* MUST be a non-NULL value -- Dummy value */
    return (a2b_Handle)&gHeapPtr;
} /* pal_memMgrOpen */


/*!****************************************************************************
*
*  \b              pal_memMgrMalloc
* 
*  <b> API Details: </b><br>
*  This routine is called to get/allocate a fixed amount of memory.
*                                                                       <br><br>
*  <b> Linux Implementation Details: </b><br>
*  Directly calls the Linux malloc routine to allocate memory.
*                                                                           <br>
*  \param          [in]    hnd      The handle of the stack's heap from
*                                   pal_memMgrOpen.
* 
*  \param          [in]    size     The amount of memory (in bytes) to allocate 
*                                   from the stack's heap.
*
*  \pre            None
*
*  \post           None
*
*  \return         Returns an aligned pointer to memory or A2B_NULL if memory
*                  could not be allocated.
*
******************************************************************************/
void*
pal_memMgrMalloc
    (
    a2b_Handle  hnd,
    a2b_UInt32  size
    )
{
    A2B_UNUSED( hnd );
    return malloc( size );
} /* pal_memMgrMalloc */


/*!****************************************************************************
*
*  \b              pal_memMgrFree
* 
*  <b> API Details: </b><br>
*  This routine is called to free previously allocated memory.
*                                                                       <br><br>
*  <b> Linux Implementation Details: </b><br>
*  Directly calls the Linux free routine to free malloc'ed memory.
*                                                                           <br>
*  \param          [in]    hnd      The handle of the stack's heap from
*                                   pal_memMgrOpen.
* 
*  \param          [in]    p        The pointer to the memory to free.
*
*  \pre            None
*
*  \post           None
*
*  \return         None
*
******************************************************************************/
void
pal_memMgrFree
    (
    a2b_Handle  hnd,
    void*       p
    )
{
    A2B_UNUSED( hnd );
    free( p );
} /* pal_memMgrFree */


/*!****************************************************************************
*
*  \b              pal_memMgrClose
* 
*  <b> API Details: </b><br>
*  This routine is called to de-initialization the memory management subsystem
*  during the stack destroy process.  All resources associated with the
*  heap are freed.
*                                                                       <br><br>
*  <b> Linux Implementation Details: </b><br>
*  Nothing to do, always returns A2B_RESULT_SUCCESS
*                                                                           <br>
*  \param          [in]    hnd      The handle to the stack's heap.
*
*  \pre            None
*
*  \post           Memory should not be requested from a stack heap that has
*                  been closed.
*
*  \return         A status code that can be checked with the A2B_SUCCEEDED()
*                  or A2B_FAILED() for success or failure.
*
******************************************************************************/
a2b_HResult
pal_memMgrClose
    (
    a2b_Handle  hnd
    )
{
    A2B_UNUSED( hnd );
    return A2B_RESULT_SUCCESS;
} /* pal_memMgrClose */


/*!****************************************************************************
*
*  \b              pal_memMgrShutdown
* 
*  <b> API Details: </b><br>
*  This routine is called to shutdown the memory manager subsystem
*  during the stack destroy process.  This routine is called immediately
*  after the pal_memMgrClose (assuming the close was successful).
*                                                                       <br><br>
*  <b> Linux Implementation Details: </b><br>
*  Nothing to do, always returns A2B_RESULT_SUCCESS.
*                                                                           <br>
*  \param          [in]    ecb      The environment control block (ecb) for
*                                   this platform.
*
*  \pre            None
*
*  \post           None
*
*  \return         A status code that can be checked with the A2B_SUCCEEDED()
*                  or A2B_FAILED() for success or failure.
*
******************************************************************************/
a2b_HResult
pal_memMgrShutdown
    (
    A2B_ECB*    ecb
    )
{
    A2B_UNUSED( ecb );
    return A2B_RESULT_SUCCESS;
} /* pal_memMgrShutdown */

#endif /* A2B_FEATURE_MEMORY_MANAGER */
