/*******************************************************************************
Copyright (c) 2021 - Analog Devices Inc. All Rights Reserved.
This software is proprietary & confidential to Analog Devices, Inc.
and its licensors.
*******************************************************************************

   Name       : a2bapp-linux.c

   Description: This is the implementation of a sample A2B application that
 *          utilizes the A2B Network Stack for the Linux platform.

   Functions  : main()
                a2bapp_resetTerm()
                a2bapp_onSignal()
                a2bapp_setRawMode()
                a2bapp_kbhit()
                a2bapp_getch()
				a2bapp_dumpRegisters()
				a2bapp_usage()
				a2bapp_keypressUsage()

   Prepared &
   Reviewed by: Automotive Software and Systems team,
                IPDC, Analog Devices,  Bangalore, India

   @version: $Revision: 66678 $
   @date: $Date: 2021-08-04 06:49:25 -0400 (Wed, 04 Aug 2021) $

******************************************************************************/


/*======================= I N C L U D E S =========================*/
#include "a2bapp_codec_init.h"
/* stddef.h *must* always be included before including stdio.h */
#include <stddef.h>

#include <stdio.h>
#include <stdint.h> /* for uint64_t type in protobuf */
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <termios.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include "a2b/a2b.h"
#include "a2b_bdd_helper.h"
#include "a2bplugin-master/plugin.h"
#include "a2bplugin-slave/plugin.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>
#include "a2bapp_defs.h"
#include "a2bapp.h"
#include "assert.h"
/*======================= D E F I N E S ===========================*/

/* Define an invalid command */
#define A2BAPP_INVALID_CMD      '\0'

/*======================= L O C A L  P R O T O T Y P E S  =========*/

/*======================= D A T A  ================================*/

static struct termios gOldTerm;
static struct termios gRawTerm;
static a2b_Bool gIsRaw = A2B_FALSE;
static a2b_Bool gDebug = A2B_FALSE;
a2b_App_t gApp_Info;
/*======================= C O D E =================================*/

/*!****************************************************************************
*
*  \b               a2bapp_resetTerm
*
*  Resets the terminal to the original configuration prior to the application
*  being run.
*
*  \pre             None
*
*  \post            None
*
*  \return          None
*
******************************************************************************/
static void
a2bapp_resetTerm(void)
{
    if ( gIsRaw )
    {
       tcsetattr( STDIN_FILENO, TCSANOW, &gOldTerm);
       gIsRaw = A2B_FALSE;
    }
}


/*!****************************************************************************
*
*  \b               a2bapp_onSignal
*
*  Signal handler for the application. Terminates the application when called.
*
*  \pre             None
*
*  \post            None
*
*  \return          None
*
******************************************************************************/
static void
a2bapp_onSignal
    (
    int sig
    )
{
    A2B_UNUSED(sig);
    exit(0);
}


/*!****************************************************************************
*
*  \b               a2bapp_setRawMode
*
*  Sets the terminal in raw mode while saving the previous settings.
*
*  \pre             None
*
*  \post            None
*
*  \return          None
*
******************************************************************************/
static void
a2bapp_setRawMode(void)
{
    if ( gIsRaw )
    {
        return;
    }

    tcgetattr( STDIN_FILENO, &gOldTerm);
    gRawTerm = gOldTerm;
    gRawTerm.c_lflag &= ~( ICANON | ECHO );
    tcsetattr( STDIN_FILENO, TCSANOW, &gRawTerm);

    gIsRaw = A2B_TRUE;
    atexit(a2bapp_resetTerm);
}


/*!****************************************************************************
*
*  \b               a2bapp_kbhit
*
*  Checks to see whether or not a keypress has been detected.
*
*  \pre             None
*
*  \post            None
*
*  \return          Returns a non-zero value if a keypress has been detected
*                   and zero otherwise.
*
******************************************************************************/
static int
a2bapp_kbhit(void)
{
    struct timeval tv;
    fd_set rdfs;

    tv.tv_sec = 0;
    tv.tv_usec = 0;

    a2bapp_setRawMode();

    FD_ZERO(&rdfs);
    FD_SET (STDIN_FILENO, &rdfs);

    select(STDIN_FILENO+1, &rdfs, NULL, NULL, &tv);
    return FD_ISSET(STDIN_FILENO, &rdfs);
}


/*!****************************************************************************
*
*  \b               a2bapp_getch
*
*  Returns the keypress (character) read from stdin.
*
*  \pre             None
*
*  \post            None
*
*  \return          Returns the character read or zero (0) if there was an
*                   error.
*
******************************************************************************/
static int
a2bapp_getch(void)
{
    unsigned char temp;

    a2bapp_setRawMode();
    if ( read(STDIN_FILENO, &temp, 1) != 1 )
    {
        return 0;
    }
    return temp;
}


/*!****************************************************************************
*
*  \b               a2bapp_dumpRegisters
*
*  Dumps all the registers of the selected AD24xx node.
*
*  \pre             None
*
*  \post            None
*
*  \return          None
*
******************************************************************************/
static void
a2bapp_dumpRegisters
    (
    struct a2b_StackContext*    ctx,
    a2b_Int16                   nodeAddr
    )
{
    a2b_UInt8   regs[A2B_REG_MBOX1B3+1];
    a2b_UInt16  numRegs = A2B_ARRAY_SIZE(regs);
    a2b_UInt16  idx;
    a2b_HResult status = a2b_diagGetRegDump(ctx, nodeAddr, 0, regs, &numRegs);
    if ( A2B_FAILED(status) )
    {
        fprintf(stderr, "Failed to dump AD24xx registers: ErrCode = 0x%X\n",
                status);
    }
    else
    {
        printf("\n==================================\n");
        printf("AD24xx Register Dump (NodeAddr=%d)\n", nodeAddr);
        printf("==================================\n");
        for( idx = 0; idx < numRegs; ++idx )
        {
            printf("0x%02X => 0x%02X\n", idx, regs[idx]);
        }
    }
}


/*!****************************************************************************
*
*  \b               a2bapp_usage
*
*  Prints out program usage/help information.
*
*  \pre             None
*
*  \post            None
*
*  \return          None
*
******************************************************************************/
static void
a2bapp_usage()
{
    printf("Usage:\n");
    printf("\ta2bapp-linux [-h]  [-d] [-D]" \
            "[-s seq_chart_file] [-v] [-k]\n");
    printf("\n");
    printf("\tWhere:\n");
    printf("\t\t-h             Shows this help message.\n");
    printf("\t\t-d             Turns on diagnostic output (off by\n");
    printf("\t\t               default).\n");
    printf("\t\t-D             Daemonize the application.\n");
    printf("\t\t-s <file>      Creates a sequence chart and saves it\n");
    printf("\t\t               in the specified file.\n");
    printf("\t\t-v             Turn on verbose output.\n");
	printf("\t\t-k             Enables audio via ALSA, if A2B netwrok is already discoverd.\n");
}


/*!****************************************************************************
*
*  \b               a2bapp_keypressUsage
*
*  Prints out helpful keypress information.
*
*  \pre             None
*
*  \post            None
*
*  \return          None
*
******************************************************************************/
static void
a2bapp_keypressUsage()
{
    printf("\n======================\n");
    printf("Recognized keypresses:\n");
    printf("======================\n");
    printf("d     Restart node discovery.\n");
    printf("h     Displays this help menu.\n");	
    printf("q     Quits the application.\n");
    printf("r     Dumps the AD242x registers of the selected slave node.\n");
    printf("R     Dumps the AD242x registers of the master node.\n");
    printf("0-9   Selects the slave node address to receive commands.\n");
}
/*!****************************************************************************
*
*  \b               main
*
*  The main entry point for the A2B sample application
*
*  \param           [in]    argc    The number of command-line arguments.
*
*  \param           [in]    argv    An array of command-line arguments.
*
*  \pre             None
*
*  \post            None
*
*  \return          Returns EXIT_SUCCESS(0) on success or EXIT_FAILURE(1)
*                   on failure.
*
******************************************************************************/
int
main
    (
    int     argc,
    char*   argv[]
    )
{
    /* Using non-customized (generic) ECB */
    a2b_Bool                    verbose = A2B_FALSE;
    a2b_UInt32                  ticks = 0;
    a2b_Bool                    quit = A2B_FALSE;
    a2b_Bool                    bEnAudioFlow = A2B_FALSE;
    a2b_UInt32                  discoveryCount = 0;
    a2b_Char                    c, bAudioRestart = 0;
    a2b_Int32                   opt;
    a2b_HResult                 result;
    a2b_Int16					slaveNodeAddr = 0;
    a2b_Bool                    daemonize = A2B_FALSE;
    const a2b_Char*             optFmt = "hb:dDs:vk";
    a2b_UInt32 nResult;
    void* i2c_dcb;

    /* Configure stdout for non-buffered mode */
    setvbuf(stdout, NULL, _IONBF, 0);
    /*
     * Parse the command-line options
     */
    while ( (opt = getopt(argc, argv, optFmt)) != -1 )
    {
        switch ( opt )
        {
            case 'd':
            	gApp_Info.bDebug = A2B_TRUE;
            	break;

            case 'D':
                daemonize = A2B_TRUE;
                break;

            case 'h':
                a2bapp_usage();
                exit(EXIT_SUCCESS);
                break;

            case 's':
#ifdef A2B_FEATURE_SEQ_CHART
            	gApp_Info.seqFile = optarg;
#endif
                break;

            case 'v':
                verbose = A2B_TRUE;
                break;

            case 'k':
#ifdef A2B_ALSA_AUDIO
            	/* Enables audio via ALSA, if A2B netwrok is already discoverd */
				pal_audioOpen();
				pal_audioConfig(0,0);
				a2b_audioStart();

				while(1)
				{
					a2b_audioTick();
				}
#endif
				break;

            default:
                fprintf(stderr, "Unknown option: '%c'\n\n", (a2b_Char)opt);
                a2bapp_usage();
                exit(EXIT_FAILURE);
                break;
        }
    }

    /* See if this application should run as a daemon */
    if ( daemonize )
    {
        if ( daemon(0,0) < 0 )
        {
            perror("Failed to daemonize");
            exit(EXIT_FAILURE);
        }
    }

    signal(SIGINT, a2bapp_onSignal);
    /*since it is not recommended to share same I2C device between two different core,
    ADC/DAC switch configuration and initialization is done in ARM core*/
    i2c_dcb = ADC_DAC_I2C_OPEN();
    nResult = SWITCH_CONFIG(i2c_dcb);
    nResult = ADAU_1962_init(i2c_dcb);  
    nResult = ADAU_1979_init(i2c_dcb);
    nResult = ADC_DAC_I2C_CLOSE(i2c_dcb);
    printf("Start A2B setup... \n");
    nResult = a2b_setup(&gApp_Info);

    if(nResult != 0)
    {
    	printf("Failed to set up bus \n");
    }
    else
    {
    	printf("Success!!! \r\n");
#ifdef A2B_ALSA_AUDIO
    	printf("Start ALSA... \r\n");
    	a2b_audioStart();
#endif

    }
	
	/* Runs until the user says quit */
    while ( !quit )
    {
    	bAudioRestart = 0;
    	if((gApp_Info.bRetry == A2B_TRUE) && (gApp_Info.bfaultDone == A2B_TRUE))
		{
			bAudioRestart = 1;
		}

		/* Monitor a2b network for faults and initiate re-discovery if enabled */
		nResult = a2b_fault_monitor(&gApp_Info);

		if (gApp_Info.discoveryDone == A2B_TRUE)
		{
#ifdef A2B_ALSA_AUDIO
			if((bAudioRestart == 1) && (nResult == 0))
			{
				bAudioRestart = 0;
				printf("Restart ALSA... \r\n");
				a2b_audioStart();
			}

			a2b_audioTick();
#endif
		}

		a2b_stackTick(gApp_Info.ctx);

        c = A2BAPP_INVALID_CMD;

        if ( !daemonize )
        {
            if ( a2bapp_kbhit() )
            {
                c = a2bapp_getch();
			}
		}
        /* If there is a command to process */
        if ( A2BAPP_INVALID_CMD != c )
        {
            switch (c)
            {
                case 'q':
                    quit = A2B_TRUE;
                    break;
                case 'd':
                    if (gApp_Info.discoveryDone == A2B_TRUE)
                    {
                    	printf("Redisocver A2B... \n");

                    	nResult = a2b_stop(&gApp_Info);
                        nResult = a2b_setup(&gApp_Info);

                        if(nResult != 0)
                        {
                        	printf("Failed to discover \n");
                        }
                        else
                        {
#ifdef A2B_ALSA_AUDIO
                        	printf("Restart ALSA... \r\n");
                        	a2b_audioStart();
#endif

                        }
                    }
                    break;

                case 'h':
                    a2bapp_keypressUsage();
                    break;

                case 'r':
                    a2bapp_dumpRegisters(gApp_Info.ctx, slaveNodeAddr);
                    break;

                case 'R':
                    a2bapp_dumpRegisters(gApp_Info.ctx, A2B_NODEADDR_MASTER);
                    break;

                case '0':    /* Fall through */
                case '1':
                case '2':
                case '3':
                case '4':
                case '5':
                case '6':
                case '7':
                case '8':
                case '9':
                    if ( gApp_Info.nodesDiscovered > (a2b_UInt32)(c - '0') )
                    {
                        slaveNodeAddr = c -'0';
                        if ( !daemonize )
                        {
                            fprintf(stdout, "Current slave node address: %d\n",
                                    slaveNodeAddr);
                        }
                    }
                    else
                    {
                        fprintf(stderr, "Slave node address does not exist.\n");
                    }
                    break;

                default:
                    fprintf(stderr, "Unknown command: %c\n", c);
                    break;
            }
        }

        ticks++;
    }

    return EXIT_SUCCESS;

}
