/*******************************************************************************
Copyright (c) 2017 - Analog Devices Inc. All Rights Reserved.
This software is proprietary & confidential to Analog Devices, Inc.
and its licensors.
*******************************************************************************

   Name       : a2bapp_audiorouter.c

   Description: This file contains the implementation of a2b system initialization and Audio Routing in SHARC core

   Prepared &
   Reviewed by: Automotive Software and Systems team,
                IPDC, Analog Devices,  Bangalore, India

   @version: $Revision: 66678 $
   @date: $Date: 2021-08-04 06:49:25 -0400 (Wed, 04 Aug 2021) $

******************************************************************************/

#include <a2bapp-audiorouter.h>
#include <sys/platform.h>
#include <sys/adi_core.h>
#include "adi_initialize.h"


/*! Maximum number application sub buffers */
#define A2B_MAX_APPBUFFER			(4u)

/*! Use on board codec for audio */
#define A2B_USE_CODEC				(1u)

/*! Bi directional SPORT to be used to communicate with AD2410 */
#define A2B_AD24xx_RXSPORT			(0u)
#define A2B_AD24xx_TXSPORT			(1u)

/*! Bi directional SPORT to be used to communicate with AD2410 */
#define A2B_CODEC_RXSPORT				(2u)
#define A2B_CODEC_TXSPORT				(3u)

typedef struct
{
	ADI_A2B_SCOMM_HANDLER oAudioCommHandler;
	ADI_SPORT_PERI_CONFIG oAD24xxRxSportConfig;
	ADI_SPORT_PERI_CONFIG oCodecRxSportConfig;
	ADI_SPORT_PERI_CONFIG oAD24xxTxSportConfig;
	ADI_SPORT_PERI_CONFIG oCodecTxSportConfig;
}adi_a2b_audio;

ADI_MEM_A2B_DATA_CRIT_CAHE
uint8 aSysMemBuf[ADI_A2B_SYS_MEMSIZE_BYTES];

a2b_UInt32 nRxSportCnt=0,nTxSportCnt=0 , nADCTxCnt=0,nADCRxCnt=0;

adi_a2b_audio goA2bAudio;
/*! Upstream data processing buffer   */
static a2b_UInt32 ganUpstreamProcessBuffer[A2B_MAX_APPBUFFER][MAX_NUM_INPUT_ANALOG_CHANNELS][SPORT_BUFFER_SIZE];
/*! ADC data processing buffer   */
static a2b_UInt32 ganADCProcessBuffer[A2B_MAX_APPBUFFER][MAX_NUM_INPUT_ANALOG_CHANNELS][SPORT_BUFFER_SIZE];
/*
** Function Prototype section
*/

static ADI_SPORT_RESULT adi_a2b_EnableAudioHost(void);
a2b_HResult a2b_pal_AudioInitFunc(void);
a2b_HResult a2b_pal_AudioConfigFunc(void);
a2b_HResult adi_a2b_SystemInit(void);

static void adi_a2b_DACTxCallback(void* pCommHandle, a2b_UInt32* pTxBufferPtr, a2b_UInt32 nChannel);
static void adi_a2b_ADCRxCallback(void* pCommHandle, a2b_UInt32* pRxBufferPtr, a2b_UInt32 nChannel);
static void adi_a2b_DownStreamTxCallback(void* pCommHandle, a2b_UInt32* pTxBufferPtr, a2b_UInt32 nChannel);
static void adi_a2b_UpstreamRxCallback(void* pCommHandle, a2b_UInt32 pRxBufferPtr[], a2b_UInt32 nChannel);

static void adi_a2b_upstreamToDwnRouting(ADI_A2B_SCOMM_HANDLER* pAudioRoutingHandle, a2b_UInt32 anDwnstreamBuffer[], a2b_UInt32 anUpstreamBuffer[][SPORT_BUFFER_SIZE], a2b_UInt32 nChannel );
#if A2B_USE_CODEC
static void adi_a2b_ADCToDwnRouting(ADI_A2B_SCOMM_HANDLER* pAudioRoutingHandle, a2b_UInt32 anDwnstreamBuffer[], a2b_UInt32 anADCBuffer[][SPORT_BUFFER_SIZE], a2b_UInt32 nChannel );
static void adi_a2b_UpstreamToDACRouting(ADI_A2B_SCOMM_HANDLER* pAudioRoutingHandle, a2b_UInt32 anDACBuffer[], a2b_UInt32 anUpstreamBuffer[][SPORT_BUFFER_SIZE], a2b_UInt32 nChannel );
static void adi_a2b_ADCToDACRouting(ADI_A2B_SCOMM_HANDLER* pAudioRoutingHandle, a2b_UInt32 anDACBuffer[], a2b_UInt32 anADCBuffer[][SPORT_BUFFER_SIZE], a2b_UInt32 nChannel);
#endif
static void adi_a2b_DeInterleaveCopy( a2b_UInt32 anDesbuffer[][SPORT_BUFFER_SIZE], a2b_UInt32 anSrcBuffer[], a2b_UInt32 nChannel );

int main(int argc, char *argv[])
{
	a2b_HResult nRet;
	/**
	 * Initialize managed drivers and/or services that have been added to 
	 * the project.
	 * @return zero on success 
	 */
	adi_initComponents();

	nRet=adi_a2b_SystemInit();

	/* Initialize the CODEC SPORT Interfaces */
	nRet=a2b_pal_AudioInitFunc();

	/* Initialize the A2B SPORT Interfaces */
	nRet=a2b_pal_AudioConfigFunc();

	/* Endless Loop */
	while(1)
	{
		asm("nop;");
	}

}

/*****************************************************************************/
/*!
@brief  This API Initializes the audio sub-system.

@param [in]:ecb  - PAL ECB structure.


@return Return code
        -1: Failure
        -0: Success

\note I2C and TWI terms are used  interchangeably

*/
/*****************************************************************************/
ADI_MEM_A2B_CODE_NO_CRIT
a2b_HResult a2b_pal_AudioInitFunc(void)
{
	ADI_SPORT_RESULT 		eResult= ADI_SPORT_SUCCESS;
	ADI_A2B_SCOMM_HANDLER 	*pAudioCommHandler;
	ADI_SPORT_PERI_CONFIG 	*pCodecRxSportConfig;
	ADI_SPORT_PERI_CONFIG 	*pCodecTxSportConfig;

	pAudioCommHandler 		= &goA2bAudio.oAudioCommHandler;
	pCodecRxSportConfig		= &goA2bAudio.oCodecRxSportConfig;
	pCodecTxSportConfig		= &goA2bAudio.oCodecTxSportConfig;

	/* Initialization */
	pAudioCommHandler->pRoutingTable 			= gaAudioRoutingtab;
	pAudioCommHandler->nWordLength 				= 32u;
	pAudioCommHandler->nUpstrProcWriteIndex 	= 0u;
	pAudioCommHandler->nADCProcWriteIndex 		= 0u;
	pAudioCommHandler->nUpstrToDwnstrReadIndex 	= 1u;
	pAudioCommHandler->nADCToDwnstrReadIndex 	= 2u;
	pAudioCommHandler->nADCtoDACReadIndex 		= 1u;
	pAudioCommHandler->nUpStrtoDACReadIndex 	= 2u;


	/* Opened as pair, bidirectional format */
	pAudioCommHandler->nCodecTDMSize = 8u;

	pCodecRxSportConfig->nMultChDelay  			= 1;
	pCodecRxSportConfig->bActiveLowFrameSync 	= 0;
	pCodecRxSportConfig->nSamplingRisingClkEdge = 0u;
	pCodecRxSportConfig->eSportNum 				= ADI_A2B_HAL_SPORT_4;
	pCodecRxSportConfig->nTDMCh 				= pAudioCommHandler->nCodecTDMSize;
	pCodecRxSportConfig->eDirection 			= ADI_SPORT_DIR_RX;
	pCodecRxSportConfig->pfSPORTCallBack 		= (ADI_A2B_SPORT_CB)&adi_a2b_ADCRxCallback;
	pCodecRxSportConfig->nStChnlNo 				= 0;
	pCodecRxSportConfig->nEndChnlNo 			= 7U;
	pCodecRxSportConfig->eSportHalf 			= ADI_HALF_SPORT_A;

	pCodecTxSportConfig->nMultChDelay  			= 1;
	pCodecTxSportConfig->bActiveLowFrameSync 	= 0;
	pCodecTxSportConfig->nSamplingRisingClkEdge = 0u;
	pCodecTxSportConfig->eSportNum 				= ADI_A2B_HAL_SPORT_5;
	pCodecTxSportConfig->nTDMCh 				= pAudioCommHandler->nCodecTDMSize;
	pCodecTxSportConfig->eDirection 			= ADI_SPORT_DIR_TX;
	pCodecTxSportConfig->pfSPORTCallBack 		= (ADI_A2B_SPORT_CB)&adi_a2b_DACTxCallback;
	pCodecTxSportConfig->nStChnlNo 				= 0;
	pCodecTxSportConfig->nEndChnlNo 			= 7U;
	pCodecTxSportConfig->eSportHalf 			= ADI_HALF_SPORT_A;

    /* Configure Codec SPORT */
    adi_a2b_InitPCGForCodec();
    eResult = adi_a2b_SerialPortOpen(A2B_CODEC_RXSPORT, pCodecRxSportConfig, (void*)pAudioCommHandler);
    eResult = adi_a2b_SerialPortOpen(A2B_CODEC_TXSPORT, pCodecTxSportConfig, (void*)pAudioCommHandler);



	return (a2b_UInt32)eResult;
}

/*****************************************************************************/
/*!
@brief  This API Initializes the audio sub-system.

@param [in]:ecb  - PAL ECB structure.


@return Return code
        -1: Failure
        -0: Success

\note I2C and TWI terms are used  interchangeably

*/
/*****************************************************************************/
ADI_MEM_A2B_CODE_NO_CRIT
a2b_Handle a2b_pal_AudioOpenFunc(void)
{
	return ((a2b_Handle*)&goA2bAudio);
}

/*****************************************************************************/
/*!
@brief  This API configures the audio sub-system based on the Master
        TDM settigns

@param [in]:ecb  - PAL ECB structure.


@return Return code
        -1: Failure
        -0: Success

\note I2C and TWI terms are used  interchangeably

*/
/*****************************************************************************/
ADI_MEM_A2B_CODE_NO_CRIT
a2b_HResult a2b_pal_AudioConfigFunc(void)
{
	ADI_SPORT_RESULT 		eResult;
	ADI_A2B_SCOMM_HANDLER 	*pAudioCommHandler;
	ADI_SPORT_PERI_CONFIG 	*pAD24xxRxSportConfig;
	ADI_SPORT_PERI_CONFIG 	*pAD24xxTxSportConfig;

	pAudioCommHandler 		= &goA2bAudio.oAudioCommHandler;
	pAD24xxRxSportConfig	= &goA2bAudio.oAD24xxRxSportConfig;
	pAD24xxTxSportConfig	= &goA2bAudio.oAD24xxTxSportConfig;

	/* Get TDM size */
		pAudioCommHandler->nAD2410TDMSize = 8;

	/* Configuration */
	pAD24xxRxSportConfig->nMultChDelay  			= 1;
	pAD24xxRxSportConfig->bActiveLowFrameSync 		= 1U;
	pAD24xxRxSportConfig->nSamplingRisingClkEdge 	= 0U;
	pAD24xxRxSportConfig->eSportNum 				= ADI_A2B_HAL_SPORT_0;
	pAD24xxRxSportConfig->eDirection 				= ADI_SPORT_DIR_RX;
	pAD24xxRxSportConfig->nTDMCh 					= pAudioCommHandler->nAD2410TDMSize;
	pAD24xxRxSportConfig->pfSPORTCallBack 			= (ADI_A2B_SPORT_CB)&adi_a2b_UpstreamRxCallback;
	pAD24xxRxSportConfig->nStChnlNo 				= 0;
	pAD24xxRxSportConfig->nEndChnlNo 				= (pAD24xxRxSportConfig->nTDMCh - 1);
	pAD24xxRxSportConfig->eSportHalf 				= ADI_HALF_SPORT_A;

	pAD24xxTxSportConfig->nMultChDelay  			= 1;
	pAD24xxTxSportConfig->bActiveLowFrameSync 		= 1U;
	pAD24xxTxSportConfig->nSamplingRisingClkEdge 	= 0U;
	pAD24xxTxSportConfig->eSportNum 				= ADI_A2B_HAL_SPORT_0;
	pAD24xxTxSportConfig->eDirection 				= ADI_SPORT_DIR_TX;
	pAD24xxTxSportConfig->nTDMCh 					= pAudioCommHandler->nAD2410TDMSize;
	pAD24xxTxSportConfig->pfSPORTCallBack 			= (ADI_A2B_SPORT_CB)&adi_a2b_DownStreamTxCallback;
	pAD24xxTxSportConfig->nStChnlNo 				= 0;
	pAD24xxTxSportConfig->nEndChnlNo 				= (pAD24xxTxSportConfig->nTDMCh - 1);
	pAD24xxTxSportConfig->eSportHalf 				= ADI_HALF_SPORT_B;

	/* Configure AD2410 SPORT */
    adi_a2b_InitPCGForAD24xx(pAudioCommHandler->nAD2410TDMSize);

	eResult = adi_a2b_SerialPortOpen(A2B_AD24xx_RXSPORT, pAD24xxRxSportConfig, (void*)pAudioCommHandler);
	if(eResult != ADI_SPORT_SUCCESS)
	{
		return (eResult);
	}

	eResult = adi_a2b_SerialPortOpen(A2B_AD24xx_TXSPORT, pAD24xxTxSportConfig, (void*)pAudioCommHandler);
	if(eResult != ADI_SPORT_SUCCESS)
	{
		return (eResult);
	}

	eResult = adi_a2b_EnableAudioHost();

	return (a2b_UInt32)eResult;
}

/****************************************************************************/
/*!
    @brief			This function triggers audio data routing

    @return         void

*/
/********************************************************************************/
static ADI_SPORT_RESULT adi_a2b_EnableAudioHost(void)
{
	ADI_SPORT_RESULT eResult;

	/* Start Upstream reception */
	eResult = adi_a2b_SerialPortEnable(A2B_AD24xx_TXSPORT, true);
	/* Start Downstream transmission */
	eResult = adi_a2b_SerialPortEnable(A2B_AD24xx_RXSPORT, true);

#if A2B_USE_CODEC
	/* Start ADC */
	eResult = adi_a2b_SerialPortEnable(A2B_CODEC_RXSPORT, true);
	/* Start DAC */
	eResult = adi_a2b_SerialPortEnable(A2B_CODEC_TXSPORT, true);

#endif

	return(eResult);

}

/*****************************************************************************/
/*!
@brief  This API Initializes the audio sub-system.

@param [in]:ecb  - PAL ECB structure.


@return Return code
        -1: Failure
        -0: Success

\note I2C and TWI terms are used  interchangeably

*/
/*****************************************************************************/
ADI_MEM_A2B_CODE_NO_CRIT
a2b_HResult a2b_pal_AudioCloseFunc()
{

	ADI_SPORT_RESULT eResult;

	adi_a2b_DeInitPCGForCodec();
	adi_a2b_DeInitPCGForAD24xx();

#if A2B_USE_CODEC
	eResult = adi_a2b_sport_Close(A2B_CODEC_RXSPORT);
	eResult = adi_a2b_sport_Close(A2B_CODEC_TXSPORT);
#endif
	eResult = adi_a2b_sport_Close(A2B_AD24xx_TXSPORT);
	eResult = adi_a2b_sport_Close(A2B_AD24xx_RXSPORT);

	return ((a2b_UInt32)eResult);
}

/*****************************************************************************/
/*!
@brief  This API Initializes the audio sub-system.

@param [in]:ecb  - PAL ECB structure.


@return Return code
        -1: Failure
        -0: Success

\note I2C and TWI terms are used  interchangeably

*/
/*****************************************************************************/
ADI_MEM_A2B_CODE_NO_CRIT
a2b_HResult a2b_pal_AudioShutdownFunc()
{

	return 1;
}

/****************************************************************************/
/*!
    @brief			Callback function for upstream data reception

    @param [in]     pTxBufferPtr    Transmission sub buffer pointer.
    @param [in]     nChannel     	TDM size.

    @return         void

    Note:
*/
/********************************************************************************/
static void adi_a2b_UpstreamRxCallback(void* pCommHandle,a2b_UInt32 pRxBufferPtr[], a2b_UInt32 nChannel)
{
	nRxSportCnt++;
	ADI_A2B_SCOMM_HANDLER* pAudioRoutingHandle = (ADI_A2B_SCOMM_HANDLER*)pCommHandle;

	adi_a2b_DeInterleaveCopy(ganUpstreamProcessBuffer[pAudioRoutingHandle->nUpstrProcWriteIndex], pRxBufferPtr, nChannel);
	pAudioRoutingHandle->nUpstrProcWriteIndex = (pAudioRoutingHandle->nUpstrProcWriteIndex + 1u) % A2B_MAX_APPBUFFER;
}

/****************************************************************************/
/*!
    @brief			Callback function

    @param [in]     pTxBufferPtr    Transmission sub buffer pointer.
    @param [in]     nChannel     	TDM size.

    @return         void

    Note:
*/
/********************************************************************************/
static void adi_a2b_DownStreamTxCallback(void* pCommHandle, a2b_UInt32* pTxBufferPtr , a2b_UInt32 nChannel)
{
	nTxSportCnt++;
	ADI_A2B_SCOMM_HANDLER* pAudioRoutingHandle = (ADI_A2B_SCOMM_HANDLER*)pCommHandle;

	adi_a2b_upstreamToDwnRouting(pAudioRoutingHandle, pTxBufferPtr, ganUpstreamProcessBuffer[pAudioRoutingHandle->nUpstrToDwnstrReadIndex], nChannel);

#if A2B_USE_CODEC
	adi_a2b_ADCToDwnRouting(pAudioRoutingHandle, pTxBufferPtr, ganADCProcessBuffer[pAudioRoutingHandle->nADCToDwnstrReadIndex], nChannel);
#endif
	pAudioRoutingHandle->nUpstrToDwnstrReadIndex = (pAudioRoutingHandle->nUpstrToDwnstrReadIndex + 1u) % A2B_MAX_APPBUFFER;
	pAudioRoutingHandle->nADCToDwnstrReadIndex = (pAudioRoutingHandle->nADCToDwnstrReadIndex + 1u ) % A2B_MAX_APPBUFFER;

}

/****************************************************************************/
/*!
    @brief			Callback function ADC data recpetionC

    @param [in]     pRxBufferPtr    Transmission sub buffer pointer.
    @param [in]     nChannel     	TDM size.

    @return         void

    Note:
*/
/********************************************************************************/
static void adi_a2b_ADCRxCallback(void* pCommHandle, a2b_UInt32* pRxBufferPtr , a2b_UInt32 nChannel)
{
	nADCRxCnt++;
	ADI_A2B_SCOMM_HANDLER* pAudioRoutingHandle = (ADI_A2B_SCOMM_HANDLER*)pCommHandle;

	/* Copy SPORT sub buffer data to application buffer */
	adi_a2b_DeInterleaveCopy(ganADCProcessBuffer[pAudioRoutingHandle->nADCProcWriteIndex], pRxBufferPtr, nChannel);
	pAudioRoutingHandle->nADCProcWriteIndex = (pAudioRoutingHandle->nADCProcWriteIndex + 1u) % A2B_MAX_APPBUFFER;

}

/****************************************************************************/
/*!
    @brief			Callback function for transmission completion to DAC

    @param [in]     pTxBufferPtr    Transmission sub buffer pointer.
    @param [in]     nChannel     	TDM size.

    @return         void

    Note:
*/
/********************************************************************************/
static void adi_a2b_DACTxCallback(void* pCommHandle, a2b_UInt32* pTxBufferPtr, a2b_UInt32 nChannel)
{
	ADI_A2B_SCOMM_HANDLER* pAudioRoutingHandle = (ADI_A2B_SCOMM_HANDLER*)pCommHandle;

	nADCTxCnt++;
#if A2B_USE_CODEC
	/* Upstream to DAC routing */
	adi_a2b_UpstreamToDACRouting(pCommHandle, pTxBufferPtr, ganUpstreamProcessBuffer[pAudioRoutingHandle->nUpStrtoDACReadIndex], nChannel);
	/* ADC data to DAC routing */
	adi_a2b_ADCToDACRouting(pCommHandle, pTxBufferPtr, ganADCProcessBuffer[pAudioRoutingHandle->nADCtoDACReadIndex], nChannel);
#endif

	/* Index update */
	pAudioRoutingHandle->nADCtoDACReadIndex = (pAudioRoutingHandle->nADCtoDACReadIndex + 1u) % A2B_MAX_APPBUFFER;
	pAudioRoutingHandle->nUpStrtoDACReadIndex = (pAudioRoutingHandle->nUpStrtoDACReadIndex + 1u) % A2B_MAX_APPBUFFER;

}

/***********************************************************************************/
/*!
    @brief          This function copies upstream data to the downstream in accordance with routing table

    @param [in]     pHandle       			Pointer to serial port structure.
    @param [in]     anDwnstreamBuffer     	Pointer to downstream buffer/Destination.
    @param [in]     anUpstreamBuffer     	Source buffer address/Upstream  .
    @param [in]     nChannel         		TDM size

    @return         Return code
                    - 0: Success
                    - 1: Failure
*/
/***********************************************************************************/
static void adi_a2b_upstreamToDwnRouting(ADI_A2B_SCOMM_HANDLER* pAudioRoutingHandle, a2b_UInt32 anDwnstreamBuffer[], a2b_UInt32 anUpstreamBuffer[][SPORT_BUFFER_SIZE], a2b_UInt32 nChannel )
{
    a2b_UInt16 	nIndex1,nIndex2,nIndex3;
    a2b_UInt8 	nRoutingIndex;

    /* Outer most loop over TX channel */
    for(nIndex1 = 0u ; nIndex1 < nChannel ; nIndex1++)
    {
    	/* Get Routing Index  */
		nRoutingIndex = pAudioRoutingHandle->pRoutingTable[nIndex1 + A2B_MAX_CODEC_CHANNELS];

		if(nRoutingIndex <= (a2b_UInt8)A2B_UPSTREAM_CH31)
		{
			nRoutingIndex -= (a2b_UInt8)A2B_UPSTREAM_CH00;
			nIndex3 = 0u;
			/* Transfer the data from Rx to Tx */
			for(nIndex2 = 0u ; nIndex2 < (a2b_UInt16)SPORT_BUFFER_SIZE * nChannel ; nIndex2 += (a2b_UInt16)nChannel  )
			{
  				anDwnstreamBuffer[nIndex1 + nIndex2] = anUpstreamBuffer[nRoutingIndex][nIndex3++];
			}
		}

    }
}

/****************************************************************************/
/*!
    @brief          This function returns the Version of the A2B Stack Software

    @param [in]     major                   Major Version
    @param [in]     minor                   Minor Version
    @param [in]     release           Release number of Software

    @return          Return code
                    - 0: Success
                    - 1: Failure
*/
/********************************************************************************/


/***********************************************************************************/
/*!
    @brief          This function copies ADC data to downstream

    @param [in]     pAudioRoutingHandle       			Pointer to serial port structure.
    @param [in]     anDwnstreamBuffer     	Pointer to downstream buffer/Destination.
    @param [in]     anUpstreamBuffer     	Source buffer address/Upstream  .
    @param [in]     nChannel         		TDM size

    @return         Return code
                    - 0: Success
                    - 1: Failure
*/
/***********************************************************************************/
static void adi_a2b_ADCToDwnRouting(ADI_A2B_SCOMM_HANDLER* pAudioRoutingHandle, a2b_UInt32 anDwnstreamBuffer[], a2b_UInt32 anADCBuffer[][SPORT_BUFFER_SIZE], a2b_UInt32 nChannel )
{
    a2b_UInt16 	nIndex1, nIndex2, nIndex3;
    a2b_UInt8 	nRoutingIndex;

    /* Outer most loop- Rx to Tx copy */
    for(nIndex1 = 0u ; nIndex1 <nChannel ; nIndex1++)
    {
    	nIndex3 = 0u;
    	/* Get routing table */
    	nRoutingIndex = pAudioRoutingHandle->pRoutingTable[nIndex1 + A2B_MAX_CODEC_CHANNELS];
		if((nRoutingIndex > (a2b_UInt8)A2B_UPSTREAM_CH31) && (nRoutingIndex < (a2b_UInt8)A2B_CHANNEL_UNUSED))
		{
			nRoutingIndex-= (a2b_UInt8)A2B_ADC_CH00;
			/* Transfer the data from Rx to Tx */
			for(nIndex2 = 0u ; nIndex2 < nChannel * SPORT_BUFFER_SIZE ; nIndex2 += (a2b_UInt16)nChannel  )
			{
				anDwnstreamBuffer[ nIndex1 + nIndex2 ]  =  anADCBuffer[nRoutingIndex][nIndex3++ ];
			}
		}

    }

}


/***********************************************************************************/
/*!
    @brief          This function copies upstream data to DAC

    @param [in]     pHandle       Pointer to serial port structure.
    @param [in]     iTxBuffer     Destination buffer address.
    @param [in]     iRxBuffer     Source buffer address  .
    @param [in]     nSize         Size of the buffer .

    @return         Return code
                    - 0: Success
                    - 1: Failure
*/
/***********************************************************************************/
static void adi_a2b_UpstreamToDACRouting(ADI_A2B_SCOMM_HANDLER* pAudioRoutingHandle, a2b_UInt32 anDACBuffer[], a2b_UInt32 anUpstreamBuffer[][SPORT_BUFFER_SIZE], a2b_UInt32 nChannel )
{
    a2b_UInt16 	nIndex1, nIndex2, nIndex3;
    a2b_UInt8 	nRoutingIndex;

    /* Outer most loop */
    for(nIndex1 = 0u ; nIndex1 < nChannel ; nIndex1++)
    {
    	nIndex3 = 0u;
		nRoutingIndex = pAudioRoutingHandle->pRoutingTable[nIndex1];

		if(nRoutingIndex <= (a2b_UInt8)A2B_UPSTREAM_CH31)
		{
			nRoutingIndex -= (a2b_UInt8)A2B_UPSTREAM_CH00;
			/* Transfer the data from Rx to Tx */
			for(nIndex2 = 0u ; nIndex2 < nChannel * SPORT_BUFFER_SIZE ; nIndex2 += (a2b_UInt16)nChannel  )
			{
				anDACBuffer[ nIndex1 + nIndex2 ]  =  anUpstreamBuffer[nRoutingIndex ][nIndex3++];
			}
		}

    }

}

/***********************************************************************************/
/*!
    @brief          This function copies ADC data to DAC

    @param [in]     pHandle       Pointer to serial port structure.
    @param [in]     iTxBuffer     Destination buffer address.
    @param [in]     iRxBuffer     Source buffer address.
    @param [in]     nSize         Size of the buffer.

    @return         Return code
                    - 0: Success
                    - 1: Failure
*/
/***********************************************************************************/
static void adi_a2b_ADCToDACRouting(ADI_A2B_SCOMM_HANDLER* pAudioRoutingHandle, a2b_UInt32 anDACBuffer[], a2b_UInt32 anADCBuffer[][SPORT_BUFFER_SIZE], a2b_UInt32 nChannel)
{
    a2b_UInt16 	nIndex1, nIndex2, nIndex3;
    a2b_UInt8 	nRoutingIndex;

    /* Outer most loop */
    for(nIndex1 = 0u ; nIndex1 < nChannel ; nIndex1++)
    {
    	nIndex3 = 0u;

    	nRoutingIndex = pAudioRoutingHandle->pRoutingTable[nIndex1];

		if((nRoutingIndex > (a2b_UInt8)A2B_UPSTREAM_CH31) && (nRoutingIndex < (a2b_UInt8)A2B_CHANNEL_UNUSED))
		{
	    	nRoutingIndex-= (a2b_UInt8)A2B_ADC_CH00;
			/* Transfer the data from Rx to Tx */
			for(nIndex2 = 0u; nIndex2 < nChannel * SPORT_BUFFER_SIZE; nIndex2 += (a2b_UInt16)nChannel )
			{
				anDACBuffer[ nIndex1 + nIndex2 ]  =  anADCBuffer[nRoutingIndex][nIndex3++];
			}
		}

    }

}

/***********************************************************************************/
/*!
    @brief          This function deinterleaves multi channel audio and stores to processing buffer


    @param [in]     anDesbuffer     Destination buffer address.
    @param [in]     anADCBuffer     Source buffer address.
    @param [in]     nChannel        Number of TDM channels.

    @return         Return code
                    - 0: Success
                    - 1: Failure
*/
/***********************************************************************************/
static void adi_a2b_DeInterleaveCopy( a2b_UInt32 anDesbuffer[][SPORT_BUFFER_SIZE], a2b_UInt32 anSrcBuffer[], a2b_UInt32 nChannel )
{
    a2b_UInt16 	nIndex1, nIndex2, nIndex3;
    a2b_UInt8 	nRoutingIndex;

    /* Outer most loop */
    for(nIndex1 = 0u ; nIndex1 < nChannel ; nIndex1++)
    {
		/* Transfer the data from SPORT buffer to App buffer */
		for(nIndex2 = 0u; nIndex2 < (a2b_UInt16)SPORT_BUFFER_SIZE; nIndex2++  )
		{
			anDesbuffer[nIndex1 ][nIndex2 ]  =  anSrcBuffer[nIndex1 + nIndex2 * (a2b_UInt16)nChannel];
		}

    }
}

ADI_MEM_A2B_CODE_CRIT
a2b_HResult adi_a2b_SystemInit(void)
{
    ADI_A2B_RESULT eResult = ADI_A2B_SUCCESS;
    ADI_GPIO_RESULT eGPIORes;
	/* Pin multiplexing for SPI */
	eGPIORes = adi_gpio_SetDirection(ADI_GPIO_PORT_A, ADI_GPIO_PIN_13, ADI_GPIO_DIRECTION_OUTPUT);
	eGPIORes = adi_gpio_Set(ADI_GPIO_PORT_A, ADI_GPIO_PIN_13);
	PADS_Init();
    return eResult;
}

