/*******************************************************************************
Copyright (c) 2014 - Analog Devices Inc. All Rights Reserved.
This software is proprietary & confidential to Analog Devices, Inc.
and its licensors.
******************************************************************************
 * @file:    adi_a2b_codecInit.c
 * @brief:   This is the HAL implementation of SPORT, SPI, PCG, ADAU1979 ADC, ADAU1962A DAC and GPIO
 * @version: $Revision: 66468 $
 * @date:    $Date: 2021-06-28 08:51:38 -0400 (Mon, 28 Jun 2021) $
 * Developed by: Automotive Software and Systems team, Bangalore, India
*****************************************************************************/
/*! \addtogroup ADI_A2B_CODECINIT
 *  @{
 */

/*============= I N C L U D E S =============*/

#include "adi_a2b_hal.h"         /*!< ADI HAL include file */
#include <stdio.h>
#ifndef _TESSY_
/*-------------- S P O R T --------------*/

/*-------------- S P I --------------*/

/*-------------- P C G --------------*/
#if !defined(__ADSPSC598_FAMILY__)
/*-------------- A D A U 1 9 7 9 --------------*/
#include <drivers/adc/adau1979/adi_adau1979.h>      /*!< ADI ADAU1979 ADC definitions include file */

/*-------------- A D A U 1 9 6 2 A --------------*/
#include <drivers/dac/adau1962a/adi_adau1962a.h>    /*!< ADI ADAU1962A DAC definitions include file */
#endif
/*-------------- G P I O --------------*/
#include <services/gpio/adi_gpio.h>                 /*!< ADI GPIO definitions include file */

#else
#include <stddef.h>
#include "tessy_defines.h"
#endif

/*============= D E F I N E S =============*/

#if !(defined (_TESSY_) || defined (_TESSY_COMPILER_DEPENDENT_))
#define CHECK_RESULT_HAL(hDevice, nExpectedValue, nObtainedValue, nErrorMessage)    \
    do                                                                              \
    {                                                                               \
        if((nObtainedValue) != (nExpectedValue))                                    \
        {                                                                           \
            (hDevice) = NULL;                                                       \
        }                                                                           \
    } while (0)  /* do-while-zero needed for Misra Rule 19.4 */
#else
#define CHECK_RESULT_HAL(hDevice, nExpectedValue, nObtainedValue, nErrorMessage)
#endif	/* _TESSY_ */
/*-------------- S P O R T --------------*/

/*-------------- S P I --------------*/

/*-------------- P C G --------------*/
/*---------------ADC-DAC-----------------------*/
/* Configuration structure for ADC and DAC */
struct ConfigTable
{
	short RegAdd;
	uint8_t  Value;
};

/*-------------- A D A U 1 9 7 9 --------------*/

#define ADI_A2B_HAL_ADAU1979_BCLK_RAISING_EDGE        (true)         /*!< Drive audio data port on raising edge of BCLK*/
#define ADI_A2B_HAL_ADAU1979_BCLK_FALLING_EDGE        (false)        /*!< Drive audio data port on falling edge of BCLK*/
#define ADI_A2B_HAL_ADAU1979_LRCLK_HIGH_TO_LOW        (true)         /*!< Set LRCLK to go high then low                */
#define ADI_A2B_HAL_ADAU1979_LRCLK_LOW_TO_HIGH        (false)        /*!< Set LRCLK to go low then high                */

#define ADI_A2B_HAL_ADAU1979_LRCLK_BCLK_INTERNAL      (true)         /*!< ADAU1979 as LRCLK/BCLK Clock master          */
#define ADI_A2B_HAL_ADAU1979_LRCLK_BCLK_EXTERNAL      (false)        /*!< Use external LRCLK/BCLK master               */
#ifndef __ADSPSC59x__
#define ADI_A2B_HAL_ADAU1979_TWIDEV_NUM               (0U)           /*!< ADAU1979 instance number to open             */
#else
#define ADI_A2B_HAL_ADAU1979_TWIDEV_NUM               (2U)           /*!< ADAU1979 instance number to open             */
#endif
#define ADI_A2B_HAL_ADAU1979_DEVICE_NUM               (0U)           /*!< ADAU1979 TWI device number                   */
#define ADI_A2B_HAL_ADAU1979_PLLLOCK_SFTDWN_VAL       (7U)           /*!< ADAU1979 PPL lock shift down value           */

#define ADI_A2B_HAL_ADAU1979_POWER_UP                 (true)         /*!< ADAU1979 power up                            */
#define ADI_A2B_HAL_ADAU1979_POWER_DOWN               (false)        /*!< ADAU1979 power down                          */

#define ADI_A2B_HAL_ADAU1979_MUTE_CHANNEL             (true)         /*!< ADAU1979 mute channel                        */
#define ADI_A2B_HAL_ADAU1979_UNMUTE_CHANNEL           (false)        /*!< ADAU1979 unmute channel                      */

#define ADI_A2B_HAL_BITM_ADAU1979_01_PLL_STAT         (0x80u)        /*!< PLL lock status                              */
#define E_ADI_A2B_HAL_ADAU1979_06_LRCLK_MODE_PULSE    (0x08u)        /*!< LRCLK Mode - Pulse (32 BCLKs per channel)    */

/*-------------- A D A U 1 9 6 2 A --------------*/

#define ADI_A2B_HAL_ADAU1962A_MCLK_IN_FREQ_8192000HZ  ( 8192000U) /*!< Definition of ADAU1962A MCLKIN freq  8.192  MHz */
#define ADI_A2B_HAL_ADAU1962A_MCLK_IN_FREQ_11289600HZ (11289600U) /*!< Definition of ADAU1962A MCLKIN freq 11.2896 MHz */
#define ADI_A2B_HAL_ADAU1962A_MCLK_IN_FREQ_12288000HZ (12288000U) /*!< Definition of ADAU1962A MCLKIN freq 12.288  MHz */
#define ADI_A2B_HAL_ADAU1962A_MCLK_IN_FREQ_16384000HZ (16384000U) /*!< Definition of ADAU1962A MCLKIN freq 16.384  MHz */
#define ADI_A2B_HAL_ADAU1962A_MCLK_IN_FREQ_16934400HZ (16934400U) /*!< Definition of ADAU1962A MCLKIN freq 16.9344 MHz */
#define ADI_A2B_HAL_ADAU1962A_MCLK_IN_FREQ_18432000HZ (18432000U) /*!< Definition of ADAU1962A MCLKIN freq 18.432  MHz */
#define ADI_A2B_HAL_ADAU1962A_MCLK_IN_FREQ_22579200HZ (22579200U) /*!< Definition of ADAU1962A MCLKIN freq 22.5792 MHz */
#define ADI_A2B_HAL_ADAU1962A_MCLK_IN_FREQ_24576000HZ (24576000U) /*!< Definition of ADAU1962A MCLKIN freq 24.576  MHz */
#define ADI_A2B_HAL_ADAU1962A_MCLK_IN_FREQ_33868800HZ (33868800U) /*!< Definition of ADAU1962A MCLKIN freq 33.8688 MHz */
#define ADI_A2B_HAL_ADAU1962A_MCLK_IN_FREQ_36864000HZ (36864000U) /*!< Definition of ADAU1962A MCLKIN freq 36.864  MHz */

#define ADI_A2B_HAL_ADAU1962A_DEVICE_NUM              (0U)     /*!< ADAU1962A instance number to open                  */
#ifndef __ADSPSC59x__
#define ADI_A2B_HAL_ADAU1962A_TWIDEV_NUM              (0U)     /*!< ADAU1962A TWI device number                        */
#else
#define ADI_A2B_HAL_ADAU1962A_TWIDEV_NUM              (2U)     /*!< ADAU1962A TWI device number                        */
#endif
#define ADI_A2B_HAL_ADAU1962A_PLLLOCK_SFTDWN_VAL      (2U)     /*!< ADAU1962A PPL lock shift down value                */

#define ADI_A2B_HAL_ADAU1962A_POWER_UP                (true)   /*!< ADAU1962A power up                                 */
#define ADI_A2B_HAL_ADAU1962A_POWER_DOWN              (false)  /*!< ADAU1962A power down                               */

#define ADI_A2B_HAL_ADAU1962A_ENABLE_XTAL_OUT         (true )  /*!< Definition for ADAU1962A enabling external output  */
#define ADI_A2B_HAL_ADAU1962A_DISABLE_XTAL_OUT        (false)  /*!< Definition for ADAU1962A disabling external output */

#define ADI_A2B_HAL_ADAU1962A_TEMP_SENSOR_PWRUP       (true )  /*!< Definition for ADAU1962A temp sensor power up      */
#define ADI_A2B_HAL_ADAU1962A_TEMP_SENSOR_PWRUPDWN    (false)  /*!< Definition for ADAU1962A temp sensor power down    */
#define ADI_A2B_HAL_ADAU1962A_PLL_PWRUP               (true )  /*!< Definition for ADAU1962A PLL power up              */
#define ADI_A2B_HAL_ADAU1962A_VOLT_REG_PWRUP          (true )  /*!< Definition for ADAU1962A voltage regulator power up*/

#define ADI_A2B_HAL_ADAU1962A_DBCLK_MASTER            (true)   /*!< DLRCLK/DBCLK as Master                             */
#define ADI_A2B_HAL_ADAU1962A_DBCLK_SLAVE             (false)  /*!< DLRCLK/DBCLK as Slave                              */

#define ADI_A2B_HAL_ADAU1962A_DBCLK_INTERNAL          (true)   /*!< Generate DBCLK internally                          */
#define ADI_A2B_HAL_ADAU1962A_DBCLK_EXTERNAL          (false)  /*!< Use external DBCLK                                 */

#define ADI_A2B_HAL_ADAU1962A_DBCLK_FALLING_EDGE      (true)   /*!< latch data in DBCLK falling edge                   */
#define ADI_A2B_HAL_ADAU1962A_DBCLK_RAISING_EDGE      (false)  /*!< latch data in DBCLK raising edge                   */

#define ADI_A2B_HAL_ADAU1962A_LRCLK_PULSE             (true)   /*!< generate LRCLK pulse for TDM                       */
#define ADI_A2B_HAL_ADAU1962A_LRCLK_DUTYCYCLE_50      (false)  /*!< 50% duty cycle                                     */

#define ADI_A2B_HAL_ADAU1962A_LSB_FIRST               (true)   /*!< LSB first on DSDATA                                */
#define ADI_A2B_HAL_ADAU1962A_MSB_FIRST               (false)  /*!< MSB first on DSDATA                                */

#define ADI_A2B_HAL_ADAU1962A_LEFTCH_CLKHIGH          (true)   /*!< Left/Odd channels when LRCLK is High               */
#define ADI_A2B_HAL_ADAU1962A_LEFTCH_CLKLOW           (false)  /*!< Left/Odd channels when LRCLK is Low                */

#define ADI_A2B_HAL_ADAU1962A_MUTE_CHANNEL            (true)   /*!< ADAU1962A mute channel                             */
#define ADI_A2B_HAL_ADAU1962A_UNMUTE_CHANNEL          (false)  /*!< ADAU1962A unmute channel                           */

#define ADI_A2B_HAL_BITM_ADAU1962A_01_PLL_LOCK        (0x04u)  /*!< PLL Lock Indicator                                 */


#ifdef __ADSPSC59x__

#define ADI_A2B_HAL_ADAU1962A_NUM_REG                    (24u)		/* Number of ADAU1962A register values to be written */
#define ADI_A2B_HAL_ADAU1962A_TARGADDR    	     		(0x04u)     /* Hardware address of ADAU1962A ADC */
#define ADI_A2B_HAL_ADAU1962A_TWI_PRESCALE             	(12u)
#define ADI_A2B_HAL_ADAU1962A_TWI_DUTYCYCLE            	(50u)
#define ADI_A2B_HAL_ADAU1962A_TWI_BITRATE              	(100u)

#define ADI_A2B_HAL_ADAU1962_PLL_CTL_CTRL0			0x00
#define ADI_A2B_HAL_ADAU1962_PLL_CTL_CTRL1			0x01
#define ADI_A2B_HAL_ADAU1962_PDN_CTRL_1 			0x02
#define ADI_A2B_HAL_ADAU1962_PDN_CTRL_2			    0x03
#define ADI_A2B_HAL_ADAU1962_PDN_CTRL_3 			0x04
#define ADI_A2B_HAL_ADAU1962_TEMP_STAT  			0x05
#define ADI_A2B_HAL_ADAU1962_DAC_CTRL0  			0x06
#define ADI_A2B_HAL_ADAU1962_DAC_CTRL1   			0x07
#define ADI_A2B_HAL_ADAU1962_DAC_CTRL2  			0x08
#define ADI_A2B_HAL_ADAU1962_DAC_MUTE1			    0x09
#define ADI_A2B_HAL_ADAU1962_DAC_MUTE2  			0x0a
#define ADI_A2B_HAL_ADAU1962_MSTR_VOL   			0x0b
#define ADI_A2B_HAL_ADAU1962_DAC1_VOL   			0x0c
#define ADI_A2B_HAL_ADAU1962_DAC2_VOL			    0x0d
#define ADI_A2B_HAL_ADAU1962_DAC3_VOL				0x0e
#define ADI_A2B_HAL_ADAU1962_DAC4_VOL			    0x0f
#define ADI_A2B_HAL_ADAU1962_DAC5_VOL			    0x10
#define ADI_A2B_HAL_ADAU1962_DAC6_VOL			    0x11
#define ADI_A2B_HAL_ADAU1962_DAC7_VOL			    0x12
#define ADI_A2B_HAL_ADAU1962_DAC8_VOL	     		0x13
#define ADI_A2B_HAL_ADAU1962_DAC9_VOL   			0x14
#define ADI_A2B_HAL_ADAU1962_DAC10_VOL			    0x15
#define ADI_A2B_HAL_ADAU1962_DAC11_VOL			    0x16
#define ADI_A2B_HAL_ADAU1962_DAC12_VOL			    0x17
#define ADI_A2B_HAL_ADAU1962_PAD_STRGTH			    0x1C
#define ADI_A2B_HAL_ADAU1962_DAC_PWR1			    0x1D
#define ADI_A2B_HAL_ADAU1962_DAC_PWR2			    0x1E
#define ADI_A2B_HAL_ADAU1962_DAC_PWR3			    0x1F


#ifndef LR_CLOCK_50
static struct ConfigTable ConfigArrayDAC[ADI_A2B_HAL_ADAU1962A_NUM_REG] = {
		   	    {     ADI_A2B_HAL_ADAU1962_PDN_CTRL_1,		0xa4 },
		   	    {     ADI_A2B_HAL_ADAU1962_PDN_CTRL_2,	    0x00 },  //Power down DAC
		   	    {     ADI_A2B_HAL_ADAU1962_PDN_CTRL_3,	    0x00 }, //Power down DAC
		   	    {     ADI_A2B_HAL_ADAU1962_DAC_CTRL0,		0x19 },
		   	    {     ADI_A2B_HAL_ADAU1962_DAC_CTRL1,		0x62 }, // pulse mode LRCLK, MSB first //orginal value =42
		   	    {     ADI_A2B_HAL_ADAU1962_DAC_CTRL2,		0x04 },  // TDM Configuration
		   	    {     ADI_A2B_HAL_ADAU1962_DAC_MUTE1,	    0x00 },
		   	    {     ADI_A2B_HAL_ADAU1962_DAC_MUTE2,	    0x00 },
		   	    {     ADI_A2B_HAL_ADAU1962_MSTR_VOL,		0x00 },
			    {     ADI_A2B_HAL_ADAU1962_DAC1_VOL,	   	0x00 } ,
			    {     ADI_A2B_HAL_ADAU1962_DAC2_VOL,		0x00 },
				{     ADI_A2B_HAL_ADAU1962_DAC3_VOL,		0x00 },
				{     ADI_A2B_HAL_ADAU1962_DAC4_VOL,		0x00 },
				{     ADI_A2B_HAL_ADAU1962_DAC5_VOL,		0x00 },
				{     ADI_A2B_HAL_ADAU1962_DAC6_VOL,		0x00 },
				{     ADI_A2B_HAL_ADAU1962_DAC7_VOL,		0x00 },
				{     ADI_A2B_HAL_ADAU1962_DAC8_VOL,	   	0x00 },
				{     ADI_A2B_HAL_ADAU1962_DAC9_VOL,		0x00 },
				{     ADI_A2B_HAL_ADAU1962_DAC10_VOL,		0x00 },
				{     ADI_A2B_HAL_ADAU1962_DAC11_VOL,		0x00 },
				{     ADI_A2B_HAL_ADAU1962_DAC12_VOL,		0x00 },
				{     ADI_A2B_HAL_ADAU1962_PAD_STRGTH,		0x02 },
				{     ADI_A2B_HAL_ADAU1962_DAC_PWR1,		0xaa },
				{     ADI_A2B_HAL_ADAU1962_DAC_PWR2,		0xaa },

};

#else
static struct ConfigTable ConfigArrayDAC[ADI_A2B_HAL_ADAU1962A_NUM_REG] = {
		   	    {     ADI_A2B_HAL_ADAU1962_PDN_CTRL_1,		0x00 },
		   	    {     ADI_A2B_HAL_ADAU1962_PDN_CTRL_2,	    0xff },  //Power down DAC
		   	    {     ADI_A2B_HAL_ADAU1962_PDN_CTRL_3,	    0x0f }, //Power down DAC
		   	    {     ADI_A2B_HAL_ADAU1962_DAC_CTRL0,		0x19 },
		   	    {     ADI_A2B_HAL_ADAU1962_DAC_CTRL1,		0x02 }, // 50% duty cycle mode LRCLK, MSB first slave DAC
		   	    {     ADI_A2B_HAL_ADAU1962_DAC_CTRL2,		0x04 },  // TDM Configuration
		   	    {     ADI_A2B_HAL_ADAU1962_DAC_MUTE1,	    0x00 },
		   	    {     ADI_A2B_HAL_ADAU1962_DAC_MUTE2,	    0x00 },
		   	    {     ADI_A2B_HAL_ADAU1962_MSTR_VOL,			0x00 },
			    {     ADI_A2B_HAL_ADAU1962_DAC1_VOL,	    	0x00 } ,
			    {     ADI_A2B_HAL_ADAU1962_DAC2_VOL,			0x00 },
				{     ADI_A2B_HAL_ADAU1962_DAC3_VOL,			0x00 },
				{     ADI_A2B_HAL_ADAU1962_DAC4_VOL,			0x00 },
				{     ADI_A2B_HAL_ADAU1962_DAC5_VOL,			0x00 },
				{     ADI_A2B_HAL_ADAU1962_DAC6_VOL,			0x00 },
				{     ADI_A2B_HAL_ADAU1962_DAC7_VOL,			0x00 },
				{     ADI_A2B_HAL_ADAU1962_DAC8_VOL,	    	0x00 },
				{     ADI_A2B_HAL_ADAU1962_DAC9_VOL,			0x00 },
				{     ADI_A2B_HAL_ADAU1962_DAC10_VOL,		0x00 },
				{     ADI_A2B_HAL_ADAU1962_DAC11_VOL,		0x00 },
				{     ADI_A2B_HAL_ADAU1962_DAC12_VOL,		0x00 },
				{     ADI_A2B_HAL_ADAU1962_PAD_STRGTH,		0x02 },
				{     ADI_A2B_HAL_ADAU1962_DAC_PWR1,			0xaa },
				{     ADI_A2B_HAL_ADAU1962_DAC_PWR2,			0xaa },
				{     ADI_A2B_HAL_ADAU1962_DAC_PWR3,			0xaa },
				{     ADI_A2B_HAL_ADAU1962_PDN_CTRL_2,	    0x00 },  //Power up DAC
				{     ADI_A2B_HAL_ADAU1962_PDN_CTRL_3,	    0x00 }, //Power up DAC

};
#endif

#define ADI_A2B_HAL_ADAU1979_REG_POWER					0x00
#define ADI_A2B_HAL_ADAU1979_REG_PLL					0x01
#define ADI_A2B_HAL_ADAU1979_REG_BOOST					0x02
#define ADI_A2B_HAL_ADAU1979_REG_MICBIAS			    0x03
#define ADI_A2B_HAL_ADAU1979_REG_BLOCK_POWER_SAI	    0x04
#define ADI_A2B_HAL_ADAU1979_REG_SAI_CTRL0				0x05
#define ADI_A2B_HAL_ADAU1979_REG_SAI_CTRL1				0x06
#define ADI_A2B_HAL_ADAU1979_REG_CMAP12					0x07
#define ADI_A2B_HAL_ADAU1979_REG_CMAP34					0x08
#define ADI_A2B_HAL_ADAU1979_REG_SAI_OVERTEMP			0x09
#define ADI_A2B_HAL_ADAU1979_REG_POST_ADC_GAIN1 		0x0a
#define ADI_A2B_HAL_ADAU1979_REG_POST_ADC_GAIN2 		0x0b
#define ADI_A2B_HAL_ADAU1979_REG_POST_ADC_GAIN3 		0x0c
#define ADI_A2B_HAL_ADAU1979_REG_POST_ADC_GAIN4 		0x0d
#define ADI_A2B_HAL_ADAU1979_REG_MISC_CONTROL			0x0e
#define ADI_A2B_HAL_ADAU1979_REG_ADC_CLIP				0x19
#define ADI_A2B_HAL_ADAU1979_REG_DC_HPF_CAL				0x1a

#define ADI_A2B_HAL_ADAU1979_NUM_REG                    (13u)		/* Number of ADAU1979 register values to be written */
#define ADI_A2B_HAL_ADAU1979_TARGADDR    	     	    (0x11u)     /* Hardware address of adau1979 ADC */
#define ADI_A2B_HAL_ADAU1979_TWI_PRESCALE             	(12u)
#define ADI_A2B_HAL_ADAU1979_TWI_DUTYCYCLE            	(50u)
#define ADI_A2B_HAL_ADAU1979_TWI_BITRATE              	(100u)

#ifndef LR_CLOCK_50
/* Array of structures containing the register address for ADAU1979 and the value to be written to each register */
static struct ConfigTable ConfigADC[ADI_A2B_HAL_ADAU1979_NUM_REG] = {


 		 {ADI_A2B_HAL_ADAU1979_REG_BLOCK_POWER_SAI	,	0xb0},//power down ADC LDO enabel
		 {ADI_A2B_HAL_ADAU1979_REG_SAI_CTRL0			,	0x5a},//TDM 48kHz
		 {ADI_A2B_HAL_ADAU1979_REG_SAI_CTRL1			,	0x08},//LRCLK pulse mode //original value =08
		 {ADI_A2B_HAL_ADAU1979_REG_CMAP12			,	0x10},
		 {ADI_A2B_HAL_ADAU1979_REG_CMAP34			,	0x32},
		 {ADI_A2B_HAL_ADAU1979_REG_SAI_OVERTEMP		,	0xf0},
		 {ADI_A2B_HAL_ADAU1979_REG_POST_ADC_GAIN1 	,	0xA0},
		 {ADI_A2B_HAL_ADAU1979_REG_POST_ADC_GAIN2 	,	0xA0},
		 {ADI_A2B_HAL_ADAU1979_REG_POST_ADC_GAIN3 	,	0xA0},
		 {ADI_A2B_HAL_ADAU1979_REG_POST_ADC_GAIN4 	,	0xA0},
		 {ADI_A2B_HAL_ADAU1979_REG_MISC_CONTROL		,	0x12},//mute all channels
		 {ADI_A2B_HAL_ADAU1979_REG_DC_HPF_CAL		,	0x00},// filter off
		 {ADI_A2B_HAL_ADAU1979_REG_BLOCK_POWER_SAI	,	0xbf},//power on  ADC LDO enable
};
#else
/* Array of structures containing the register address for ADAU1979 and the value to be written to each register */
static struct ConfigTable ConfigADC[ADI_A2B_HAL_ADAU1979_NUM_REG] = {

		 {ADI_A2B_HAL_ADAU1979_REG_BOOST			,		0x00},// boost disable
		 {ADI_A2B_HAL_ADAU1979_REG_MICBIAS			,	0x00},//not used
 		 {ADI_A2B_HAL_ADAU1979_REG_BLOCK_POWER_SAI	,	0x70},//power down ADC LDO enabel
		 {ADI_A2B_HAL_ADAU1979_REG_SAI_CTRL0		,		0x5a},//TDM 48kHz
		 {ADI_A2B_HAL_ADAU1979_REG_SAI_CTRL1		,		0x00},//50% duty cycle LRCLK
		 {ADI_A2B_HAL_ADAU1979_REG_CMAP12			,	0x10},
		 {ADI_A2B_HAL_ADAU1979_REG_CMAP34			,	0x32},
		 {ADI_A2B_HAL_ADAU1979_REG_SAI_OVERTEMP		,	0xf0},
		 {ADI_A2B_HAL_ADAU1979_REG_POST_ADC_GAIN1 	,	0xA0},
		 {ADI_A2B_HAL_ADAU1979_REG_POST_ADC_GAIN2 	,	0xA0},
		 {ADI_A2B_HAL_ADAU1979_REG_POST_ADC_GAIN3 	,	0xA0},
		 {ADI_A2B_HAL_ADAU1979_REG_POST_ADC_GAIN4 	,	0xA0},
		 {ADI_A2B_HAL_ADAU1979_REG_MISC_CONTROL		,	0x12},//mute all channels
		 {ADI_A2B_HAL_ADAU1979_REG_DC_HPF_CAL		,	0x00},// filter off
		 {ADI_A2B_HAL_ADAU1979_REG_BLOCK_POWER_SAI	,	0x7f},//power on  ADC LDO enable

};

#endif
#endif
/*-------------- G P I O --------------*/

/*-------------- S P D I F Rx--------------*/

/*-------------- A S R C --------------*/

/*============= D A T A =============*/

/*-------------- S P O R T --------------*/

/*-------------- S P I --------------*/

/*-------------- P C G --------------*/

/*-------------- A D A U 1 9 7 9 --------------*/

/*-------------- A D A U 1 9 6 2 A --------------*/

/*-------------- G P I O --------------*/

static uint32_t aHalGpioPort[] = {
                            (uint32_t)ADI_GPIO_PORT_A,
                            (uint32_t)ADI_GPIO_PORT_B,
                            (uint32_t)ADI_GPIO_PORT_C,
                            (uint32_t)ADI_GPIO_PORT_D,
                            (uint32_t)ADI_GPIO_PORT_E,
                            (uint32_t)ADI_GPIO_PORT_F,
                            (uint32_t)ADI_GPIO_PORT_G
#ifdef __ADSPSC59x__
						   ,(uint32_t)ADI_GPIO_PORT_H
						   ,(uint32_t)ADI_GPIO_PORT_I
#endif
                          };


static uint32_t aHalGpioPin[] = {
                            ADI_GPIO_PIN_0,
                            ADI_GPIO_PIN_1,
                            ADI_GPIO_PIN_2,
                            ADI_GPIO_PIN_3,
                            ADI_GPIO_PIN_4,
                            ADI_GPIO_PIN_5,
                            ADI_GPIO_PIN_6,
                            ADI_GPIO_PIN_7,
                            ADI_GPIO_PIN_8,
                            ADI_GPIO_PIN_9,
                            ADI_GPIO_PIN_10,
                            ADI_GPIO_PIN_11,
                            ADI_GPIO_PIN_12,
                            ADI_GPIO_PIN_13,
                            ADI_GPIO_PIN_14,
                            ADI_GPIO_PIN_15
                         };

/*-------------- S P D I F Rx--------------*/

/*-------------- A S R C --------------*/

/*============= C O D E =============*/

/*
 * Function Definition section
 */
/*-------------- A D A U 1 9 7 9 --------------*/

/*!
    @brief          This function initializes ADAU1979 as per the settings in
                    pADAU1979Config using TWI.
    @param          pMemory         Pointer to a block of memory.
    @param          pADAU1979Config Pointer to ADAU1979 configuration structure to be
                                    populated by the caller.
    @return         ADI_A2B_HAL_ADAU1979_HANDLE: Device handle.
                       - A valid handle ADAU1979 handle
                       - NULL

   <b> Global Variables Used: <b>
                   - None
*/
#ifndef __ADSPSC59x__
ADI_A2B_HAL_ADAU1979_HANDLE adi_a2b_hal_adau1979_Open(uint32_t                        *pMemory,
                                                    uint32_t                        nMemSize,
                                                    ADI_A2B_HAL_ADAU1979_CONFIG      *pADAU1979Config)
{
    ADI_ADAU1979_RESULT         eADAU1979Res;
    ADI_ADAU1979_HANDLE         hADAU1979;
    ADI_ADAU1979_TWI_CONFIG     oADAU1979TWIConfig;
    ADI_ADAU1979_SAMPLE_RATE    eADAU1979SampleRate=ADI_ADAU1979_SAMPLE_RATE_48000HZ;
    uint8_t                     nValRegPllCtl;
    uint32_t                    nDelay=(uint32_t)0xFFFF;
    uint8_t                     nValRegSaiCtl0;

    if((pMemory!=NULL) && (nMemSize>=ADI_A2B_HAL_ADAU1979_MEMORY_SIZE) && (pADAU1979Config!=NULL))
    {
        switch(pADAU1979Config->eSampleRate)
        {
            case ADI_A2B_HAL_ADAU1979_SAMPLE_RATE_32000HZ:
                eADAU1979SampleRate = ADI_ADAU1979_SAMPLE_RATE_32000HZ;
                break;
            case ADI_A2B_HAL_ADAU1979_SAMPLE_RATE_44100HZ:
                eADAU1979SampleRate = ADI_ADAU1979_SAMPLE_RATE_44100HZ;
                break;
            case ADI_A2B_HAL_ADAU1979_SAMPLE_RATE_48000HZ:
                eADAU1979SampleRate = ADI_ADAU1979_SAMPLE_RATE_48000HZ;
                break;
            case ADI_A2B_HAL_ADAU1979_SAMPLE_RATE_96000HZ:
                eADAU1979SampleRate = ADI_ADAU1979_SAMPLE_RATE_96000HZ;
                break;
            case ADI_A2B_HAL_ADAU1979_SAMPLE_RATE_192000HZ:
                eADAU1979SampleRate = ADI_ADAU1979_SAMPLE_RATE_192000HZ;
                break;
            case ADI_A2B_HAL_ADAU1979_SAMPLE_RATE_NONE:
                eADAU1979SampleRate = ADI_ADAU1979_SAMPLE_RATE_NONE;
                break;
            default:
                break;
        }

        eADAU1979Res = adi_adau1979_Open(ADI_A2B_HAL_ADAU1979_DEVICE_NUM,
                                            ADI_ADAU1979_SERIAL_MODE_TDM8,
                                            pMemory,
                                            ADI_A2B_HAL_ADAU1979_MEMORY_SIZE,
                                            &hADAU1979);
        CHECK_RESULT_HAL(hADAU1979, ADI_ADAU1979_SUCCESS, eADAU1979Res, "adi_adau1979_Open failed");

        if(eADAU1979Res == ADI_ADAU1979_SUCCESS)
        {
            oADAU1979TWIConfig.TwiDevNum      = ADI_A2B_HAL_ADAU1979_TWIDEV_NUM;
            oADAU1979TWIConfig.pTwiDevMem     = (void*)((uint32_t)pMemory+ADI_ADAU1979_MEMORY_SIZE);
            oADAU1979TWIConfig.TwiDevMemSize  = ADI_TWI_MEMORY_SIZE;
            oADAU1979TWIConfig.eTwiAddr       = ADI_ADAU1979_TWI_ADDR_11;

            eADAU1979Res = adi_adau1979_ConfigTwi(hADAU1979,&oADAU1979TWIConfig);
            CHECK_RESULT_HAL(hADAU1979, ADI_ADAU1979_SUCCESS, eADAU1979Res, "adi_adau1979_ConfigTwi failed");

            eADAU1979Res = adi_adau1979_PowerUp(hADAU1979,ADI_ADAU1979_POWER_MASTER,ADI_A2B_HAL_ADAU1979_POWER_UP);
            CHECK_RESULT_HAL(hADAU1979, ADI_ADAU1979_SUCCESS, eADAU1979Res, "adi_adau1979_PowerUp failed");

            if(eADAU1979Res == ADI_ADAU1979_SUCCESS)
            {
                while(nDelay)
                {
                    nDelay--;
                }

                /* Wait for the PLL to get locked */
                eADAU1979Res = adi_adau1979_ReadRegister(hADAU1979,ADI_ADAU1979_REG_PLL_CTL,&nValRegPllCtl);
                CHECK_RESULT_HAL(hADAU1979, ADI_ADAU1979_SUCCESS, eADAU1979Res, "adi_adau1979_ReadRegister failed");

                while(!((nValRegPllCtl & ADI_A2B_HAL_BITM_ADAU1979_01_PLL_STAT)>>ADI_A2B_HAL_ADAU1979_PLLLOCK_SFTDWN_VAL))
                {
                    eADAU1979Res = adi_adau1979_ReadRegister(hADAU1979,ADI_ADAU1979_REG_PLL_CTL,&nValRegPllCtl);
                    CHECK_RESULT_HAL(hADAU1979, ADI_ADAU1979_SUCCESS, eADAU1979Res, "adi_adau1979_ReadRegister failed");
                }
            }
            else
            {
                eADAU1979Res = ADI_ADAU1979_FAILURE;
            }

            /* The "ADI_ADAU1979_MCLK_IN_FREQ_24576000HZ" is being ignored since the ADC is configured in slave mode. */
            eADAU1979Res = adi_adau1979_ConfigPllClk(hADAU1979,
                                                        ADI_A2B_HAL_ADAU1979_LRCLK_BCLK_EXTERNAL,
                                                        ADI_ADAU1979_MCLK_IN_FREQ_24576000HZ,
                                                        ADI_ADAU1979_PLL_SOURCE_LRCLK);
            CHECK_RESULT_HAL(hADAU1979, ADI_ADAU1979_SUCCESS, eADAU1979Res, "adi_adau1979_ConfigPllClk failed");

            /* CODEC_SLAVE and pulse mode (Refer JIRA EZKSW23-204 issue) */
            eADAU1979Res = adi_adau1979_WriteRegister(hADAU1979,
                                                        ADI_ADAU1979_REG_SAI_CTL2,
                                                        E_ADI_A2B_HAL_ADAU1979_06_LRCLK_MODE_PULSE);
            CHECK_RESULT_HAL(hADAU1979, ADI_ADAU1979_SUCCESS, eADAU1979Res, "adi_adau1979_WriteRegister failed");

            eADAU1979Res = adi_adau1979_SetSampleRate(hADAU1979,eADAU1979SampleRate);
            CHECK_RESULT_HAL(hADAU1979, ADI_ADAU1979_SUCCESS, eADAU1979Res, "adi_adau1979_SetSampleRate failed");

            eADAU1979Res = adi_adau1979_ConfigSerialClk(hADAU1979,
            											ADI_A2B_HAL_ADAU1979_BCLK_RAISING_EDGE,
                                                        ADI_A2B_HAL_ADAU1979_LRCLK_LOW_TO_HIGH);
            CHECK_RESULT_HAL(hADAU1979, ADI_ADAU1979_SUCCESS, eADAU1979Res, "adi_adau1979_ConfigSerialClk failed");

            eADAU1979Res = adi_adau1979_SetWordWidth(hADAU1979,ADI_ADAU1979_WORD_WIDTH_24);
            CHECK_RESULT_HAL(hADAU1979, ADI_ADAU1979_SUCCESS, eADAU1979Res, "adi_adau1979_SetWordWidth failed");

            eADAU1979Res = adi_adau1979_MuteChannel(hADAU1979,
                                                        ADI_ADAU1979_AUDIO_CHANNEL_ALL,
                                                        ADI_A2B_HAL_ADAU1979_MUTE_CHANNEL);
            CHECK_RESULT_HAL(hADAU1979, ADI_ADAU1979_SUCCESS, eADAU1979Res, "adi_adau1979_MuteChannel failed");

            /* Set SDATA_FMT to left justified mode. Currently there is no single SSL/DD API to do this */
            /* Hence the register is read, modified and re-written as below. */
            eADAU1979Res = adi_adau1979_ReadRegister(hADAU1979,ADI_ADAU1979_REG_SAI_CTL1,&nValRegSaiCtl0);
            CHECK_RESULT_HAL(hADAU1979, ADI_ADAU1979_SUCCESS, eADAU1979Res, "adi_adau1979_ReadRegister failed");

            nValRegSaiCtl0 |= 0x40;

            eADAU1979Res = adi_adau1979_WriteRegister(hADAU1979,
                                                      ADI_ADAU1979_REG_SAI_CTL1,
                                                      nValRegSaiCtl0);
            CHECK_RESULT_HAL(hADAU1979, ADI_ADAU1979_SUCCESS, eADAU1979Res, "adi_adau1979_WriteRegister failed");

            /*verifyRegistersAdau1979(hADAU1979);*/
        }
    }
    else
    {
        hADAU1979 = NULL;
    }

    return ((ADI_A2B_HAL_ADAU1979_HANDLE)hADAU1979);
}


/*!
    @brief          SigmaStudio for SHARC HAL ADAU1979 enable.
    @param          hADAU1979: Device handle.
    @return         Return type of ADI_A2B_HAL_ADAU1979_STATUS.
                       - ADI_A2B_HAL_ADAU1979_STATUS_SUCCESS : ADAU1962A enabled
                       - ADI_A2B_HAL_ADAU1979_STATUS_FAILED  : Error while enabling ADAU1962A

   <b> Global Variables Used: <b>
                   - None

   This function shall enable the ADAU1979 device.

  @note        None.

*/

ADI_A2B_HAL_ADAU1979_STATUS adi_a2b_hal_adau1979_Enable(ADI_A2B_HAL_ADAU1979_HANDLE hADAU1979)
{
	ADI_A2B_HAL_ADAU1979_STATUS eADAU1979Res=ADI_A2B_HAL_ADAU1979_STATUS_FAILED;


    if(hADAU1979!=NULL)
    {
        eADAU1979Res = adi_adau1979_MuteChannel(hADAU1979,
                                                    ADI_ADAU1979_AUDIO_CHANNEL_ALL,
                                                    ADI_A2B_HAL_ADAU1979_UNMUTE_CHANNEL);
    }
    return ((eADAU1979Res != ADI_ADAU1979_SUCCESS) ? ADI_A2B_HAL_ADAU1979_STATUS_FAILED :
                                                     ADI_A2B_HAL_ADAU1979_STATUS_SUCCESS);
}

/*-------------- A D A U 1 9 6 2 A --------------*/

/*!
   @brief   This function initializes ADAU1962A as per the settings in
            pADAU1962AConfig using TWI.
   @param   [in]    pMemory            Pointer to a block of memory.
   @param   [in]    pADAU1962AConfig   Pointer to ADAU1962A configuration structure to be
                                       populated by the caller.


   @return  ADI_A2B_HAL_ADAU1962A_HANDLE: Device Handle
                   - A valid handle ADAU1962A handle
                   - NULL

   <b> Global Variables Used: <b>
                   - None

   This function shall open the ADAU1962A DAC device instance, allocates memory required for the instance,
   configures the ADAU1962A sampling rate, word length and other parameters specified in the pADAU1962AConfig
   configuration structure.

  @note        None.
*/

ADI_A2B_HAL_ADAU1962A_HANDLE adi_a2b_hal_adau1962a_Open(uint32_t                      *pMemory,
                                                      uint32_t                      nMemSize,
                                                      ADI_A2B_HAL_ADAU1962A_CONFIG   *pADAU1962AConfig)
{
    ADI_ADAU1962A_RESULT                eADAU1962ARes;
    ADI_ADAU1962A_HANDLE                hADAU1962A;
    ADI_ADAU1962A_TWI_CONFIG            oADAU1962ATWIConfig;
    uint8_t                             nValRegPllCtl1;
    uint32_t                            nDelay=(uint32_t)0xFFFF;
    uint8_t 						    nValRegPllClkCtrl0 = 0;

    if((pMemory!=NULL) && (nMemSize>=ADI_A2B_HAL_ADAU1962A_MEMORY_SIZE) && (pADAU1962AConfig!=NULL))
    {
        eADAU1962ARes = adi_adau1962a_Open(ADI_A2B_HAL_ADAU1962A_DEVICE_NUM,
                                                            ADI_ADAU1962A_SERIAL_MODE_TDM8,
                                                            pMemory,
                                                            ADI_A2B_HAL_ADAU1962A_MEMORY_SIZE,
                                                            &hADAU1962A);
        CHECK_RESULT_HAL(hADAU1962A, ADI_ADAU1962A_SUCCESS, eADAU1962ARes, "adi_adau1962a_Open failed");

        if(eADAU1962ARes == ADI_ADAU1962A_SUCCESS)
        {
            oADAU1962ATWIConfig.TwiDevNum     = ADI_A2B_HAL_ADAU1962A_TWIDEV_NUM;
            oADAU1962ATWIConfig.pTwiDevMem    = (void*)((uint32_t)pMemory+ADI_ADAU1962A_MEMORY_SIZE);
            oADAU1962ATWIConfig.TwiDevMemSize = ADI_TWI_MEMORY_SIZE;
            oADAU1962ATWIConfig.eTwiAddr      = ADI_ADAU1962A_TWI_ADDR_04;

            eADAU1962ARes = adi_adau1962a_ConfigTwi(hADAU1962A,&oADAU1962ATWIConfig);
            CHECK_RESULT_HAL(hADAU1962A, ADI_ADAU1962A_SUCCESS, eADAU1962ARes, "adi_adau1962a_ConfigTwi failed");

            eADAU1962ARes = adi_adau1962a_ConfigDacPwr(hADAU1962A,
                                                            ADI_ADAU1962A_CHNL_DAC_MSTR,
                                                            ADI_ADAU1962A_DAC_PWR_LOW,
                                                            ADI_A2B_HAL_ADAU1962A_POWER_UP);
            CHECK_RESULT_HAL(hADAU1962A, ADI_ADAU1962A_SUCCESS, eADAU1962ARes, "adi_adau1962a_ConfigDacPwr failed");

            if(eADAU1962ARes == ADI_ADAU1962A_SUCCESS)
            {
                while(nDelay)
                {
                    nDelay--;
                }

                /* Wait for the PLL to get locked */
                eADAU1962ARes = adi_adau1962a_ReadRegister(hADAU1962A,ADI_ADAU1962A_REG_PLL_CLK_CTRL1,&nValRegPllCtl1);
                CHECK_RESULT_HAL(hADAU1962A, ADI_ADAU1962A_SUCCESS, eADAU1962ARes, "adi_adau1962a_ReadRegister failed");

                while(!((nValRegPllCtl1&ADI_A2B_HAL_BITM_ADAU1962A_01_PLL_LOCK)>>ADI_A2B_HAL_ADAU1962A_PLLLOCK_SFTDWN_VAL))
                {
                    eADAU1962ARes = adi_adau1962a_ReadRegister(hADAU1962A,ADI_ADAU1962A_REG_PLL_CLK_CTRL1,&nValRegPllCtl1);
                    CHECK_RESULT_HAL(hADAU1962A,ADI_ADAU1962A_SUCCESS,eADAU1962ARes,"adi_adau1962a_ReadRegister failed");
                }
            }
            else
            {
                eADAU1962ARes = ADI_ADAU1962A_FAILURE;
            }

            /* Refer JIRA issue EZKSW23-241 - DAC is not initialized to the correct "eSerialMode" which is obtained   */
            /* in "adi_adau1962a_Open()" function. Hence the below configuration is directly written into the DAC     */
            /* register using adi_adau1962a_WriteRegister() API.                                                      */
            /* DAC control 0 configuration: 1 SDATA_FMT to 'I2S, 1 BCLK Cycle Delay', TDM8, 48kHz, all channels un-muted    */
            /* This manual writing using adi_adau1962a_WriteRegister() function has to be removed                     */
            /* once there's a fix from SSLDD team.                                                                    */
            eADAU1962ARes = adi_adau1962a_WriteRegister(hADAU1962A, ADI_ADAU1962A_REG_DAC_CTRL0,(uint8_t)0x18);
            CHECK_RESULT_HAL(hADAU1962A, ADI_ADAU1962A_SUCCESS, eADAU1962ARes, "adi_adau1962a_WriteRegister failed");

            /* The "ADI_A2B_HAL_ADAU1962A_MCLK_IN_FREQ_24576000HZ" is being ignored since DAC is being configured in
             * slave mode.
             */
            eADAU1962ARes = adi_adau1962a_ConfigPllClk(hADAU1962A,
                                                            ADI_A2B_HAL_ADAU1962A_MCLK_IN_FREQ_24576000HZ,
                                                            ADI_ADAU1962A_MCLK_SEL_PLL,
                                                            ADI_ADAU1962A_PLL_IN_DLRCLK);
            CHECK_RESULT_HAL(hADAU1962A, ADI_ADAU1962A_SUCCESS, eADAU1962ARes, "adi_adau1962a_ConfigPllClk failed");

            /* Workaround for EZKSW23-404 */
            eADAU1962ARes = adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hADAU1962A, ADI_ADAU1962A_REG_PLL_CLK_CTRL0, &nValRegPllClkCtrl0);
            CHECK_RESULT_HAL(hADAU1962A,ADI_ADAU1962A_SUCCESS,eADAU1962ARes,"adi_adau1962a_ReadRegister failed");

            /* Set PLL source to DLRCLK */
            nValRegPllClkCtrl0 |= 0x40;
            eADAU1962ARes = adi_adau1962a_WriteRegister(hADAU1962A, ADI_ADAU1962A_REG_PLL_CLK_CTRL0,(uint8_t)nValRegPllClkCtrl0);
            CHECK_RESULT_HAL(hADAU1962A, ADI_ADAU1962A_SUCCESS, eADAU1962ARes, "adi_adau1962a_WriteRegister failed");

            eADAU1962ARes = adi_adau1962a_ConfigMClkOut(hADAU1962A,
                                                            ADI_A2B_HAL_ADAU1962A_DISABLE_XTAL_OUT,
                                                            ADI_ADAU1962A_MCLK_OUT_BUFFERED_MCLKI);
            CHECK_RESULT_HAL(hADAU1962A, ADI_ADAU1962A_SUCCESS, eADAU1962ARes, "adi_adau1962a_ConfigMClkOut failed");

            eADAU1962ARes = adi_adau1962a_ConfigSerialClk (hADAU1962A,
                                                            ADI_A2B_HAL_ADAU1962A_DBCLK_SLAVE,
                                                            ADI_A2B_HAL_ADAU1962A_DBCLK_EXTERNAL,
                                                            ADI_A2B_HAL_ADAU1962A_DBCLK_FALLING_EDGE,
                                                            ADI_A2B_HAL_ADAU1962A_LRCLK_PULSE,
                                                            ADI_A2B_HAL_ADAU1962A_MSB_FIRST,
                                                            ADI_A2B_HAL_ADAU1962A_LEFTCH_CLKLOW);
            CHECK_RESULT_HAL(hADAU1962A, ADI_ADAU1962A_SUCCESS, eADAU1962ARes, "adi_adau1962a_ConfigSerialClk failed");

            eADAU1962ARes = adi_adau1962a_ConfigBlockPwr (hADAU1962A,
                                                            ADI_A2B_HAL_ADAU1962A_TEMP_SENSOR_PWRUPDWN,
                                                            ADI_A2B_HAL_ADAU1962A_PLL_PWRUP,
                                                            ADI_A2B_HAL_ADAU1962A_VOLT_REG_PWRUP);
            CHECK_RESULT_HAL(hADAU1962A, ADI_ADAU1962A_SUCCESS, eADAU1962ARes, "adi_adau1962a_ConfigBlockPwr failed");

            eADAU1962ARes = adi_adau1962a_SetSampleRate(hADAU1962A, (uint32_t)pADAU1962AConfig->eSampleRate);
            CHECK_RESULT_HAL(hADAU1962A, ADI_ADAU1962A_SUCCESS, eADAU1962ARes, "adi_adau1962a_SetSampleRate failed");

            eADAU1962ARes = adi_adau1962a_SetWordWidth(hADAU1962A, ADI_ADAU1962A_WORD_WIDTH_24);
            CHECK_RESULT_HAL(hADAU1962A, ADI_ADAU1962A_SUCCESS, eADAU1962ARes, "adi_adau1962a_SetWordWidth failed");

            eADAU1962ARes = adi_adau1962a_MuteChannel(hADAU1962A,
                                                            ADI_ADAU1962A_CHNL_DAC_MSTR,
                                                            ADI_A2B_HAL_ADAU1962A_MUTE_CHANNEL);
            CHECK_RESULT_HAL(hADAU1962A, ADI_ADAU1962A_SUCCESS, eADAU1962ARes, "adi_adau1962a_MuteChannel failed");

            eADAU1962ARes = adi_adau1962a_ConfigDeEmphasis(hADAU1962A,false,true,false,false);
            CHECK_RESULT_HAL(hADAU1962A, ADI_ADAU1962A_SUCCESS, eADAU1962ARes, "adi_adau1962a_MuteChannel failed");

            /*verifyRegistersAdau1962a(hADAU1962A);*/
        }
    }
    else
    {
        hADAU1962A = NULL;
    }

    return ((ADI_A2B_HAL_ADAU1962A_HANDLE)hADAU1962A);

}

/*!
   @brief   SigmaStudio for SHARC HAL ADAU1962A enable.
   @param   [in]    hADAU1962A      Handle to ADAU1962A device.
   @return  ADI_A2B_HAL_ADAU1962A_STATUS
                   - ADI_A2B_HAL_ADAU1962A_STATUS_SUCCESS   : ADAU1962A enabled
                   - ADI_A2B_HAL_ADAU1962A_STATUS_FAILED    : Error while enabling ADAU1962A

   <b> Global Variables Used: <b>
                   - None

   This function shall enable the ADAU1962A device.

  @note        None.
*/

ADI_A2B_HAL_ADAU1962A_STATUS adi_a2b_hal_adau1962a_Enable(ADI_A2B_HAL_ADAU1962A_HANDLE hADAU1962A)
{
    ADI_ADAU1962A_RESULT    eADAU1962ARes=ADI_ADAU1962A_FAILURE;

    if(hADAU1962A!=NULL)
    {
        eADAU1962ARes = adi_adau1962a_MuteChannel(hADAU1962A,
                                                            ADI_ADAU1962A_CHNL_DAC_MSTR,
                                                            ADI_A2B_HAL_ADAU1962A_UNMUTE_CHANNEL);
    }
    return ((eADAU1962ARes != ADI_ADAU1962A_SUCCESS) ? ADI_A2B_HAL_ADAU1962A_STATUS_FAILED :
                                                       ADI_A2B_HAL_ADAU1962A_STATUS_SUCCESS);
}
#else

ADI_A2B_HAL_ADAU1979_STATUS adi_a2b_hal_adau1979_Config(uint32_t                        *pMemory,
                                                    uint32_t                        nMemSize,
                                                    ADI_A2B_HAL_ADAU1979_CONFIG      *pADAU1979Config,
													uint32_t *hTWI)
{


		ADI_A2B_HAL_ADAU1979_STATUS   eADAU1979Res				=ADI_A2B_HAL_ADAU1979_STATUS_SUCCESS;
	    uint32_t                     nValRegPllCtl;
	    uint32_t                     nDelay=(uint32_t)0xFFFF;
	    uint32_t                     nValRegSaiCtl0;
	    uint8_t                      devBuffer[2];
	    ADI_TWI_RESULT               eTWIResult= ADI_TWI_FAILURE;
	    uint32_t                     nNumRegs;
	    int               			 status;
	    uint32_t                     nTWIDeviceNum;
	    void        				 *pTotMemory;
	    uint32_t           			 nMemorySize;
	    ADI_TWI_HANDLE               phDevice;
	    uint8_t			 eADAU1979SampleRate;
	    unsigned char 		 Rx_Data;

	    if((pMemory!=NULL) && (nMemSize>=ADI_A2B_HAL_ADAU1979_MEMORY_SIZE) && (pADAU1979Config!=NULL))
	    {


	    		/* Get sample rate */
	        	eADAU1979SampleRate =(uint8_t)pADAU1979Config->eSampleRate ;

			nTWIDeviceNum= (uint32_t)ADI_A2B_HAL_ADAU1979_TWIDEV_NUM;
			pTotMemory= (void*)((uint32_t)pMemory+100u);
			nMemorySize =(uint32_t)ADI_TWI_MEMORY_SIZE;

			/* Configure TWI for ADAU1979 */
			eTWIResult = adi_twi_Open( nTWIDeviceNum, ADI_TWI_MASTER, pTotMemory,nMemSize, &phDevice);
			if(eTWIResult!=ADI_TWI_SUCCESS)
			{
				 eADAU1979Res=ADI_A2B_HAL_ADAU1979_STATUS_FAILED;
			}

			eTWIResult = adi_twi_SetHardwareAddress(phDevice, ADI_A2B_HAL_ADAU1979_TARGADDR);
			if(eTWIResult!=ADI_TWI_SUCCESS)
			{
				 eADAU1979Res=ADI_A2B_HAL_ADAU1979_STATUS_FAILED;
			}

			eTWIResult = adi_twi_SetPrescale(phDevice, ADI_A2B_HAL_ADAU1979_TWI_PRESCALE);
			if(eTWIResult!=ADI_TWI_SUCCESS)
			{
				 eADAU1979Res=ADI_A2B_HAL_ADAU1979_STATUS_FAILED;
			}

			eTWIResult = adi_twi_SetBitRate(phDevice, ADI_A2B_HAL_ADAU1979_TWI_BITRATE);
			if(eTWIResult!=ADI_TWI_SUCCESS)
			{
				 eADAU1979Res=ADI_A2B_HAL_ADAU1979_STATUS_FAILED;
			}
			eTWIResult = adi_twi_SetDutyCycle(phDevice, ADI_A2B_HAL_ADAU1979_TWI_DUTYCYCLE);
			if(eTWIResult!=ADI_TWI_SUCCESS)
			{
				 eADAU1979Res=ADI_A2B_HAL_ADAU1979_STATUS_FAILED;
			}

			/*  Master Power */
			devBuffer[0]=0x00;
			devBuffer[1]=0x01;
			/* Power up */
			eTWIResult = adi_twi_Write( phDevice,devBuffer , (uint32_t)2u, false);

			if(eTWIResult!=ADI_TWI_SUCCESS)
			{
				 eADAU1979Res=ADI_A2B_HAL_ADAU1979_STATUS_FAILED;
			}

			if(eTWIResult==ADI_TWI_SUCCESS)
			{
				/* PLL configuration for ADAU1979 */
				devBuffer[0]=ADI_A2B_HAL_ADAU1979_REG_PLL;
				devBuffer[1]=0x43;
				eTWIResult = adi_twi_Write( phDevice,devBuffer , (uint32_t)2u, false);
				if(eTWIResult!=ADI_TWI_SUCCESS)
				{
					 eADAU1979Res=ADI_A2B_HAL_ADAU1979_STATUS_FAILED;
				}

				/* Read the PLL status. Put the register address on the line */
				eTWIResult = adi_twi_Write(phDevice,devBuffer, (uint32_t)1u, true);
				if(eTWIResult!=ADI_TWI_SUCCESS)
				{
					 eADAU1979Res=ADI_A2B_HAL_ADAU1979_STATUS_FAILED;
				}

				/* Read the PLL register */
				eTWIResult = adi_twi_Read(phDevice,&Rx_Data,1u,false);
				if(eTWIResult!=ADI_TWI_SUCCESS)
				{
					 eADAU1979Res=ADI_A2B_HAL_ADAU1979_STATUS_FAILED;
				}

				/* While the PLL locks */
				while(nDelay!=0u)
				{
					nDelay--;
				}

				/* Wait for PLL lock */
				while(!((Rx_Data &  ADI_A2B_HAL_BITM_ADAU1979_01_PLL_STAT)>> ADI_A2B_HAL_ADAU1979_PLLLOCK_SFTDWN_VAL))
				{

					eTWIResult = adi_twi_Write(phDevice,devBuffer, (uint32_t)1u, true);
					if(eTWIResult!=ADI_TWI_SUCCESS)
					{
						 eADAU1979Res=ADI_A2B_HAL_ADAU1979_STATUS_FAILED;
					}
					eTWIResult = adi_twi_Read(phDevice,&Rx_Data,1u,false);
					if(eTWIResult!=ADI_TWI_SUCCESS)
					{
						 eADAU1979Res=ADI_A2B_HAL_ADAU1979_STATUS_FAILED;
					}
				}
				/* PLL configuration for ADAU1979 */
				devBuffer[0]=ADI_A2B_HAL_ADAU1979_REG_PLL;
				devBuffer[1]=0x53;
				eTWIResult = adi_twi_Write( phDevice,devBuffer , (uint32_t)2u, false);
				if(eTWIResult!=ADI_TWI_SUCCESS)
				{
					 eADAU1979Res=ADI_A2B_HAL_ADAU1979_STATUS_FAILED;
				}

				/* Read the PLL status. Put the register address on the line */
				eTWIResult = adi_twi_Write(phDevice,devBuffer, (uint32_t)1u, true);
				if(eTWIResult!=ADI_TWI_SUCCESS)
				{
					 eADAU1979Res=ADI_A2B_HAL_ADAU1979_STATUS_FAILED;
				}

				/* Read the PLL register */
				eTWIResult = adi_twi_Read(phDevice,&Rx_Data,1u,false);
				if(eTWIResult!=ADI_TWI_SUCCESS)
				{
					 eADAU1979Res=ADI_A2B_HAL_ADAU1979_STATUS_FAILED;
				}

				/* Static configuration. Write the register values */
				for(nNumRegs=0;nNumRegs<ADI_A2B_HAL_ADAU1979_NUM_REG;nNumRegs++)
				{
					 devBuffer[0]=(uint8_t)ConfigADC[nNumRegs].RegAdd;
					 devBuffer[1]=(uint8_t)ConfigADC[nNumRegs].Value;
					 eTWIResult = adi_twi_Write(phDevice,devBuffer, (uint32_t)2u, true);


					 /*test code*/
					 eTWIResult = adi_twi_Write(phDevice,devBuffer, (uint32_t)1u, true);
					if(eTWIResult!=ADI_TWI_SUCCESS)
					{
						 eADAU1979Res=ADI_A2B_HAL_ADAU1979_STATUS_FAILED;
					}
					eTWIResult = adi_twi_Read(phDevice,&Rx_Data,1u,false);
					if(eTWIResult!=ADI_TWI_SUCCESS)
					{
						 eADAU1979Res=ADI_A2B_HAL_ADAU1979_STATUS_FAILED;
					}

					if(Rx_Data!=devBuffer[1])
					{
						 eADAU1979Res=ADI_A2B_HAL_ADAU1979_STATUS_FAILED;
					}

					/*-------------*/

					 if(eTWIResult!=ADI_TWI_SUCCESS)
					 {
						 eADAU1979Res=ADI_A2B_HAL_ADAU1979_STATUS_FAILED;
					 }
				}

			}

	    *hTWI =(uint32_t)phDevice;
	    }
	    else
	    {
	    	 eADAU1979Res=ADI_A2B_HAL_ADAU1979_STATUS_FAILED;
	    }


	    return (eADAU1979Res);
}

ADI_A2B_HAL_ADAU1979_STATUS adi_a2b_hal_adau1979_Enable(void *phDevice)
{

	ADI_A2B_HAL_ADAU1979_STATUS   eADAU1979Res=ADI_A2B_HAL_ADAU1979_STATUS_SUCCESS;
	uint8_t                      devBuffer[2];
	ADI_TWI_RESULT               eTWIResult= ADI_TWI_FAILURE;

	/* Un-mute all ADC channels */
	devBuffer[0]=ADI_A2B_HAL_ADAU1979_REG_MISC_CONTROL;
	devBuffer[1]=0x02;

	eTWIResult = adi_twi_Write( phDevice,devBuffer , (uint32_t)2u, false);

	if(eTWIResult!=ADI_TWI_SUCCESS)
	{
		 eADAU1979Res=ADI_A2B_HAL_ADAU1979_STATUS_FAILED;
	}


    return eADAU1979Res;

}

ADI_A2B_HAL_ADAU1979_STATUS adi_a2b_hal_adau1979_Close(void *phDevice)
{

	ADI_A2B_HAL_ADAU1979_STATUS   eADAU1979Res=ADI_A2B_HAL_ADAU1979_STATUS_SUCCESS;
	ADI_TWI_RESULT               eTWIResult= ADI_TWI_FAILURE;

	eTWIResult=adi_twi_Close(phDevice);
	if(eTWIResult!=ADI_TWI_SUCCESS)
	{
		 eADAU1979Res=ADI_A2B_HAL_ADAU1979_STATUS_FAILED;
	}

    return eADAU1979Res;
}

/*!
   @brief   This function initializes ADAU1962A as per the settings in
            pADAU1962AConfig using TWI.
   @param   [in]    pMemory            Pointer to a block of memory.
   @param   [in]    pADAU1962AConfig   Pointer to ADAU1962A configuration structure to be
                                       populated by the caller.


   @return  ADI_A2B_HAL_ADAU1962A_HANDLE: Device Handle
                   - A valid handle ADAU1962A handle
                   - NULL

   <b> Global Variables Used: <b>
                   - None

   This function shall open the ADAU1962A DAC device instance, allocates memory required for the instance,
   configures the ADAU1962A sampling rate, word length and other parameters specified in the pADAU1962AConfig
   configuration structure.

  @note        None.
*/
ADI_A2B_HAL_ADAU1962A_STATUS adi_a2b_hal_adau1962a_Config(uint32_t                      *pMemory,
                                                      uint32_t                      nMemSize,
                                                      ADI_A2B_HAL_ADAU1962A_CONFIG   *pADAU1962AConfig,
													  uint32_t *hTWI)
{
    ADI_A2B_HAL_ADAU1962A_STATUS         eADAU1962ARes=ADI_A2B_HAL_ADAU1962A_STATUS_SUCCESS;
    uint32_t                             nValRegPllCtl1;
    uint32_t                             nDelay=(uint32_t)0xFFFF;
    uint32_t 			         		 nValRegPllClkCtrl0 = 0U;
    uint32_t 				 			 eADAU1962ASampleRate;
    uint8_t                     	 	 devBuffer[2];
    uint32_t                     	 	 nTWIDeviceNum;
    void        			 			*pTotMemory;
    uint32_t           			 		 nMemorySize;
    ADI_TWI_RESULT               	 	 eTWIResult= ADI_TWI_FAILURE;
    uint32_t                             delay1=65535u;
    ADI_TWI_HANDLE               	 	 phDevice;
    uint32_t 				 			 nNumRegs;
    unsigned char 			 			 Rx_Data;

    eADAU1962ASampleRate=(uint32_t)pADAU1962AConfig->eSampleRate;

    if((pMemory!=NULL) && (nMemSize>=ADI_A2B_HAL_ADAU1962A_MEMORY_SIZE) && (pADAU1962AConfig!=NULL))
    {

    	nTWIDeviceNum= (uint32_t)ADI_A2B_HAL_ADAU1962A_TWIDEV_NUM;
		pTotMemory= (void*)((uint32_t)pMemory+60u);
		nMemorySize =(uint32_t)ADI_TWI_MEMORY_SIZE;

		/* Configure TWI for ADAU1962A */
		eTWIResult = adi_twi_Open( nTWIDeviceNum, ADI_TWI_MASTER, pTotMemory,nMemSize, &phDevice);
		if(eTWIResult!=ADI_TWI_SUCCESS)
		{
			eADAU1962ARes=ADI_A2B_HAL_ADAU1962A_STATUS_FAILED;
		}

		eTWIResult = adi_twi_SetHardwareAddress(phDevice, ADI_A2B_HAL_ADAU1962A_TARGADDR);
		if(eTWIResult!=ADI_TWI_SUCCESS)
		{
			eADAU1962ARes=ADI_A2B_HAL_ADAU1962A_STATUS_FAILED;
		}

		eTWIResult = adi_twi_SetPrescale(phDevice, ADI_A2B_HAL_ADAU1962A_TWI_PRESCALE);
		if(eTWIResult!=ADI_TWI_SUCCESS)
		{
			eADAU1962ARes=ADI_A2B_HAL_ADAU1962A_STATUS_FAILED;
		}

		eTWIResult = adi_twi_SetBitRate(phDevice, ADI_A2B_HAL_ADAU1962A_TWI_BITRATE);
		if(eTWIResult!=ADI_TWI_SUCCESS)
		{
			eADAU1962ARes=ADI_A2B_HAL_ADAU1962A_STATUS_FAILED;
		}
		eTWIResult = adi_twi_SetDutyCycle(phDevice, ADI_A2B_HAL_ADAU1962A_TWI_DUTYCYCLE);
		if(eTWIResult!=ADI_TWI_SUCCESS)
		{
			eADAU1962ARes=ADI_A2B_HAL_ADAU1962A_STATUS_FAILED;
		}


		devBuffer[0]=ADI_A2B_HAL_ADAU1962_PLL_CTL_CTRL0;
		devBuffer[1]=0x01;

		eTWIResult = adi_twi_Write( phDevice,devBuffer , (uint32_t)2u, false);

		if(eTWIResult!=ADI_TWI_SUCCESS)
		{
			eADAU1962ARes=ADI_A2B_HAL_ADAU1962A_STATUS_FAILED;
		}

		while(delay1!=0u)
		{
		    delay1--;
		}


		delay1=65535u;

		if(eTWIResult==ADI_TWI_SUCCESS)
		{

			devBuffer[0]=ADI_A2B_HAL_ADAU1962_PLL_CTL_CTRL1;
			devBuffer[1]=0xaa;
			eTWIResult = adi_twi_Write( phDevice,devBuffer , (uint32_t)2u, false);

			if(eTWIResult!=ADI_TWI_SUCCESS)
			{
				eADAU1962ARes=ADI_A2B_HAL_ADAU1962A_STATUS_FAILED;
			}

			while(delay1!=0u)
			{
		    	    delay1--;
			}


			/* Read the PLL status. Put the register address on the line */
			eTWIResult = adi_twi_Write(phDevice,devBuffer, (uint32_t)1u, true);
			if(eTWIResult!=ADI_TWI_SUCCESS)
			{
				eADAU1962ARes=ADI_A2B_HAL_ADAU1962A_STATUS_FAILED;
			}

			/* Read the PLL register */
			eTWIResult = adi_twi_Read(phDevice,&Rx_Data,1u,false);
			if(eTWIResult!=ADI_TWI_SUCCESS)
			{
				eADAU1962ARes=ADI_A2B_HAL_ADAU1962A_STATUS_FAILED;
			}


			/* Wait for PLL lock */
			while(!((Rx_Data & ADI_A2B_HAL_BITM_ADAU1962A_01_PLL_LOCK)>>ADI_A2B_HAL_ADAU1962A_PLLLOCK_SFTDWN_VAL))
			{

				/* Read the PLL status. Put the register address on the line */
				eTWIResult = adi_twi_Write(phDevice,devBuffer, (uint32_t)1u, true);
				if(eTWIResult!=ADI_TWI_SUCCESS)
				{
					eADAU1962ARes=ADI_A2B_HAL_ADAU1962A_STATUS_FAILED;
				}

				/* Read the PLL register */
				eTWIResult = adi_twi_Read(phDevice,&Rx_Data,1u,false);
				if(eTWIResult!=ADI_TWI_SUCCESS)
				{
					eADAU1962ARes=ADI_A2B_HAL_ADAU1962A_STATUS_FAILED;
				}
			}

		delay1=65535u;

		devBuffer[0]=ADI_A2B_HAL_ADAU1962_PLL_CTL_CTRL0;
		devBuffer[1]=0x75;

		eTWIResult = adi_twi_Write( phDevice,devBuffer , (uint32_t)2u, false);

		if(eTWIResult!=ADI_TWI_SUCCESS)
		{
			eADAU1962ARes=ADI_A2B_HAL_ADAU1962A_STATUS_FAILED;
		}

		while(delay1!=0u)
		{
		    delay1--;
		}


		/* Read the PLL status. Put the register address on the line */
		eTWIResult = adi_twi_Write(phDevice,devBuffer, (uint32_t)1u, true);
		if(eTWIResult!=ADI_TWI_SUCCESS)
		{
			eADAU1962ARes=ADI_A2B_HAL_ADAU1962A_STATUS_FAILED;
		}

		/* Read the PLL register */
		eTWIResult = adi_twi_Read(phDevice,&Rx_Data,1u,false);
		if(eTWIResult!=ADI_TWI_SUCCESS)
		{
			eADAU1962ARes=ADI_A2B_HAL_ADAU1962A_STATUS_FAILED;
		}

			/* Static configuration. Write the register values */
			for(nNumRegs=0;nNumRegs<ADI_A2B_HAL_ADAU1962A_NUM_REG;nNumRegs++)
			{
				 devBuffer[0]=(uint8_t)ConfigArrayDAC[nNumRegs].RegAdd;
				 devBuffer[1]=(uint8_t)ConfigArrayDAC[nNumRegs].Value;
				 eTWIResult = adi_twi_Write(phDevice,devBuffer, (uint32_t)2u, true);
				 if(eTWIResult!=ADI_TWI_SUCCESS)
				 {
					 eADAU1962ARes=ADI_A2B_HAL_ADAU1962A_STATUS_FAILED;
				 }


				 /*test code*/
				 eTWIResult = adi_twi_Write(phDevice,devBuffer, (uint32_t)1u, true);
				if(eTWIResult!=ADI_TWI_SUCCESS)
				{
					eADAU1962ARes=ADI_A2B_HAL_ADAU1962A_STATUS_FAILED;
				}
				eTWIResult = adi_twi_Read(phDevice,&Rx_Data,1u,false);
				if(eTWIResult!=ADI_TWI_SUCCESS)
				{
					eADAU1962ARes=ADI_A2B_HAL_ADAU1962A_STATUS_FAILED;
				}


				if(Rx_Data!=devBuffer[1])
				{
					eADAU1962ARes=ADI_A2B_HAL_ADAU1962A_STATUS_FAILED;

				}
				/*-------------*/
			}

			*hTWI =(uint32_t)phDevice;
		}
    }
    else
    {
    	eADAU1962ARes = ADI_A2B_HAL_ADAU1962A_STATUS_FAILED;
    }

    return eADAU1962ARes ;

}

ADI_A2B_HAL_ADAU1962A_STATUS adi_a2b_hal_adau1962a_Enable(void *phDevice)
{

	ADI_A2B_HAL_ADAU1962A_STATUS   eADAU1962ARes=ADI_A2B_HAL_ADAU1962A_STATUS_SUCCESS;
	uint8_t                      devBuffer[2];
	ADI_TWI_RESULT               eTWIResult= ADI_TWI_FAILURE;

	/* Un-mute all DAC channels */
	devBuffer[0]=ADI_A2B_HAL_ADAU1962_DAC_CTRL0;
	devBuffer[1]=0x18;

	eTWIResult = adi_twi_Write( phDevice,devBuffer , (uint32_t)2u, false);

	if(eTWIResult!=ADI_TWI_SUCCESS)
	{
		eADAU1962ARes=ADI_A2B_HAL_ADAU1962A_STATUS_FAILED;
	}


    return eADAU1962ARes;

}

ADI_A2B_HAL_ADAU1962A_STATUS adi_a2b_hal_adau1962a_Close(void *phDevice)
{

	ADI_A2B_HAL_ADAU1962A_STATUS   eADAU1962ARes=ADI_A2B_HAL_ADAU1962A_STATUS_SUCCESS;
	ADI_TWI_RESULT               eTWIResult= ADI_TWI_FAILURE;

	eTWIResult=adi_twi_Close(phDevice);
	if(eTWIResult!=ADI_TWI_SUCCESS)
	{
		eADAU1962ARes=ADI_A2B_HAL_ADAU1962A_STATUS_FAILED;
	}

    return eADAU1962ARes;
}
#endif


/*-------------- G P I O --------------*/

/*!
    @brief      Sets the given GPIO pin(s) to an input or output direction.
    @param      ePort       The GPIO port.
    @param      ePin        The pin(s) to set as input or output.
    @param      eDirection  The direction to set.
    @return     Return code of ADI_A2B_HAL_GPIO_STATUS type.
                    - #ADI_A2B_HAL_GPIO_STATUS_SUCCESS   : The API call succeeded.
                    - #ADI_A2B_HAL_GPIO_STATUS_FAILED    : The API call failed.

    <b> Global Variables Used: <b>
                None

   @note        This API has to be individually called for each pin to set the direction.
 */

ADI_A2B_HAL_GPIO_STATUS adi_a2b_hal_gpio_SetDirection(ADI_A2B_HAL_GPIO_PORT        ePort,
                                                    ADI_A2B_HAL_GPIO_PIN         ePin,
                                                    ADI_A2B_HAL_GPIO_DIRECTION   eDirection)
{
    ADI_GPIO_RESULT     eGpioResult=ADI_GPIO_FAILURE;
    ADI_GPIO_DIRECTION  eGpioDirection=ADI_GPIO_DIRECTION_OUTPUT;

    switch(eDirection)
    {
        case ADI_A2B_HAL_GPIO_DIRECTION_INPUT:
            eGpioDirection = ADI_GPIO_DIRECTION_INPUT;
            break;
        case ADI_A2B_HAL_GPIO_DIRECTION_OUTPUT:
            eGpioDirection = ADI_GPIO_DIRECTION_OUTPUT;
            break;
        default :
            break;
    }

    eGpioResult = adi_gpio_SetDirection((ADI_GPIO_PORT)aHalGpioPort[ePort], (uint32_t)aHalGpioPin[ePin],eGpioDirection);

    return ((eGpioResult != ADI_GPIO_SUCCESS) ? ADI_A2B_HAL_GPIO_STATUS_FAILED : ADI_A2B_HAL_GPIO_STATUS_SUCCESS);
}

/*!
   @brief       Sets the given GPIO pin(s) to a logic high level.
                The logic level of other pins is not changed.
   @param       ePort       The GPIO port.
   @param       ePin        The pin(s) to set to a logic high level.
   @return      Return code of ADI_A2B_HAL_GPIO_STATUS type.
                    - #ADI_A2B_HAL_GPIO_STATUS_SUCCESS   : The API call succeeded.
                    - #ADI_A2B_HAL_GPIO_STATUS_FAILED    : The API call failed.

   <b> Global Variables Used: <b>
                None

  @note         This API has to be individually called for each pin to Set.
*/

ADI_A2B_HAL_GPIO_STATUS adi_a2b_hal_gpio_Set(ADI_A2B_HAL_GPIO_PORT ePort, ADI_A2B_HAL_GPIO_PIN ePin)
{
    ADI_GPIO_RESULT eGpioResult;

    eGpioResult = adi_gpio_Set((ADI_GPIO_PORT)aHalGpioPort[ePort], (uint32_t)aHalGpioPin[ePin]);

    return ((eGpioResult != ADI_GPIO_SUCCESS) ? ADI_A2B_HAL_GPIO_STATUS_FAILED : ADI_A2B_HAL_GPIO_STATUS_SUCCESS);
}

/*!
   @brief       Sets the given GPIO pin(s) to a logic low level.
                The logic level of other pins is not changed.
   @param       ePort       The GPIO port.
   @param       ePin        The pin(s) to set to a logic low level.
   @return      Return code of ADI_A2B_HAL_GPIO_STATUS type.
                    - #ADI_A2B_HAL_GPIO_STATUS_SUCCESS   : The API call succeeded.
                    - #ADI_A2B_HAL_GPIO_STATUS_FAILED    : The API call failed.

   <b> Global Variables Used: <b>
                None

  @note         This API has to be individually called for each pin to Clear.
*/

ADI_A2B_HAL_GPIO_STATUS adi_a2b_hal_gpio_Clear(ADI_A2B_HAL_GPIO_PORT ePort, ADI_A2B_HAL_GPIO_PIN ePin)
{
    ADI_GPIO_RESULT eGpioResult;

    eGpioResult = adi_gpio_Clear((ADI_GPIO_PORT)aHalGpioPort[ePort], (uint32_t)aHalGpioPin[ePin]);

    return ((eGpioResult != ADI_GPIO_SUCCESS) ? ADI_A2B_HAL_GPIO_STATUS_FAILED : ADI_A2B_HAL_GPIO_STATUS_SUCCESS);
}

/*!
   @brief       Toggle the logic level of the given GPIO pin(s).
                The logic level of other pins is not changed.
   @param       ePort       The GPIO port.
   @param       ePin        The pin(s) to set to toggle.
   @return      Return code of ADI_A2B_HAL_GPIO_STATUS type.
                    - #ADI_A2B_HAL_GPIO_STATUS_SUCCESS   : The API call succeeded.
                    - #ADI_A2B_HAL_GPIO_STATUS_FAILED    : The API call failed.

   <b> Global Variables Used: <b>
                None

  @note         This API has to be individually called for each pin to Toggle.
*/

ADI_A2B_HAL_GPIO_STATUS adi_a2b_hal_gpio_Toggle(ADI_A2B_HAL_GPIO_PORT ePort, ADI_A2B_HAL_GPIO_PIN ePin)
{
    ADI_GPIO_RESULT eGpioResult;

    eGpioResult = adi_gpio_Toggle((ADI_GPIO_PORT)aHalGpioPort[ePort], (uint32_t)aHalGpioPin[ePin]);

    return ((eGpioResult != ADI_GPIO_SUCCESS) ? ADI_A2B_HAL_GPIO_STATUS_FAILED : ADI_A2B_HAL_GPIO_STATUS_SUCCESS);
}

#if 0
void verifyRegistersAdau1979(ADI_A2B_HAL_ADAU1979_HANDLE hAdau1979)
{
    uint8_t valRegPwr=0,valRegPllCtl=0,valRegPwrSai=0,valRegSaiCtl1=0;
    uint8_t valRegSaiCtl2=0,valRegChMap1=0,valRegChMap2=0,valRegTemp=0;
    uint8_t valRegGain1=0,valRegGain2=0,valRegGain3=0,valRegGain4=0;
    uint8_t valRegMisCtl=0,valRegClip=0,valRegHPF=0;

    adi_adau1979_ReadRegister((ADI_ADAU1979_HANDLE)hAdau1979,ADI_ADAU1979_REG_M_PWR,&valRegPwr);
    adi_adau1979_ReadRegister((ADI_ADAU1979_HANDLE)hAdau1979,ADI_ADAU1979_REG_PLL_CTL,&valRegPllCtl);
    adi_adau1979_ReadRegister((ADI_ADAU1979_HANDLE)hAdau1979,ADI_ADAU1979_REG_BLK_PWR_SAI,&valRegPwrSai);
    adi_adau1979_ReadRegister((ADI_ADAU1979_HANDLE)hAdau1979,ADI_ADAU1979_REG_SAI_CTL1,&valRegSaiCtl1);
    adi_adau1979_ReadRegister((ADI_ADAU1979_HANDLE)hAdau1979,ADI_ADAU1979_REG_SAI_CTL2,&valRegSaiCtl2);
    adi_adau1979_ReadRegister((ADI_ADAU1979_HANDLE)hAdau1979,ADI_ADAU1979_REG_SAI_CMAP12,&valRegChMap1);
    adi_adau1979_ReadRegister((ADI_ADAU1979_HANDLE)hAdau1979,ADI_ADAU1979_REG_SAI_CMAP34,&valRegChMap2);
    adi_adau1979_ReadRegister((ADI_ADAU1979_HANDLE)hAdau1979,ADI_ADAU1979_REG_SAI_OVRTEMP,&valRegTemp);
    adi_adau1979_ReadRegister((ADI_ADAU1979_HANDLE)hAdau1979,ADI_ADAU1979_REG_POSTADC_GAIN1,&valRegGain1);
    adi_adau1979_ReadRegister((ADI_ADAU1979_HANDLE)hAdau1979,ADI_ADAU1979_REG_POSTADC_GAIN2,&valRegGain2);
    adi_adau1979_ReadRegister((ADI_ADAU1979_HANDLE)hAdau1979,ADI_ADAU1979_REG_POSTADC_GAIN3,&valRegGain3);
    adi_adau1979_ReadRegister((ADI_ADAU1979_HANDLE)hAdau1979,ADI_ADAU1979_REG_POSTADC_GAIN4,&valRegGain4);
    adi_adau1979_ReadRegister((ADI_ADAU1979_HANDLE)hAdau1979,ADI_ADAU1979_REG_MISC_CTL,&valRegMisCtl);
    adi_adau1979_ReadRegister((ADI_ADAU1979_HANDLE)hAdau1979,ADI_ADAU1979_REG_ASC_CLIP,&valRegClip);
    adi_adau1979_ReadRegister((ADI_ADAU1979_HANDLE)hAdau1979,ADI_ADAU1979_REG_DC_HPF_CAL,&valRegHPF);

    printf("ADI_ADAU1979_REG_M_PWR			=0x%x\n",	valRegPwr);
    printf("ADI_ADAU1979_REG_PLL_CTL		=0x%x\n",	valRegPllCtl);
    printf("ADI_ADAU1979_REG_BLK_PWR_SAI	=0x%x\n",	valRegPwrSai);
    printf("ADI_ADAU1979_REG_SAI_CTL1		=0x%x\n",	valRegSaiCtl1);
    printf("ADI_ADAU1979_REG_SAI_CTL2		=0x%x\n",	valRegSaiCtl2);
    printf("ADI_ADAU1979_REG_SAI_CMAP12		=0x%x\n",	valRegChMap1);
    printf("ADI_ADAU1979_REG_SAI_CMAP34		=0x%x\n",	valRegChMap2);
    printf("ADI_ADAU1979_REG_SAI_OVRTEMP	=0x%x\n",	valRegTemp);
    printf("ADI_ADAU1979_REG_POSTADC_GAIN1	=0x%x\n",	valRegGain1);
    printf("ADI_ADAU1979_REG_POSTADC_GAIN2	=0x%x\n",	valRegGain2);
    printf("ADI_ADAU1979_REG_POSTADC_GAIN3	=0x%x\n",	valRegGain3);
    printf("ADI_ADAU1979_REG_POSTADC_GAIN4	=0x%x\n",	valRegGain4);
    printf("ADI_ADAU1979_REG_MISC_CTL		=0x%x\n",	valRegMisCtl);
    printf("ADI_ADAU1979_REG_ASC_CLIP		=0x%x\n",	valRegClip);
    printf("ADI_ADAU1979_REG_DC_HPF_CAL		=0x%x\n",	valRegHPF);
}

void verifyRegistersAdau1962a(ADI_A2B_HAL_ADAU1962A_HANDLE hAdau1962a)
{
    uint8_t valReg_pll_clk_ctrl0        = 0;
    uint8_t valReg_pll_clk_ctrl1        = 0;
    uint8_t valReg_pdn_thrmsens_ctrl_1  = 0;
    uint8_t valReg_pdn_ctrl2            = 0;
    uint8_t valReg_pdn_ctrl3            = 0;
    uint8_t valReg_thrm_temp_stat       = 0;
    uint8_t valReg_dac_ctrl0            = 0;
    uint8_t valReg_dac_ctrl1            = 0;
    uint8_t valReg_dac_ctrl2            = 0;
    uint8_t valReg_dac_mute1            = 0;
    uint8_t valReg_dac_mute2            = 0;
    uint8_t valReg_dacmstr_vol          = 0;
    uint8_t valReg_dac1_vol             = 0;
    uint8_t valReg_dac2_vol             = 0;
    uint8_t valReg_dac3_vol             = 0;
    uint8_t valReg_dac4_vol             = 0;
    uint8_t valReg_dac5_vol             = 0;
    uint8_t valReg_dac6_vol             = 0;
    uint8_t valReg_dac7_vol             = 0;
    uint8_t valReg_dac8_vol             = 0;
    uint8_t valReg_dac9_vol             = 0;
    uint8_t valReg_dac10_vol            = 0;
    uint8_t valReg_dac11_vol            = 0;
    uint8_t valReg_dac12_vol            = 0;
    uint8_t valReg_pad_strgth           = 0;
    uint8_t valReg_dac_power1           = 0;
    uint8_t valReg_dac_power2           = 0;
    uint8_t valReg_dac_power3           = 0;

    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_PLL_CLK_CTRL0      ,&valReg_pll_clk_ctrl0      );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_PLL_CLK_CTRL1      ,&valReg_pll_clk_ctrl1      );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_PDN_THRMSENS_CTRL_1,&valReg_pdn_thrmsens_ctrl_1);
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_PDN_CTRL2          ,&valReg_pdn_ctrl2          );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_PDN_CTRL3          ,&valReg_pdn_ctrl3          );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_THRM_TEMP_STAT     ,&valReg_thrm_temp_stat     );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_DAC_CTRL0          ,&valReg_dac_ctrl0          );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_DAC_CTRL1          ,&valReg_dac_ctrl1          );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_DAC_CTRL2          ,&valReg_dac_ctrl2          );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_DAC_MUTE1          ,&valReg_dac_mute1          );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_DAC_MUTE2          ,&valReg_dac_mute2          );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_DACMSTR_VOL        ,&valReg_dacmstr_vol        );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_DAC1_VOL           ,&valReg_dac1_vol           );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_DAC2_VOL           ,&valReg_dac2_vol           );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_DAC3_VOL           ,&valReg_dac3_vol           );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_DAC4_VOL           ,&valReg_dac4_vol           );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_DAC5_VOL           ,&valReg_dac5_vol           );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_DAC6_VOL           ,&valReg_dac6_vol           );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_DAC7_VOL           ,&valReg_dac7_vol           );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_DAC8_VOL           ,&valReg_dac8_vol           );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_DAC9_VOL           ,&valReg_dac9_vol           );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_DAC10_VOL          ,&valReg_dac10_vol          );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_DAC11_VOL          ,&valReg_dac11_vol          );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_DAC12_VOL          ,&valReg_dac12_vol          );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_PAD_STRGTH         ,&valReg_pad_strgth         );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_DAC_POWER1         ,&valReg_dac_power1         );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_DAC_POWER2         ,&valReg_dac_power2         );
    adi_adau1962a_ReadRegister((ADI_ADAU1962A_HANDLE)hAdau1962a, ADI_ADAU1962A_REG_DAC_POWER3         ,&valReg_dac_power3         );


    printf("\n ADI_ADAU1962A_REG_PLL_CLK_CTRL0      = 0x%02x",    valReg_pll_clk_ctrl0      );
    printf("\n ADI_ADAU1962A_REG_PLL_CLK_CTRL1      = 0x%02x",    valReg_pll_clk_ctrl1      );
    printf("\n ADI_ADAU1962A_REG_PDN_THRMSENS_CTRL_1= 0x%02x",    valReg_pdn_thrmsens_ctrl_1);
    printf("\n ADI_ADAU1962A_REG_PDN_CTRL2          = 0x%02x",    valReg_pdn_ctrl2          );
    printf("\n ADI_ADAU1962A_REG_PDN_CTRL3          = 0x%02x",    valReg_pdn_ctrl3          );
    printf("\n ADI_ADAU1962A_REG_THRM_TEMP_STAT     = 0x%02x",    valReg_thrm_temp_stat     );
    printf("\n ADI_ADAU1962A_REG_DAC_CTRL0          = 0x%02x",    valReg_dac_ctrl0          );
    printf("\n ADI_ADAU1962A_REG_DAC_CTRL1          = 0x%02x",    valReg_dac_ctrl1          );
    printf("\n ADI_ADAU1962A_REG_DAC_CTRL2          = 0x%02x",    valReg_dac_ctrl2          );
    printf("\n ADI_ADAU1962A_REG_DAC_MUTE1          = 0x%02x",    valReg_dac_mute1          );
    printf("\n ADI_ADAU1962A_REG_DAC_MUTE2          = 0x%02x",    valReg_dac_mute2          );
    printf("\n ADI_ADAU1962A_REG_DACMSTR_VOL        = 0x%02x",    valReg_dacmstr_vol        );
    printf("\n ADI_ADAU1962A_REG_DAC1_VOL           = 0x%02x",    valReg_dac1_vol           );
    printf("\n ADI_ADAU1962A_REG_DAC2_VOL           = 0x%02x",    valReg_dac2_vol           );
    printf("\n ADI_ADAU1962A_REG_DAC3_VOL           = 0x%02x",    valReg_dac3_vol           );
    printf("\n ADI_ADAU1962A_REG_DAC4_VOL           = 0x%02x",    valReg_dac4_vol           );
    printf("\n ADI_ADAU1962A_REG_DAC5_VOL           = 0x%02x",    valReg_dac5_vol           );
    printf("\n ADI_ADAU1962A_REG_DAC6_VOL           = 0x%02x",    valReg_dac6_vol           );
    printf("\n ADI_ADAU1962A_REG_DAC7_VOL           = 0x%02x",    valReg_dac7_vol           );
    printf("\n ADI_ADAU1962A_REG_DAC8_VOL           = 0x%02x",    valReg_dac8_vol           );
    printf("\n ADI_ADAU1962A_REG_DAC9_VOL           = 0x%02x",    valReg_dac9_vol           );
    printf("\n ADI_ADAU1962A_REG_DAC10_VOL          = 0x%02x",    valReg_dac10_vol          );
    printf("\n ADI_ADAU1962A_REG_DAC11_VOL          = 0x%02x",    valReg_dac11_vol          );
    printf("\n ADI_ADAU1962A_REG_DAC12_VOL          = 0x%02x",    valReg_dac12_vol          );
    printf("\n ADI_ADAU1962A_REG_PAD_STRGTH         = 0x%02x",    valReg_pad_strgth         );
    printf("\n ADI_ADAU1962A_REG_DAC_POWER1         = 0x%02x",    valReg_dac_power1         );
    printf("\n ADI_ADAU1962A_REG_DAC_POWER2         = 0x%02x",    valReg_dac_power2         );
    printf("\n ADI_ADAU1962A_REG_DAC_POWER3         = 0x%02x",    valReg_dac_power3         );
}
#endif

/*
 *  @}
 */

/*
 *
 * EOF:
 *
*/
