#!/usr/bin/env python

#============================================================================== 
sHelpText = """
Simple script to add more detail and/or cleanup the A2B I2C UML.  

Features Include:
- Register addr decode
- Some register values decoded
- Errors shown in red with salmon colored notes to the left for easy detection
- Interrupt polling will be blue and bolded when interrupt actually detected
- Some less cared about groups can be collapsed (-c)
- I2C successful responses can be removed to make it smaller and easier to read (-s)
- Some stats are tracked when processed (error, irqCount, duration, etc)
- Starting discovery note changed to use readable text to show Simple, Modified, etc
- ALL a2b_i2cWrite errors are logged to stdout with their line number.
- Can process multiple files using a wildcard for the filename 
  NOTE: (be sure to quote the complete filename on the command line)
- Can bold/color specific text to make debugging and finding things easier.
- Will append a missing "@enduml" in case it was missing due to failed seqClose
- Will append a missing "end" in case it was missing due to failed seqClose or 
  coding problem with a group/end.
- Adds color to some lines for easier reading/debugging

./postProcessUML.py -b | -c | -f [uml_filename] | -p | -s

-b  Bold some text.  This is a simple find and bold option.  This makes it 
    easier during some debugging to see parts of text or register names 
    you are interested in. Space separate for more than one search name.
    For example,
    -b "SWCTL" or -b "SWCTL(=09)" "NODEARDR"
    NOTE: Each string searched gets a different color. Also the replace is
          done after ALL other cleanup has been made (i.e. other color 
          coding, etc); so adjustments may be needed to find the match.
    NOTE: The processed UML will add a legend that will contain all the 
          keywords that were bolded and show the colors for each.
    (".b" appended to filename)

-s  Shorten/simple mode.  This will remove "Success" logging for all
    a2b_i2cWrite calls--any errors are retained.  This reduces the noise
    when you are not interested in this information.
    (".s" appended to filename)

-c  Collapse some groups that can be noise during debugging and
    verifying the flow of discovery, etc.  NOTE: collapsed groups will
    have "[collapsed]" to the right of the group label to indicate they 
    were not normally empty.  Also, if the group contains notes they are
    NOT removed.  To remove notes as well, use "-C".
    (".c" appended to filename)

-C  Same as "-c" except it even removes notes contained within the group.

-p  [Experimental] Suppress all IRQ polling that does not read a triggered 
    interrupt.  This can clean up useless polling for some debugging.  
    (".p" appended to filename)
    CAVEAT:  If timestamping is enabled the cleaned UML file will still contain
             a note for each poll but the generated PNG will not show this.
    WARNING: PlantUML has a bug that will show the wrong timestamp for the 
             message that is just prior to a delay.  They are looking into the
             problem and hopefully a solution will be offered soon--thus this
             option is considered "experimental"

-t  Used to remove timestamp information.  This can be useful when diffing 
    two UML files for content.
    (".t" appended to filename)

Example: ./postProcessUML.py -f /tmp/something.uml
         will generate: /tmp/something.detailed.uml (without options)
         will generate: /tmp/something.detailed.s.uml (shorten mode)
         will generate: /tmp/something.detailed.c.uml (collapse mode)
         will generate: /tmp/something.detailed.s.c.uml (shorten & collapse mode)

         ./postProcessUML.py -f "/tmp/something*.uml"
         will generate cleaned files for all something files

Sample output:
  Plugin -[#blue]>> Platform : a2b_i2cWrite(0x68, 2) -> 01 00
  converted to:
  Plugin -[#blue]>> Platform : a2b_i2cWrite(0x68, 2) -> NODEADR(=01) val= 00

  Plugin -[#blue]>> Platform : a2b_i2cWrite(0x68, 4) -> 1B 10 00 09
  converted to:
  Plugin -[#blue]>> Platform : a2b_i2cWrite(0x68, 4) -> INTMSK0(=1B) val= 10 00 09

  Plugin -[#blue]>> Platform : a2b_i2cWriteRead(0x68, 1, 4) -> 14
  converted to:
  Plugin -[#blue]>> Platform : a2b_i2cWriteRead(0x68, 1, 4) -> SWSTAT(=14)

  Example with triggered interrupt: (bolded and colored decode)
  Platform -[#blue]->> Plugin : a2b_i2cWriteRead(0x68, 1, 4) <- <color:blue><b>(INT=YES,MSTINT=YES,TYPE=MSTR_RUNNING)</color></b>(00 01 80 FF)

  Example WITOUT triggered interrupt: (normal black coloring)
  Platform -[#blue]->> Plugin : a2b_i2cWriteRead(0x68, 1, 4) <- (INT=YES,MSTINT=YES,TYPE=MSTR_RUNNING)(00 01 80 FF)

"""
#============================================================================== 

import os
import sys
import re
import glob

import argparse

APP_VER = "v2.9"

SLAVE_ADDR = "0x69"

mmrPage = ""

SPICMDMAP = {
"00" : "LOCAL WRITE",
"01" : "LOCAL READ",
"0A" : "ABORT",
"05" : "BUS FIFO READ",  
"04" : "STATUS READ",
"0B" : "TUNNEL FIFO READ",
"02" : "REMOTE REGISTER WRITE",
"C0" : "REMOTE REGISTER READ REQUEST",
"C1" : "REMOTE REGISTER READ REQUEST",
"C2" : "REMOTE REGISTER READ REQUEST",
"C3" : "REMOTE REGISTER READ REQUEST",
"03" : "TUNNEL ATOMIC WRITE",
"0D" : "TUNNEL ATOMIC READ REQUEST",
"09" : "FULL DUPLEX",
"06" : "BULK",
"07" : "SPI TO I2C WRITE", 
"08" : "SPI TO I2C READ",
"0E" : "EXTENDED FULL DUPLEX", 
"0F" : "EXTENDED BULK"
    
}

REGMAP = {
"00" : "CHIP",
"01" : "NODEADR",
"02" : "VENDOR",
"03" : "PRODUCT",
"04" : "VERSION",
"05" : "CAPABILITY",
"09" : "SWCTL",
"0A" : "BCDNSLOTS",
"0B" : "LDNSLOTS",
"0C" : "LUPSLOTS",
"0D" : "DNSLOTS",
"0E" : "UPSLOTS",
"0F" : "RESPCYCS",
"10" : "SLOTFMT",
"11" : "DATCTL",
"12" : "CONTROL",
"13" : "DISCVRY",
"14" : "SWSTAT",
"15" : "INTSTAT",
"16" : "INTSRC",
"17" : "INTTYPE",
"18" : "INTPND0",
"19" : "INTPND1",
"1A" : "INTPND2",
"1B" : "INTMSK0",
"1C" : "INTMSK1",
"1D" : "INTMSK2",
"1E" : "BECCTL",
"1F" : "BECNT",
"20" : "TESTMODE",
"21" : "ERRCNT0",
"22" : "ERRCNT1",
"23" : "ERRCNT2",
"24" : "ERRCNT3",

"25" : "SEED0",
"26" : "SEED1",
"27" : "SEED2",
"28" : "SEED3",
"29" : "NODE",
"2A" : "TRANSTAT",
"2B" : "DISCSTAT",
"2C" : "NSCURCNT",
"2D" : "VREGCTL",
"2E" : "TXACTL",
"2F" : "RXACTL",
"30" : "TXBCTL",
"31" : "RXBCTL",
"32" : "RESPCCC",
"33" : "DCTLCC",
"34" : "DNSCC",
"35" : "LDNSCC",
"36" : "UPSCC",
"37" : "LUPSCC",
"38" : "BCDNSCC",
"39" : "SFMTCC",
"3A" : "PTSTMODE",
"3B" : "STRAPVAL",

"3E" : "LINTTYPE",
"3F" : "I2CCFG",
"40" : "PLLCTL",
"41" : "I2SGCFG",
"42" : "I2SCFG",
"43" : "I2SRATE",
"44" : "I2STXOFFSET",
"45" : "I2SRXOFFSET",
"46" : "SYNCOFFSET",
"47" : "PDMCTL",
"48" : "ERRMGMT",
"49" : "CLKCFG",
"4A" : "GPIODAT",
"4B" : "GPIODATSET",
"4C" : "GPIODATCLR",
"4D" : "GPIOOEN",
"4E" : "GPIOIEN",
"4F" : "GPIOIN",
"50" : "PINTEN",
"51" : "PINTINV",
"52" : "PINCFG",
"53" : "I2STEST",
"54" : "RAISE",
"55" : "GENERR",
"56" : "I2SRRATE",
"57" : "I2SRRCTL",
"58" : "I2SRRSOFFS",
"59" : "CLK1CFG",
"5A" : "CLK2CFG",
"5B" : "BMMCFG",
"5C" : "SUSCFG",
"5D" : "PDMCTL2",
"5E" : "BSDSTAT",

"60" : "UPMASK0",
"61" : "UPMASK1",
"62" : "UPMASK2",
"63" : "UPMASK3",
"64" : "UPOFFSET",
"65" : "DNMASK0",
"66" : "DNMASK1",
"67" : "DNMASK2",
"68" : "DNMASK3",
"69" : "DNOFFSET",

"6A" : "CHIPID0",
"6B" : "CHIPID1",
"6C" : "CHIPID2",
"6D" : "CHIPID3",
"6E" : "CHIPID4",
"6F" : "CHIPID5",
"7C" : "DTCFG",
"7D" : "DTSLOTS",
"7E" : "DTDNOFFS",
"7F" : "DTUPOFFS",


"80" : "GPIODEN",
"81" : "GPIOD0MSK",
"82" : "GPIOD1MSK",
"83" : "GPIOD2MSK",
"84" : "GPIOD3MSK",
"85" : "GPIOD4MSK",
"86" : "GPIOD5MSK",
"87" : "GPIOD6MSK",
"88" : "GPIOD7MSK",
"89" : "GPIODDAT",
"8A" : "GPIODINV",

"90" : "MBOX0CTL",
"91" : "MBOX0STAT",
"92" : "MBOX0B0",
"93" : "MBOX0B1",
"94" : "MBOX0B2",
"95" : "MBOX0B3",
"96" : "MBOX1CTL",
"97" : "MBOX1STAT",
"98" : "MBOX1B0",
"99" : "MBOX1B1",
"9A" : "MBOX1B2",
"9B" : "MBOX1B3",

"A0" : "SWCTL2",
"A3" : "SWCTL5",
"A5" : "SWSTAT2",
"AF" : "SPIDTLCMD",
"B0" : "SPICFG",
"B1" : "SPISTAT",
"B2" : "SPICKDIV",
"B3" : "SPIFDSIZE",
"B4" : "SPIFDTARG",
"B5" : "SPIPINCFG",
"B6" : "SPIINT",
"B7" : "SPIMSK",
"B8" : "RXMASK0",
"B9" : "RXMASK1",
"BA" : "RXMASK2",
"BB" : "RXMASK3",
"BC" : "RXMASK4",
"BD" : "RXMASK5",
"BE" : "RXMASK6",
"BF" : "RXMASK7",
"C0" : "TXXBAR0",
"C1" : "TXXBAR1",
"C2" : "TXXBAR2",
"C3" : "TXXBAR3",
"C4" : "TXXBAR4",
"C5" : "TXXBAR5",
"C6" : "TXXBAR6",
"C7" : "TXXBAR7",
"C8" : "TXXBAR8",
"C9" : "TXXBAR9",
"CA" : "TXXBAR10",
"CB" : "TXXBAR11",
"CC" : "TXXBAR12",
"CD" : "TXXBAR13",
"CE" : "TXXBAR14",
"CF" : "TXXBAR15",
"D0" : "TXXBAR16",
"D1" : "TXXBAR17",
"D2" : "TXXBAR18",
"D3" : "TXXBAR19",
"D4" : "TXXBAR20",
"D5" : "TXXBAR21",
"D6" : "TXXBAR22",
"D7" : "TXXBAR23",
"D8" : "TXXBAR24",
"D9" : "TXXBAR25",
"DA" : "TXXBAR26",
"DB" : "TXXBAR27",
"DC" : "TXXBAR28",
"DD" : "TXXBAR29",
"DE" : "TXXBAR30",
"DF" : "TXXBAR31",
"E0" : "MMRPAGE0",

"100" : "VMTR_VEN",
"101" : "VMTR_INTEN",
"102" : "VMTR_MXSTAT",
"103" : "VMTR_MNSTAT",
"120" : "VMTR_VLTG0",
"121" : "VMTR_VMAX0",
"122" : "VMTR_VMIN0",
"123" : "VMTR_VLTG1",
"124" : "VMTR_VMAX1",
"125" : "VMTR_VMIN1",
"126" : "VMTR_VLTG2",
"127" : "VMTR_VMAX2",
"128" : "VMTR_VMIN2",
"129" : "VMTR_VLTG3",
"12A" : "VMTR_VMAX3",
"12B" : "VMTR_VMIN3",
"12C" : "VMTR_VLTG4",
"12D" : "VMTR_VMAX4",
"12E" : "VMTR_VMIN4",
"12F" : "VMTR_VLTG5",
"130" : "VMTR_VMAX5",
"131" : "VMTR_VMIN5",
"132" : "VMTR_VLTG6",
"133" : "VMTR_VMAX6",
"134" : "VMTR_VMIN6",
"140" : "PWMCFG",
"141" : "PWMFREQ",
"142" : "PWMBLINK1",
"143" : "PWMBLINK2",
"148" : "PWM1VALL",
"149" : "PWM1VALH",
"14A" : "PWM2VALL",
"14B" : "PWM2VALH",
"14C" : "PWM3VALL",
"14D" : "PWM3VALH",
"14E" : "PWMOEVALL",
"14F" : "PWMOEVALH",
"1E0" : "MMRPAGE1"


}

COLLAPSE_GROUPS = [
"Interrupt Registers",
"Pin I/O Registers",
"I2C/I2S Registers",
"Control Registers",
"^nodeAddr \d*, cfgBlock \d*$"
]

INTTYPE_MAP = {
"00"  : "HDCNTERR",
"01"  : "DDERR",
"02"  : "CRCERR",
"03"  : "DPERR",
"04"  : "BECOVF",
"05"  : "SRFERR",
"09"  : "PWRERR (Cable Shorted to GND)",
"0A" : "PWRERR (Cable Shorted to VBat)",
"0B" : "PWRERR (Cable Shorted Together)",
"0C" : "PWRERR (Cable Disconnected or Open Circuit)",
"0D" : "PWRERR (Cable Reverse Connected)",
"0D" : "PWRERR (Cable Reverse Connected)",
"0E" : "PWRERR (Open circuit or Reverse connected)", #AD243x specific
"10" : "IO0PND", 
"11" : "IO1PND", 
"12" : "IO2PND", 
"13" : "IO3PND",
"14" : "IO4PND",
"15" : "IO5PND",
"16" : "IO6PND",
"17" : "IO7PND",
"18" : "DSCDONE", # Master Only
"19" : "I2CERR",  # Master Only
"1A" : "ICRCERR", # Master Only
"29" : "PWRERR - Non-Localized Short to GND",
"2A" : "PWRERR - Non-Localized Short to VBat",
"30" : "Mailbox 0 full",
"31" : "Mailbox 0 empty",
"32" : "Mailbox 1 full",
"33" : "Mailbox 1 empty",
"40" : "SPI done",
"41" : "SPI remote register access error", # Master only 
"42" : "SPI remote I2C access error", # Master only 
"43" : "SPI Data Tunnel Access Error", 
"44" : "SPI Bad Command", 
"45" : "SPI FIFO Overflow", 
"46" : "SPI FIFO Underflow",
"50" : "VMTR Interrupt",
"80" : "INTERR",
"FC" : "Start up error", # - Return to Factory
"FD" : "Slave INTTYPE Read Error", # - Master Only
"FE" : "Standby Done",# - Master Only
"FF" : "MSTR_RUNNING" # - Master Only
}

# Color name for the "-b" option.  Refer to:
# http://www.w3schools.com/htmL/html_colornames.asp
# The default if the list is too large gets [0].
BOLD_COLORS=["fuchsia", "orange", "lime",    "navy",  
             "teal",    "olive",  "purple",  "yellow",  
             "green",   "aqua",   "silver",  "gray",  
             "maroon",  "blue",   "black",   "red"]

g_stats = {"irqCount" : 0, "errors" : 0, "duration(msec)":0, "bolded":[]}

from argparse import RawTextHelpFormatter
parser = argparse.ArgumentParser(description=sHelpText, formatter_class=RawTextHelpFormatter)
parser.add_argument('-b', '--bold',                             default=None,   help='Bold selected text (space separate)', nargs='*')
parser.add_argument('-c', '--collapse',    action='store_true', default=False,  help='Collapse some groups [keeps notes]')
parser.add_argument('-C', '--collapseall', action='store_true', default=False,  help='Collapse some groups [deletes notes]')
parser.add_argument('-f', '--filename',                         default=None,   help='UML filename to process')
parser.add_argument('-p', '--irqpolloff',  action='store_true', default=False,  help='Suppress IRQ polling')
parser.add_argument('-s', '--shorten',     action='store_true', default=False,  help='Shorten/simple mode')
parser.add_argument('-t', '--notimestamp', action='store_true', default=False,  help='Remove timestamps')
parser.add_argument('--version',           action='version',    version=APP_VER)
g_args = parser.parse_args()

print "\nCommand line options:"
print g_args
print ""

if g_args.collapseall:
    # Collapse all adds to collapse mode
    g_args.collapse = True

if g_args.bold:
    # Create a stat for each bold keyword
    for idx in range(len(g_args.bold)):
        g_stats["bolded"].append(0)
else:
    del g_stats["bolded"]

# Arg processing
#----------------
if g_args.filename == None:
    print ""
    print sHelpText
    print ""
    sys.exit(1)


def colorize ( line ):
    """-------------------------------------------------------------
      colorize

      bold and colorize the user defined text.

      @tparam   (string)  [in]    line

      @treturn  (string) updated line
    --------------------------------------------------------------"""
    global BOLD_COLORS
    global g_stats

    idx = 0
    for boldKey in g_args.bold:
        try:
            color = BOLD_COLORS[idx]
        except:
            color = BOLD_COLORS[0]
            idx   = 0
        newline = line.replace(boldKey, "<color:"+color+"><b>"+boldKey+"</b></color>")
        if newline != line:
            # Ignore stating the legend
            if line.find("NOTE: Post processed using:") == -1:
                g_stats["bolded"][idx] += 1
            line = newline
        idx += 1

    return line


def ProcessFile ( filename ):
    """-------------------------------------------------------------
      ProcessFile

      [add description here]

      @tparam   (string)  [in]    filename

      @treturn  (boolean) True=success, False=failure
    --------------------------------------------------------------"""

    global SPICMDMAP
    global REGMAP
    global INTTYPE_MAP
    global COLLAPSE_GROUPS
    global BOLD_COLORS
    global g_args
    global g_stats
    global mmrPage
    


    # Open Src File
    #---------------

    try:
        rfp = open(filename,'r')
    except:
        print "\nERROR: Unable to open:", filename, "\n"
        return False

    # Open Destination File
    #-----------------------

    name, ext = os.path.splitext( filename )
    filename = name+".detailed"
    if g_args.bold:
        filename += ".b"
    if g_args.shorten:
        filename += ".s"
    if g_args.collapseall:
        filename += ".C"
    elif g_args.collapse:
        filename += ".c"
    if g_args.notimestamp:
        filename += ".t"
    if g_args.irqpolloff:
        filename += ".p"
    filename += ext

    wfp = open(filename, 'w')

    print "\nGenerating:", filename

    lineCount = 0
    bWriteLine = False
    bSkipTS = False
    bErrorTS = False
    sLine = ""
    bWithinGroup = False
    bWithinNote = False
    prevReadRegAddr = "" # Register previously read
    startDuration = None
    endDuration = 0
    i2cWriteRead = ""
    titleFound = None
    hasEndUml = False
    groupCount = 0

    
    for line in rfp.readlines():

        lineCount += 1
        bWriteLine = True

        # Handle the 0a0d, rest of the script expects the \n
        line = line.strip()+"\n";

        mObj = re.match("^== Starting Discovery Mode (\d).*", line)
        if mObj:
            modeStr = "Unknown"
            modeVal = mObj.group(1)
            if modeVal == "0":
                modeStr = "Simple"
            elif modeVal == "1":
                modeStr = "Modified"
            elif modeVal == "2":
                modeStr = "Optimized"
            elif modeVal == "3":
                modeStr = "Advanced"
            line = "== Starting Discovery Mode %s (%s) ==\n" % \
                   (modeVal, modeStr)

        if titleFound == None:
            mObj = re.match("^title .*", line)
            if mObj:
                line += "\nlegend center\n"
                sargs = str(g_args).replace("Namespace", "")
                sargs = sargs.replace("'", "")
                sargs = sargs.replace(",", "\n")
                sCmd = os.path.basename(sys.argv[0])
                line += "NOTE: Post processed using: <b>%s %s</b> options:\n%s\n" % \
                        ( sCmd, APP_VER, sargs )
                line += "endlegend\n"

        mObj = re.match("(.*)nodeAddr -1(.*)", line)
        if mObj:
            line = "%snodeAddr -1 (Master)%s\n" % (mObj.group(1), mObj.group(2))
            mObj = None

        mObj = re.match(".*\) <- Error:", line)
        if mObj:
            # Show errors in stdout and change to RED line
            print "Line %5d: %s" % (lineCount, line.strip())
            line = line.replace("[#blue]", "[#red]")
            bErrorTS = True
            prevReadRegAddr = ""
            g_stats["errors"] += 1

        if mObj == None:
            # Look for the timestamp to remove in shorten mode
            mObj = re.match("note left: TS\=(.*)", line)
            if mObj:

                if startDuration == None:
                    startDuration = int(mObj.group(1))
                endDuration = int(mObj.group(1))
                    
                if g_args.notimestamp or bSkipTS:
                    bWriteLine = False
                elif bErrorTS:
                    line = line.replace("note left:", "note left #FFAAAA:")
                    bErrorTS = False

            else:
                bErrorTS = False

            if bWriteLine:
                
                # Track groups so we can look for errors
                bIsEnd = False
                mObj   = re.match("group (.*)", line)
                if mObj:
                    groupCount += 1
                elif re.match("^end$", line):
                    bIsEnd = True
                    groupCount -= 1

                # Check to see if this is a group that needs collapsing
                if g_args.collapse:

                    if mObj:
                        groupName = mObj.group(1)
                        for keyGroupName in COLLAPSE_GROUPS:
                            mObj = re.match(keyGroupName, groupName)
                            if mObj:
                                bWithinGroup = True
                                # indicate the group was collapsed and not normally empty
                                line = "group %s [collapsed]\n" % groupName
                                break

                    elif bWithinGroup:
                        # Look for the end
                        if bIsEnd:
                            bWithinGroup = False
                        else:
                            # do not write out this line
                            bWriteLine = False

                            if not g_args.collapseall:
                                # If a note is contained within, keep it
                                mObj = re.match("note right of.*", line)
                                if mObj:
                                    bWithinNote = True

                                elif re.match("^end note$", line):
                                    bWithinNote = False
                                    bWriteLine  = True

                                if bWithinNote:
                                    bWriteLine  = True

        bSkipTS = False

        if mObj == None and bWriteLine:
            # Add some color coding to make it pop and make it easier to read
            mObj = re.match("== Plugin(\d*) Mock: A2B_MSGREQ_PLUGIN_PERIPH_(.*) Processed ==", line)
            if mObj:
                line = "== <color blue>Plugin%s Mock:</color> A2B_MSGREQ_PLUGIN_PERIPH_%s Processed ==\n" % \
                       (mObj.group(1), mObj.group(2))
            else:
                mObj = re.match("== Slave(\d*) Mock: A2B_MSGREQ_PLUGIN_PERIPH_(.*) Processed ==", line)
                if mObj:
                    line = "== <color blue>Slave%s Mock:</color> A2B_MSGREQ_PLUGIN_PERIPH_%s Processed ==\n" % \
                           (mObj.group(1), mObj.group(2))


        if mObj == None and bWriteLine:
            mObj = re.match(".*a2b_i2cWriteRead\(.*, (\d*), \d*\) -> (..)", line)
            if mObj:
                readCnt   = mObj.group(1)
                regAddr   = mObj.group(2)
                if readCnt == "1" and regAddr in REGMAP:
                    line = line.replace(") -> "+regAddr, 
                                       ") -> <b>%s</b>(=%s)" % (REGMAP[regAddr], regAddr))
                    prevReadRegAddr = regAddr

                    # Disable if "-p" option is enabled AND
                    # if we have a register that we will decode the 
                    # response data.
                    if g_args.irqpolloff or regAddr in ["16", "17"]:
                        i2cWriteRead = line
                        bWriteLine   = False # wait to see if we should log this
        if mObj == None and bWriteLine:
            mObj = re.match(".*a2b_spiWriteRead\(.*, (\d*), (\d*)\) -> (.*)", line)
            if mObj:
                writeCnt   = mObj.group(1)
                readCnt   = mObj.group(2)
                data = mObj.group(3).split()
                spiCmd   = data[0]
                if SPICMDMAP[spiCmd] == "STATUS READ":
                    regAddr = ""
                    if readCnt == "1":
                        line = line.replace("[#blue]", "[#gray]")
                        line = line.replace(") -> "+ spiCmd, 
                                       ") -> %s(=%s)" % (SPICMDMAP[spiCmd], spiCmd))
                elif SPICMDMAP[spiCmd] == "LOCAL READ":
                    regAddr = data[1]
                    prevReadRegAddr = regAddr
                    if  regAddr in REGMAP:
                        line = line.replace(") -> "+ spiCmd + " " + regAddr, 
                                       ") -> %s <b>%s</b>(=%s)" % (spiCmd, REGMAP[regAddr], regAddr))
                elif SPICMDMAP[spiCmd] == "BUS FIFO READ":
                    regAddr = prevReadRegAddr
                    if  regAddr in REGMAP:
                        line = line.replace(") -> "+ spiCmd, 
                                       ") -> %s(=%s) for <b>%s</b>" % (SPICMDMAP[spiCmd], spiCmd, REGMAP[regAddr]))

        if mObj == None and bWriteLine and prevReadRegAddr != "":
            mObj = re.match(".*a2b_spiWriteRead\(.*, (\d*), (\d*)\) <- (.*)", line)
            if mObj:
                writeCnt   = mObj.group(1)
                readCnt   = mObj.group(2)
                data = mObj.group(3).split()                
                if SPICMDMAP[spiCmd] == "STATUS READ":
                    if readCnt == "1":
                        if data[0] == "00":
                            line = line.replace("[#blue]", "[#gray]")
                            line = line.replace(") <- "+ mObj.group(3), 
                                       ") <- <color:green><b>%s</b></color>(=%s)" % ("SPI Done", data[0]))
                        else:
                            line = line.replace(") <- "+ mObj.group(3), 
                                       ") <- <color:yellow><b>%s</b></color>(=%s)" % ("SPI Busy", data[0]))                            
                elif SPICMDMAP[spiCmd] == "LOCAL READ":
                    if prevReadRegAddr == "16":
                        # INTSRC 
                        dataVal    = int("0x"+data[0], 0)
                        boldDecode = True
                        if dataVal & 0x80:
                            decode = "MST_INT=YES"
                        elif dataVal & 0x40:
                            decode = "SLV%d_INT=YES" % (dataVal & 0x0F)
                        elif dataVal == 0:
                            boldDecode = False
                            decode = "NO IRQ"
                            if g_args.irqpolloff:
                                # Suppress this IRQ poll request
                                bWriteLine = False
                        else:
                            boldDecode = False
                            
                        if boldDecode == True:
                            line = line.replace(") <- "+ mObj.group(3), 
                                       ") <- <color:blue><b>%s</b></color>(=%s)" % (decode, data[0]))                           
                    elif prevReadRegAddr == "17":
                        # INTTYPE
                        if data[0] in INTTYPE_MAP:
                            decode = "TYPE=%s" % INTTYPE_MAP[data[0]]
                        else:
                            decode = "TYPE=%s" % data[0]
                        line = line.replace(") <- "+ mObj.group(3), 
                                       ") <- <color:blue><b>%s</b></color>(=%s)" % (decode, data[0]))    
        if mObj == None and bWriteLine and prevReadRegAddr != "":
            mObj = re.match(".*a2b_i2cWriteRead\(.*, \d*, (\d*)\) <- (.*)", line)
            if mObj:
                if prevReadRegAddr == "14" and mObj.group(1) == "4": 
                    # SWSTAT decode
                    data = mObj.group(2).split()
                    SWSTAT  = data[0]
                    INTSTAT = data[1]
                    INTSRC  = data[2]
                    INTTYPE = data[3]

                    decode  = []
                    irq     = False
                    dataVal = int("0x"+INTSTAT, 0)
                    if dataVal & 0x01:
                        decode.append("INT=YES")
                        irq = True
                        g_stats["irqCount"] += 1
                    else:
                        decode.append("INT=no")

                    dataVal = int("0x"+INTSRC, 0)
                    if dataVal & 0x80:
                        decode.append("MST_INT=YES")
                    elif dataVal & 0x40:
                        decode.append("SLV%d_INT=YES" % (dataVal & 0x0F))

                    if INTTYPE in INTTYPE_MAP:
                        decode.append("TYPE=%s" % INTTYPE_MAP[INTTYPE])
                    else:
                        decode.append("TYPE=%s" % INTTYPE)

                    mObj = re.match("(.*)a2b_i2cWriteRead(.*) <- (.*)", line)
                    if mObj:
                        if irq:
                            line = ""

                            # See if we need to add the previous line to the log
                            if g_args.irqpolloff:
                                line = i2cWriteRead
                                if endDuration != 0 and g_args.notimestamp == False:
                                    line += "note left: TS=%010d\n" % endDuration

                            # Bold and color important decoded text
                            # NOTE: For some reason the "</b>" was being ignored
                            #       and shown as normal text.  PlantUML bug.
                            #line += "%sa2b_i2cWriteRead%s <- <color:blue><b>(%s)</color></b> (%s)\n" % \
                            line += "%sa2b_i2cWriteRead%s <->  (%s)=<color:blue><b>(%s)</b></color>\n" % \
                                   (mObj.group(1), mObj.group(2), \
                                   mObj.group(3), ",".join(decode))

                        else:
                            if g_args.irqpolloff == False:
                                line = "%sa2b_i2cWriteRead%s <-> (%s)=(%s)\n" % \
                                       (mObj.group(1), mObj.group(2), \
                                       mObj.group(3), ",".join(decode))
                            else:
                                # Suppress this IRQ poll request
                                bWriteLine   = False

                elif prevReadRegAddr != "" and mObj.group(1) == "1":

                    decode     = ""
                    data       = mObj.group(2)
                    boldDecode = False

                    if prevReadRegAddr == "16":
                        # INTSRC 
                        dataVal    = int("0x"+data, 0)
                        boldDecode = True
                        if dataVal & 0x80:
                            decode = "MST_INT=YES"
                        elif dataVal & 0x40:
                            decode = "SLV%d_INT=YES" % (dataVal & 0x0F)
                        elif dataVal == 0:
                            boldDecode = False
                            decode = "NO IRQ"

                            if g_args.irqpolloff:
                                # Suppress this IRQ poll request
                                bWriteLine = False
                        else:
                            boldDecode = False

                    elif prevReadRegAddr == "17":
                        # INTTYPE
                        boldDecode = True
                        if data in INTTYPE_MAP:
                            decode = "TYPE=%s" % INTTYPE_MAP[data]
                        else:
                            decode = "TYPE=%s" % data

                    if bWriteLine:
                        if decode != "":
                            if boldDecode:
                                line = line.replace(") <- "+data, 
                                ") <-> %s(=%s) val= (%s)=<color:blue><b>(%s)</b></color>" % (REGMAP[regAddr], prevReadRegAddr, data, decode))
                            else:
                                line = line.replace(") <- "+data, 
                                ") <-> %s(=%s) val= (%s)=(%s)" % (REGMAP[regAddr], prevReadRegAddr, data, decode))
                        else:
                            line = line.replace(") <- "+data, 
                            ") <-> %s(=%s) val= (%s)" % (REGMAP[regAddr], prevReadRegAddr, data))

                prevReadRegAddr = ""
                i2cWriteRead    = ""

        
        if mObj == None and bWriteLine:
            mObj = re.match(".*a2b_i2cWrite\((.*), (\d*)\) -> (..)", line)
            if mObj:
                i2cAddr = mObj.group(1)
                readCnt = int(mObj.group(2))
                regAddr = mObj.group(3)
                if regAddr in REGMAP:
                    if i2cAddr == SLAVE_ADDR and readCnt > 2:
                        # Leave the line alone since it is LIKELY
                        # a write to a peripheral that we cannot decode
                        pass

                    elif readCnt != 1:
                        line = line.replace(") -> "+regAddr, 
                               ") -> <b>%s</b>(=%s) val=" % (REGMAP[mmrPage + regAddr],regAddr))

                        # add further detail for some data
                        #---------------------------------
                        if regAddr == "E0":
                            # A2B_MMRPAGE
                            mObj = re.match(".*val= (..)", line)
                            if mObj:
                                dataVal = mObj.group(1)
                                if dataVal == "01":
                                    mmrPage = "1"
                                else:
                                    mmrPage = ""

                        elif regAddr == "01":
                            # A2B_NODEADR
                            mObj = re.match(".*val= (..)", line)
                            if mObj:
                                dataVal = int("0x"+mObj.group(1), 0)
                                decode  = []
                                if dataVal & 0x80:
                                    decode.append("BCAST")

                                if dataVal & 0x20:
                                    decode.append("PERI")

                                decode.append("NODE=%d" % (dataVal & 0x0F))

                                line = line.replace("val= "+mObj.group(1),
                                                "val= (%s)=(%s)" \
                                                % (mObj.group(1),
                                                  ",".join(decode)))

                        elif regAddr == "09":
                            # A2B_SWCTL
                            mObj = re.match(".*val= (..)", line)
                            if mObj:
                                dataVal = int("0x"+mObj.group(1), 0)
                                decode  = []
                                if dataVal & 0x01:
                                    decode.append("ENSW=ON")
                                else:
                                    decode.append("ENSW=off")

                                if dataVal & 0x02:
                                    decode.append("DIAG=ON")
                                else:
                                    decode.append("DIAG=off")

                                if (dataVal & 0x30) == 0:
                                    decode.append("MODE=0")
                                elif (dataVal & 0x30) == 0x30:
                                    decode.append("MODE=3")
                                elif (dataVal & 0x10):
                                    decode.append("MODE=1")
                                elif (dataVal & 0x20):
                                    decode.append("MODE=2")

                                line = line.replace("val= "+mObj.group(1),
                                                "val= (%s)=(%s)" \
                                                % (mObj.group(1),
                                                  ",".join(decode)))

                        elif regAddr == "11":
                            # A2B_DATCTL
                            mObj = re.match(".*val= (..)", line)
                            if mObj:
                                dataVal = int("0x"+mObj.group(1), 0)
                                decode  = []
                                if dataVal & 0x01:
                                    decode.append("UPS=ON")
                                else:
                                    decode.append("UPS=off")

                                if dataVal & 0x02:
                                    decode.append("DNS=ON")
                                else:
                                    decode.append("DNS=off")

                                if dataVal & 0x20:
                                    decode.append("ENDSNIFF=ON")
                                else:
                                    decode.append("ENDSNIFF=off")

                                if dataVal & 0x80:
                                    decode.append("STANDBY=ON")
                                else:
                                    decode.append("STANDBY=off")

                                line = line.replace("val= "+mObj.group(1),
                                                "val= (%s)=(%s)" \
                                                % (mObj.group(1),
                                                  ",".join(decode)))

                        elif regAddr == "12":
                            # A2B_CONTROL
                            mObj = re.match(".*val= (..)", line)
                            if mObj:
                                dataVal = mObj.group(1)
                                if dataVal == "04":
                                    line = line.replace("val= 04",
                                                        "val= (04)=(RESET_PE)")
                                elif dataVal == "84":
                                    line = line.replace("val= 84",
                                                        "val= (84)=(MSTR,RESET_PE)")                                    
                                elif dataVal == "02":
                                    line = line.replace("val= 02",
                                                        "val= (02)=(ENDDSC)")
                                elif dataVal == "82":
                                    line = line.replace("val= 82",
                                                        "val= (82)=(MSTR,ENDDSC)")                                    
                                elif dataVal == "01":
                                    line = line.replace("val= 01",
                                                        "val= (01)=(NEWSTRCT)")
                                elif dataVal == "81":
                                    line = line.replace("val= 81",
                                                        "val= (81)=(MSTR,NEWSTRCT)")                                                      
                              


                    else:    
                        line = line.replace(") -> "+regAddr,
                        ") -> %s(=%s)" % (REGMAP[mmrPage + regAddr], "1" + regAddr))                     
        if mObj == None and bWriteLine:            
            mObj = re.match(".*a2b_spiWrite\((.*), (\d*)\) -> (..) (..) (..)", line)
            if mObj:
                #print(mObj.groups())
                spiss  = mObj.group(1)
                readCnt = int(mObj.group(2))
                spiCmd = mObj.group(3)
                if SPICMDMAP[spiCmd] == "LOCAL WRITE":
                    regAddr = mObj.group(4)
                elif SPICMDMAP[spiCmd] == "REMOTE REGISTER WRITE":
                    nodeNum = mObj.group(4) 
                    regAddr = mObj.group(5)
                elif SPICMDMAP[spiCmd] == "REMOTE REGISTER READ REQUEST":
                    nodeNum = mObj.group(4) 
                    regAddr = mObj.group(5)
                    prevReadRegAddr = regAddr
                      
                #print(mObj.group(3).split())
                if regAddr in REGMAP:
                    if readCnt != 1:
                        if SPICMDMAP[spiCmd] == "LOCAL WRITE":
                            line = line.replace(") -> "+ spiCmd + " " + regAddr, 
                                   ") -> %s(=%s) <b>%s</b>(=%s) val=" % (SPICMDMAP[spiCmd], spiCmd, REGMAP[mmrPage + regAddr],regAddr))

                        else:
                            line = line.replace(") -> "+ spiCmd + " " + nodeNum + " " + regAddr, 
                                   ") -> %s(=%s) #Node=%s <b>%s</b>(=%s) val=" % (SPICMDMAP[spiCmd], spiCmd, nodeNum, REGMAP[mmrPage + regAddr],regAddr))
                        # add further detail for some data
                        #---------------------------------
                        if regAddr == "E0":
                            # A2B_MMRPAGE
                            mObj = re.match(".*val= (..)", line)
                            if mObj:
                                dataVal = mObj.group(1)
                                if dataVal == "01":
                                    mmrPage = "1"
                                else:
                                    mmrPage = ""

                        elif regAddr == "01":
                            # A2B_NODEADR
                            mObj = re.match(".*val= (..)", line)
                            if mObj:
                                dataVal = int("0x"+mObj.group(1), 0)
                                decode  = []
                                if dataVal & 0x80:
                                    decode.append("BCAST")

                                if dataVal & 0x20:
                                    decode.append("PERI")

                                decode.append("NODE=%d" % (dataVal & 0x0F))

                                line = line.replace("val= "+mObj.group(1),
                                                "val= (%s)=(%s)" \
                                                % (mObj.group(1),
                                                  ",".join(decode)))

                        elif regAddr == "09":
                            # A2B_SWCTL
                            mObj = re.match(".*val= (..)", line)
                            if mObj:
                                dataVal = int("0x"+mObj.group(1), 0)
                                decode  = []
                                if dataVal & 0x01:
                                    decode.append("ENSW=ON")
                                else:
                                    decode.append("ENSW=off")

                                if dataVal & 0x02:
                                    decode.append("DIAG=ON")
                                else:
                                    decode.append("DIAG=off")

                                if (dataVal & 0x30) == 0:
                                    decode.append("MODE=0")
                                elif (dataVal & 0x30) == 0x30:
                                    decode.append("MODE=3")
                                elif (dataVal & 0x10):
                                    decode.append("MODE=1")
                                elif (dataVal & 0x20):
                                    decode.append("MODE=2")

                                line = line.replace("val= "+mObj.group(1),
                                                "val= (%s)=(%s)" \
                                                % (mObj.group(1),
                                                  ",".join(decode)))

                        elif regAddr == "11":
                            # A2B_DATCTL
                            mObj = re.match(".*val= (..)", line)
                            if mObj:
                                dataVal = int("0x"+mObj.group(1), 0)
                                decode  = []
                                if dataVal & 0x01:
                                    decode.append("UPS=ON")
                                else:
                                    decode.append("UPS=off")

                                if dataVal & 0x02:
                                    decode.append("DNS=ON")
                                else:
                                    decode.append("DNS=off")

                                if dataVal & 0x20:
                                    decode.append("ENDSNIFF=ON")
                                else:
                                    decode.append("ENDSNIFF=off")

                                if dataVal & 0x80:
                                    decode.append("STANDBY=ON")
                                else:
                                    decode.append("STANDBY=off")

                                line = line.replace("val= "+mObj.group(1),
                                                "val= (%s)=(%s)" \
                                                % (mObj.group(1),
                                                  ",".join(decode)))

                        elif regAddr == "12":
                            # A2B_CONTROL
                            mObj = re.match(".*val= (..)", line)
                            if mObj:
                                dataVal = mObj.group(1)
                                if dataVal == "04":
                                    line = line.replace("val= 04",
                                                        "val= (04)=(RESET_PE)")
                                elif dataVal == "84":
                                    line = line.replace("val= 84",
                                                        "val= (84)=(MSTR,RESET_PE)")                                    
                                elif dataVal == "02":
                                    line = line.replace("val= 02",
                                                        "val= (02)=(ENDDSC)")
                                elif dataVal == "82":
                                    line = line.replace("val= 82",
                                                        "val= (82)=(MSTR,ENDDSC)")                                    
                                elif dataVal == "01":
                                    line = line.replace("val= 01",
                                                        "val= (01)=(NEWSTRCT)")
                                elif dataVal == "81":
                                    line = line.replace("val= 81",
                                                        "val= (81)=(MSTR,NEWSTRCT)")                                                      
                              


                    else:    
                        line = line.replace(") -> "+regAddr,
                        ") -> %s(=%s)" % (REGMAP[mmrPage + regAddr], "1" + regAddr))

        if mObj == None and g_args.shorten:
            # Shorten mode checking
            mObj = re.match(".*a2b_i2cWrite\(.*<- Success", line)
            if mObj:
                bWriteLine = False
                bSkipTS = True

        if mObj == None:
            mObj = re.match("@enduml.*", line)
            if mObj:
                hasEndUml = True

        if bWriteLine:

            if g_args.bold:
                # Look for replacements
                line = colorize( line )

            #print ">[%s]" % line
            wfp.write(line)

    rfp.close()

    if groupCount > 0:
        # This is when we have a group but NO "end" to match
        for idx in range(groupCount):
            print "line %5d: ERROR: 'group' missing 'end', appending one" % (lineCount)
            wfp.write("end")
            g_stats["errors"] += 1

    elif groupCount < 0:
        print "line %5d: ERROR: %d extra 'end' found, cannot resolve" % (lineCount, abs(groupCount))
        g_stats["errors"] += 1

    if hasEndUml == False:
        # This can happen if we have UML from a failed test that
        # had to abort before being able to properly close.
        print "line %5d: ERROR: missing '@enduml', appending one" % (lineCount)
        wfp.write("@enduml")
        g_stats["errors"] += 1

    wfp.close()

    if startDuration:
        g_stats["duration(msec)"] = endDuration - startDuration


# Do Processing
#----------------
files = []

if g_args.filename.find("*") == -1:
    files.append(g_args.filename)

else:
    files = glob.glob(g_args.filename)

filesWithErrors = 0
totalErrors = 0
for file in files:
    # reset stats
    for stat in g_stats:
        if type(g_stats[stat]) == list:
            statLen       = len(g_stats[stat])
            g_stats[stat] = []
            for idx in range(statLen):
                g_stats[stat].append(0)
        else:
            g_stats[stat] = 0

    ProcessFile ( file )

    if g_stats["errors"] != 0:
        totalErrors += g_stats["errors"]
        filesWithErrors += 1

    print "     Stats: %s" % (str(g_stats))

print "\n%d File(s) Complete (%d errors in %d/%d files)\n" % \
    (len(files), totalErrors, filesWithErrors, len(files))




